&ANALYZE-SUSPEND _VERSION-NUMBER UIB_v9r12
&ANALYZE-RESUME
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _DEFINITIONS Procedure 
/*------------------------------------------------------------------------
  File        : rtb_rpt_schema.p
  Description : Roundtable schema reports procedure library
  ----------------------------------------------------------------------*/
DEFINE VARIABLE MfirstPfilePrint AS LOGICAL NO-UNDO.

{rtb/i/rtb_rptinit.i}

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


&ANALYZE-SUSPEND _UIB-PREPROCESSOR-BLOCK 

/* ********************  Preprocessor Definitions  ******************** */

&Scoped-define PROCEDURE-TYPE Procedure
&Scoped-define DB-AWARE no



/* _UIB-PREPROCESSOR-BLOCK-END */
&ANALYZE-RESUME



/* *********************** Procedure Settings ************************ */

&ANALYZE-SUSPEND _PROCEDURE-SETTINGS
/* Settings for THIS-PROCEDURE
   Type: Procedure
   Allow: 
   Frames: 0
   Add Fields to: Neither
   Other Settings: CODE-ONLY COMPILE
 */
&ANALYZE-RESUME _END-PROCEDURE-SETTINGS

/* *************************  Create Window  ************************** */

&ANALYZE-SUSPEND _CREATE-WINDOW
/* DESIGN Window definition (used by the UIB) 
  CREATE WINDOW Procedure ASSIGN
         HEIGHT             = 27.86
         WIDTH              = 90.4.
/* END WINDOW DEFINITION */
                                                                        */
&ANALYZE-RESUME

 


&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _MAIN-BLOCK Procedure 


/* ***************************  Main Block  *************************** */

{rtb/i/rtb_s_dbparam.i}

fnRtbStartSuperProc("rtb/p/rtb_fileio.p",SESSION).

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


/* **********************  Internal Procedures  *********************** */

&IF DEFINED(EXCLUDE-rptRtbDatabaseDefinition) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbDatabaseDefinition Procedure 
PROCEDURE rptRtbDatabaseDefinition :
/*------------------------------------------------------------------------------
  Purpose:     Print database definition report
  Parameters:  INPUT Pwspace-id  - Workspace ID
               INPUT PdbaseObj   - PDBASE object name
               INPUT Ppmod       - PFILE object pmod
               INPUT Pversion    - PFILE object version
               INPUT PbriefList  - TRUE to print brief field list
               INPUT PfullDesc   - TRUE to print full field definitions
               INPUT TABLE-HANDLE PhTT - Fielspecs temp-table
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PdbaseObj  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PbriefList AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PfullDesc  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_database_definition_report (INPUT Pwspace-id,
                                      INPUT PdbaseObj,
                                      INPUT Ppmod,
                                      INPUT Pversion,
                                      INPUT PbriefList,
                                      INPUT PfullDesc).
  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbDbIntegrityCheck) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbDbIntegrityCheck Procedure 
PROCEDURE rptRtbDbIntegrityCheck :
/*------------------------------------------------------------------------------
  Purpose:     Run DB integrity check report.
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pdbase     AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PdiffOnly  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  DEFINE VARIABLE Merror    AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mfilename AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mldbname  AS CHARACTER   NO-UNDO.

  DEFINE BUFFER Brtb_dbparam FOR rtb.rtb_dbparam.
  DEFINE BUFFER Brtb_object  FOR rtb.rtb_object.

  rpt-block:
  DO:
    /*
     Make sure the required databases are connected
    */
    Merror = DYNAMIC-FUNCTION('fnRtbCheckDbConnections', Pwspace-id).
    IF Merror <> "" THEN LEAVE rpt-block.

    /*
     Get logical name for current db
    */
    FIND Brtb_dbparam
     WHERE Brtb_dbparam.wspace-id = Pwspace-id
       AND Brtb_dbparam.obj-type  = "PDBASE":U
       AND Brtb_dbparam.object    = Pdbase     NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_dbparam THEN DO:
      Merror = SUBSTITUTE("DB Param record for '&1' not available.",Pdbase).
      LEAVE rpt-block.
    END.
    Mldbname = DYNAMIC-FUNCTION('fnRtbGetLogicalDbName':U,ROWID(Brtb_dbparam)).

    IF NOT CONNECTED(Mldbname) THEN DO:
      Merror = SUBSTITUTE("Database '&1' is not connected.",Mldbname).
      LEAVE rpt-block.
    END.
    /*
     Make sure current database has the DICTDB alias
    */
    DELETE ALIAS DICTDB.
    CREATE ALIAS DICTDB FOR DATABASE VALUE(Mldbname).

    FIND Brtb_object
      WHERE Brtb_object.wspace-id = Pwspace-id
        AND Brtb_object.obj-type  = "PDBASE":U
        AND Brtb_object.object    = Pdbase     NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_object THEN DO:
      Merror = SUBSTITUTE("Object record for PDBASE '&1' not available.",Pdbase).
      LEAVE rpt-block.
    END.

    OUTPUT STREAM Sout CLOSE.

    /*
     Get source filename for output
    */
    Mfilename = DYNAMIC-FUNCTION('fnRtbGetFilespecPathnames':U,
                                 INPUT TABLE-HANDLE PhTT,
                                 INPUT "Source":U,
                                 INPUT CHR(1),
                                 INPUT NO).
    /*
     Run the report - Compiled at run time with include above
    */
    RUN rtb/p/rtb_dbinckrpt.p (INPUT ROWID(Brtb_object),
                               INPUT Mfilename,
                               INPUT PdiffOnly).
    Merror = RETURN-VALUE.

    
  END.

  /*
   Restore (client) or remove (remote partition) DB connections
   as required.
  */
  IF SESSION:REMOTE THEN
    DYNAMIC-FUNCTION('fnRtbDisconnectWspaceDbs':U,Pwspace-id).
  ELSE
    DYNAMIC-FUNCTION('fnRtbCheckDbConnections':U,Pwspace-id).
  
  
  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbPdbaseObject) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbPdbaseObject Procedure 
PROCEDURE rptRtbPdbaseObject :
/*------------------------------------------------------------------------------
  Purpose:     Print PDBASE object report
  Parameters:  INPUT Pwspace-id  - Workspace ID
               INPUT PdbaseObj   - PDBASE object name
               INPUT Ppmod       - PDBASE product module
               INPUT Pversion    - PDBASE object version
               INPUT TABLE-HANDLE PhTT - Filespecs temp-table
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PdbaseObj  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_pdbase_object_report (INPUT Pwspace-id,
                                INPUT PdbaseObj,
                                INPUT Ppmod,
                                INPUT Pversion).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbPfieldObject) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbPfieldObject Procedure 
PROCEDURE rptRtbPfieldObject :
/*------------------------------------------------------------------------------
  Purpose:     Print PFIELD object report.
  Parameters:  INPUT Pwspace-id   - Workspace ID
               INPUT PfieldObj    - PFIELD object name
               INPUT Ppmod        - PFIELD object product module
               INPUT Pversion     - PFIELD object version
               INPUT TEMP-TABLE PhTT - Filespecs temp-table
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PfieldObj  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_pfield_object_report (INPUT Pwspace-id,
                                INPUT PfieldObj,
                                INPUT Ppmod,
                                INPUT Pversion).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbPfileObject) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbPfileObject Procedure 
PROCEDURE rptRtbPfileObject :
/*------------------------------------------------------------------------------
  Purpose:     Print PFILE object report
  Parameters:  INPUT Pwspace-id - Workspace ID
               INPUT PfileObj   - PFILE object name
               INPUT Ppmod      - Product module
               INPUT Pversion   - PFILE object version
               INPUT TABLE-HANDLE PhTT - Filespecs temp-table.
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PfileObj   AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_pfile_object_report (INPUT Pwspace-id,
                               INPUT PfileObj,
                               INPUT Ppmod,
                               INPUT Pversion).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbTableDefinition) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbTableDefinition Procedure 
PROCEDURE rptRtbTableDefinition :
/*------------------------------------------------------------------------------
  Purpose:     Print table definition report
  Parameters:  INPUT Pwspace-id  - Workspace ID
               INPUT PfileObj    - PFILE object name
               INPUT Ppmod       - PFILE object pmod
               INPUT Pversion    - PFILE object version
               INPUT Pbrieflist  - TRUE to print brief field list
               INPUT Pfulldesc   - TRUE to print field definitions
               INPUT TABLE-HANDLE PhTT - Filespecs temp-table
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PfileObj   AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PbriefList AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PfullDesc  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_table_definition_report (INPUT Pwspace-id,
                                   INPUT PfileObj,
                                   INPUT Ppmod,
                                   INPUT Pversion,
                                   INPUT PbriefList,
                                   INPUT PfullDesc).
  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.
END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbUnappliedChanges) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbUnappliedChanges Procedure 
PROCEDURE rptRtbUnappliedChanges :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id    AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PdbChanges    AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PtableChanges AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PfieldChanges AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_unapplied_changes_report (INPUT Pwspace-id,
                                    INPUT PdbChanges,
                                    INPUT PtableChanges,
                                    INPUT PfieldChanges).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbUnappliedDbChanges) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbUnappliedDbChanges Procedure 
PROCEDURE rptRtbUnappliedDbChanges :
/*------------------------------------------------------------------------------
  Purpose:     Print unapplied database changes report.
  Parameters:  INPUT Pwspace-id  - Workspace ID
               INPUT PdbaseObj   - PDBASE object name.
               INPUT TABLE-HANDLE PhTT - Fielspecs temp-table.
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PdbaseObj  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_unapplied_dbase_changes (INPUT Pwspace-id,
                                   INPUT PdbaseObj).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbUnappliedFieldChanges) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbUnappliedFieldChanges Procedure 
PROCEDURE rptRtbUnappliedFieldChanges :
/*------------------------------------------------------------------------------
  Purpose:     Print unapplied database changes report.
  Parameters:  INPUT Pwspace-id  - Workspace ID
               INPUT PfieldObj    - PFILE object name.
               INPUT TABLE-HANDLE PhTT - Fielspecs temp-table.
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PfieldObj  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_unapplied_field_changes (INPUT Pwspace-id,
                                   INPUT PfieldObj).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbUnappliedFileChanges) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbUnappliedFileChanges Procedure 
PROCEDURE rptRtbUnappliedFileChanges :
/*------------------------------------------------------------------------------
  Purpose:     Print unapplied database changes report.
  Parameters:  INPUT Pwspace-id  - Workspace ID
               INPUT PfileObj    - PFILE object name.
               INPUT TABLE-HANDLE PhTT - Fielspecs temp-table.
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PfileObj   AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_unapplied_file_changes (INPUT Pwspace-id,
                                  INPUT PfileObj).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_check_file_field_changes) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_check_file_field_changes Procedure 
PROCEDURE rtb_check_file_field_changes PRIVATE :
/*------------------------------------------------------------------------------
  Purpose:     Check specified file for field changes
  Parameters:  OUTPUT Pmatch  - TRUE if no field changes
               BUFFER Brtb_file  FOR rtb.rtb_file
               BUFFER B2rtb_file FOR rtb.rtb_file
  Notes:       Split out from rtb_unapplied_file_changes because of section
               editor size limit.
------------------------------------------------------------------------------*/
  DEFINE OUTPUT PARAMETER Pmatch AS LOGICAL    NO-UNDO.
  DEFINE PARAMETER BUFFER Brtb_file  FOR rtb.rtb_file.  /* "new" buffer */
  DEFINE PARAMETER BUFFER B2rtb_file FOR rtb.rtb_file.  /* committed buffer */

  DEFINE BUFFER Brtb_field    FOR rtb.rtb_field.
  DEFINE BUFFER B2rtb_field   FOR rtb.rtb_field.
  DEFINE BUFFER Brtb_fldtrig  FOR rtb.rtb_fldtrig.
  DEFINE BUFFER B2rtb_fldtrig FOR rtb.rtb_fldtrig.

  Pmatch = YES.

  FOR EACH B2rtb_field OF B2rtb_file NO-LOCK: /* current image */
    FIND FIRST Brtb_field OF Brtb_file /* edit image */
      WHERE Brtb_field.field-id  = B2rtb_field.field-id NO-LOCK NO-ERROR.

    IF NOT AVAILABLE Brtb_field
       OR NOT(Brtb_field.local-field-name = B2rtb_field.local-field-name
              AND Brtb_field.rtb_order        = B2rtb_field.rtb_order
              AND Brtb_field.flddef-obj-type  = B2rtb_field.flddef-obj-type
              AND Brtb_field.flddef-object    = B2rtb_field.flddef-object
              AND Brtb_field.rtb_mandatory    = B2rtb_field.rtb_mandatory) THEN DO:
      IF MfirstPfilePrint THEN DO:
        PUT STREAM Sout SKIP(1)
                        SUBSTITUTE("Unapplied Changes in PFILE &1 (old, new):",B2rtb_file.object)
                        FORMAT "x(70)"
                        SKIP(1).
        MfirstPfilePrint = NO.
      END.
      IF NOT AVAILABLE Brtb_field THEN 
        PUT STREAM Sout "  Field: " B2rtb_field.local-field-name + " DELETED " 
                        FORMAT "x(50)" SKIP.
      ELSE DO:
        IF Brtb_field.local-field-name <> B2rtb_field.local-field-name THEN
          PUT STREAM Sout "  Field: " B2rtb_field.local-field-name + " , "
                     + Brtb_field.local-field-name FORMAT "x(68)" SKIP.
        ELSE
          PUT STREAM Sout "  Field: " B2rtb_field.local-field-name SKIP.

        IF Brtb_field.rtb_order <> B2rtb_field.rtb_order THEN 
          PUT STREAM Sout "  Order: " B2rtb_field.rtb_order FORMAT ">>>>9" " , "
                                      Brtb_field.rtb_order FORMAT ">>>>9" SKIP.

        IF Brtb_field.rtb_mandatory <> B2rtb_field.rtb_mandatory THEN 
          PUT STREAM Sout UNFORMATTED 
                          "   Mand: " B2rtb_field.rtb_mandatory " , "
                                      Brtb_field.rtb_mandatory SKIP.

        IF Brtb_field.flddef-object <> B2rtb_field.flddef-object THEN 
          PUT STREAM Sout UNFORMATTED
                          " FldObj: " B2rtb_field.flddef-object  " , "
                          Brtb_field.flddef-object SKIP.
      END. /* else do */
    Pmatch = NO.
    END. /* not available brtb_field... */

    IF AVAILABLE Brtb_field AND Pmatch THEN 
      /*
       Look for field trigger changes
      */
      FOR EACH B2rtb_fldtrig OF B2rtb_field NO-LOCK: /* current image */
        FIND FIRST Brtb_fldtrig OF Brtb_field /* edit image */
          WHERE Brtb_fldtrig.rtb_event  = B2rtb_fldtrig.rtb_event NO-LOCK NO-ERROR.
        IF NOT AVAILABLE Brtb_fldtrig
           OR NOT(Brtb_fldtrig.rtb_override  = B2rtb_fldtrig.rtb_override
                  AND DYNAMIC-FUNCTION('fnRtbCompareStrings':U,Brtb_fldtrig.rtb_proc-name,B2rtb_fldtrig.rtb_proc-name)
                  AND Brtb_fldtrig.rtb_trig-crc  = B2rtb_fldtrig.rtb_trig-crc) THEN DO:
          IF MfirstPfilePrint THEN DO:
            PUT STREAM Sout SKIP(1)
                            SUBSTITUTE("Unapplied Changes in PFILE &1 (old, new):",B2rtb_file.object)
                            FORMAT "x(70)"
                            SKIP(1).
            MfirstPfilePrint = NO.
          END.
          IF NOT AVAILABLE Brtb_fldtrig THEN
            PUT STREAM Sout "  Field Trigger: " B2rtb_fldtrig.rtb_event + " DELETED " 
                            FORMAT "x(50)" SKIP.
          ELSE DO:
            PUT STREAM Sout UNFORMATTED "  Field Trigger: " B2rtb_fldtrig.rtb_event SKIP.
            IF Brtb_fldtrig.rtb_override <> B2rtb_fldtrig.rtb_override THEN
              PUT STREAM Sout UNFORMATTED "   Override: " B2rtb_fldtrig.rtb_override " , "
                              Brtb_fldtrig.rtb_override SKIP.
            IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,Brtb_fldtrig.rtb_proc-name, B2rtb_fldtrig.rtb_proc-name) THEN 
              PUT STREAM Sout "  Proc Name: " B2rtb_fldtrig.rtb_proc-name FORMAT "x(32)" " , "
                              Brtb_fldtrig.rtb_proc-name FORMAT "x(32)" SKIP.
            IF Brtb_fldtrig.rtb_trig-crc <> B2rtb_fldtrig.rtb_trig-crc THEN 
              PUT STREAM Sout "        CRC: " B2rtb_fldtrig.rtb_trig-crc FORMAT ">>>>>9" " , "
                              Brtb_fldtrig.rtb_trig-crc FORMAT ">>>>>9" SKIP.
          END. /* else do */
        Pmatch = NO.
        END. /* not available brtb_fldtrig... */
      END. /* each b2rtb_fldtrig ... */

    IF AVAILABLE Brtb_field AND Pmatch THEN
      /*
       Look for new field triggers
      */
      FOR EACH Brtb_fldtrig OF Brtb_field NO-LOCK: /* edit image */
        IF NOT CAN-FIND(FIRST B2rtb_fldtrig OF B2rtb_field
                        WHERE B2rtb_fldtrig.rtb_event = Brtb_fldtrig.rtb_event) THEN DO:
          IF MfirstPfilePrint THEN DO:
            PUT STREAM Sout SKIP(1)
                            SUBSTITUTE("Unapplied Changes in PFILE &1 (old, new):",B2rtb_file.object)
                            FORMAT "x(70)"
                            SKIP(1).
            MfirstPfilePrint = NO.
          END.
          PUT STREAM Sout SKIP "  " Brtb_field.flddef-object " Field Trigger: " Brtb_fldtrig.rtb_event + " ADDED " 
                          FORMAT "x(50)" SKIP.
        Pmatch = NO.
        END. /* not can-find... */
      END. /* each brtb_fldtrig ... */
  END. /* each b2rtb_field ... */

  /*
   Look for new fields
  */
  IF Pmatch THEN  
    FOR EACH Brtb_field OF Brtb_file NO-LOCK:
      IF NOT CAN-FIND(FIRST B2rtb_field OF B2rtb_file WHERE
                            B2rtb_field.field-id = Brtb_field.field-id) THEN DO:
        IF MfirstPfilePrint THEN DO:
          PUT STREAM Sout SKIP(1)
                          SUBSTITUTE("Unapplied Changes in PFILE &1 (old, new):",B2rtb_file.object)
                          FORMAT "x(70)"
                          SKIP(1).
          MfirstPfilePrint = NO.
        END.
        PUT STREAM Sout
          "  Field: " Brtb_field.local-field-name + " ADDED " FORMAT "x(50)" SKIP.
        Pmatch = NO.
        LEAVE.
      END.
    END. /* of EACH Brtb_field ... */

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_check_file_index_changes) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_check_file_index_changes Procedure 
PROCEDURE rtb_check_file_index_changes PRIVATE :
/*------------------------------------------------------------------------------
  Purpose:     Check specified file for index changes
  Parameters:  OUTPUT Pmatch  - TRUE if no index changes
               BUFFER Brtb_file  FOR rtb.rtb_file
               BUFFER B2rtb_file FOR rtb.rtb_file
  Notes:       Split out from rtb_unapplied_file_changes because of section
               editor size limit.
------------------------------------------------------------------------------*/
  DEFINE OUTPUT PARAMETER Pmatch AS LOGICAL    NO-UNDO.
  DEFINE PARAMETER BUFFER Brtb_file  FOR rtb.rtb_file. /* "new" buffer */
  DEFINE PARAMETER BUFFER B2rtb_file FOR rtb.rtb_file. /* committed buffer */

  DEFINE VARIABLE Mtemp         AS CHARACTER  NO-UNDO.
  DEFINE VARIABLE Mi            AS INTEGER    NO-UNDO.
  DEFINE VARIABLE MidxPrinted   AS LOGICAL    NO-UNDO.
  DEFINE VARIABLE MidxFieldDiff AS LOGICAL    NO-UNDO.

  DEFINE BUFFER Brtb_field   FOR rtb.rtb_field.
  DEFINE BUFFER Brtb_idxfld  FOR rtb.rtb_idxfld.
  DEFINE BUFFER B2rtb_idxfld FOR rtb.rtb_idxfld.
  DEFINE BUFFER Brtb_index   FOR rtb.rtb_index.
  DEFINE BUFFER B2rtb_index  FOR rtb.rtb_index.

  Pmatch = YES.

  FOR EACH B2rtb_index OF B2rtb_file NO-LOCK:

    MidxPrinted = NO.

    FIND FIRST Brtb_index OF Brtb_file
         WHERE Brtb_index.index-id  = B2rtb_index.index-id NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_index OR
       NOT(Brtb_index.rtb_index-name         = B2rtb_index.rtb_index-name 
           AND DYNAMIC-FUNCTION('fnRtbCompareStrings':U,Brtb_index.rtb_desc,B2rtb_index.rtb_desc) AND
           Brtb_index.rtb_unique             = B2rtb_index.rtb_unique AND
           Brtb_index.rtb_active             = B2rtb_index.rtb_active AND
           Brtb_index.rtb_wordidx            = B2rtb_index.rtb_wordidx AND
           Brtb_index.rtb_index-attributes1 <> B2rtb_index.rtb_index-attributes1) THEN DO:

      IF MfirstPfilePrint THEN DO:
        PUT STREAM Sout SKIP(1)
                        SUBSTITUTE("Unapplied Changes in PFILE &1 (old, new):",B2rtb_file.object)
                        FORMAT "x(70)"
                        SKIP(1).
        MfirstPfilePrint = NO.
      END.

      IF NOT AVAILABLE Brtb_index THEN DO:
        MidxPrinted = YES.
        PUT STREAM Sout
          SKIP(1)
          "  Index: " B2rtb_index.rtb_index-name + " DELETED " FORMAT "x(50)" SKIP.
          Pmatch = NO.
          NEXT.
      END.
      ELSE DO:
        MidxPrinted = YES.
        IF Brtb_index.rtb_index-name <> B2rtb_index.rtb_index-name THEN
          PUT STREAM Sout
            SKIP(1)
            "  Index: " B2rtb_index.rtb_index-name + " , "
            +  Brtb_index.rtb_index-name FORMAT "x(68)" SKIP.
        ELSE 
          PUT STREAM Sout "  Index: " B2rtb_index.rtb_index-name SKIP.

        IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,Brtb_index.rtb_desc,
                                B2rtb_index.rtb_desc) THEN DO:
          DO Mi = 0 TO 3:
            Mtemp = SUBSTRING(B2rtb_index.rtb_desc,Mi * 63 + 1,63).
            IF Mi = 0 AND Mtemp = ? THEN Mtemp = "?".
            IF (Mtemp = ? OR (TRIM(Mtemp) = "" AND Mi > 0)) THEN NEXT.
            PUT STREAM Sout
              (IF Mi = 0 THEN "   Desc: " ELSE "         ") + Mtemp FORMAT "x(80)" SKIP.
          END.

          DO Mi = 0 TO 3:
            Mtemp = SUBSTRING(Brtb_index.rtb_desc,Mi * 63 + 1,63).
            IF Mi = 0 AND Mtemp = ? THEN Mtemp = "?".
            IF (Mtemp = ? OR (TRIM(Mtemp) = "" AND Mi > 0)) THEN NEXT.
            PUT STREAM Sout
              (IF Mi = 0 THEN "   ,   : " ELSE "         ") + Mtemp FORMAT "x(80)" SKIP.
          END.
        END. /* of rtb_desc */
               
        IF Brtb_index.rtb_unique <> B2rtb_index.rtb_unique THEN
          PUT STREAM Sout " Unique: " B2rtb_index.rtb_unique " , "
                          Brtb_index.rtb_unique SKIP.

        IF Brtb_index.rtb_active <> B2rtb_index.rtb_active THEN
          PUT STREAM Sout " Active: " B2rtb_index.rtb_active " , "
                                 Brtb_index.rtb_active SKIP.

        IF Brtb_index.rtb_wordidx <> B2rtb_index.rtb_wordidx THEN
          PUT STREAM Sout " Word Index: " ( B2rtb_index.rtb_wordidx = 1 ) " , "
                            ( Brtb_index.rtb_wordidx = 1 ) SKIP.

        IF Brtb_index.rtb_index-attributes1 <> B2rtb_index.rtb_index-attributes1 THEN
          PUT STREAM Sout " Local Index: " B2rtb_index.rtb_index-attributes1 " , "
                              Brtb_index.rtb_index-attributes1 SKIP.
      END.
      Pmatch = NO.
      LEAVE.  
    END. /* not available brtb_index */

    MidxFieldDiff = NO.

    FOR EACH B2rtb_idxfld OF B2rtb_index NO-LOCK:
      FIND FIRST Brtb_idxfld OF Brtb_index
        WHERE Brtb_idxfld.rtb_index-seq  = B2rtb_idxfld.rtb_index-seq
          AND Brtb_idxfld.field-id       = B2rtb_idxfld.field-id
          AND Brtb_idxfld.rtb_ascending  = B2rtb_idxfld.rtb_ascending
          AND Brtb_idxfld.rtb_abbreviate = B2rtb_idxfld.rtb_abbreviate
          AND Brtb_idxfld.rtb_unsorted   = B2rtb_idxfld.rtb_unsorted NO-LOCK NO-ERROR.
      IF NOT AVAILABLE Brtb_idxfld THEN DO:
        ASSIGN MidxFieldDiff = YES
               Pmatch        = NO.
        LEAVE.
      END.
    END. /* of EACH Brtb_idxfld ... */

    IF Pmatch THEN
      FOR EACH Brtb_idxfld OF Brtb_index NO-LOCK:
        IF NOT CAN-FIND(FIRST B2rtb_idxfld OF B2rtb_index
                        WHERE B2rtb_idxfld.rtb_index-seq = Brtb_idxfld.rtb_index-seq) THEN DO:
          Pmatch = NO.
          LEAVE.
        END.
      END. /* of EACH Brtb_idxfld */

    IF MidxFieldDiff THEN DO:

      IF MfirstPfilePrint THEN DO:
        PUT STREAM Sout SKIP(1)
                        SUBSTITUTE("Unapplied Changes in PFILE &1 (old, new):",B2rtb_file.object)
                        FORMAT "x(70)"
                        SKIP(1).
        MfirstPfilePrint = NO.
      END.

      IF NOT MidxPrinted THEN 
        PUT STREAM Sout
          SKIP(1)
          "  Index: " Brtb_index.rtb_index-name 
          +  " -- field changes (old,new) " FORMAT "x(60)" SKIP.
        ELSE
          PUT STREAM Sout " Index field changes (old,new) " SKIP.

      MidxPrinted = YES.
      DO Mi = 1 TO 16:  /* show indicies side by side */

        FIND B2rtb_idxfld OF B2rtb_index 
          WHERE B2rtb_idxfld.rtb_index-seq = Mi NO-LOCK NO-ERROR.

        IF AVAILABLE B2rtb_idxfld THEN 
          FIND FIRST Brtb_field OF B2rtb_file 
            WHERE Brtb_field.field-id = B2rtb_idxfld.field-id NO-LOCK.

        FIND Brtb_idxfld OF Brtb_index 
          WHERE Brtb_idxfld.rtb_index-seq = Mi NO-LOCK NO-ERROR.

        IF AVAILABLE Brtb_idxfld THEN 
          FIND FIRST Brtb_field OF Brtb_file 
             WHERE Brtb_field.field-id = Brtb_idxfld.field-id NO-LOCK.

        IF NOT AVAILABLE Brtb_idxfld 
           AND NOT AVAILABLE B2rtb_idxfld THEN LEAVE.

        IF AVAILABLE B2rtb_idxfld THEN DO:
          PUT STREAM Sout
            "   " 
            (IF B2rtb_idxfld.rtb_ascending THEN "+" ELSE "-") FORMAT "x"
            (IF B2rtb_idxfld.rtb_abbreviate THEN "Ab" ELSE "  ") FORMAT "xx"
            " " Brtb_field.local-field-name. 
        END. /* available rtb_idxfld */

        IF AVAILABLE Brtb_idxfld THEN DO:
          PUT STREAM Sout ", " AT 40 
            (IF Brtb_idxfld.rtb_ascending THEN "+" ELSE "-") FORMAT "x"
            (IF Brtb_idxfld.rtb_abbreviate THEN "Ab" ELSE "  ") FORMAT "xx"
             " " Brtb_field.local-field-name.
        END. /* available Brtb_idxfld */

        PUT STREAM Sout SKIP.

      END. /* of Mi = 1 to 16 */
    END. /* not MidxFieldDiff */
  END. /* of EACH B2rtb_index */

  IF Pmatch THEN
    /*
     Look for new indicies
    */
    FOR EACH Brtb_index OF Brtb_file NO-LOCK:
      IF NOT CAN-FIND(FIRST B2rtb_index OF B2rtb_file
                      WHERE B2rtb_index.index-id = Brtb_index.index-id) THEN DO:
        PUT STREAM Sout
          SKIP(1)
          "  Index: " Brtb_index.rtb_index-name + " ADDED " FORMAT "x(50)" SKIP.

        ASSIGN MidxFieldDiff = YES
               Pmatch        = NO.
        LEAVE.
      END.
    END. /* of EACH Brtb_index */

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_database_definition_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_database_definition_report Procedure 
PROCEDURE rtb_database_definition_report :
/*------------------------------------------------------------------------------
  Purpose:     Print database definition report
  Parameters:  INPUT Pwspace-id  - Workspace ID
               INPUT PdbaseObj   - PDBASE object name
               INPUT PbriefList  - TRUE to print brief field list
               INPUT PfullDesc   - TRUE to print full field definitions
  Notes:       Formerly rtb_1101.w
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PdbaseObj  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PbriefList AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PfullDesc  AS LOGICAL     NO-UNDO.

  DEFINE VARIABLE Merror          AS CHARACTER NO-UNDO.
  DEFINE VARIABLE Mline-out       AS CHARACTER NO-UNDO.
  DEFINE VARIABLE Mevent          AS INTEGER   NO-UNDO.
  DEFINE VARIABLE Mi              AS INTEGER   NO-UNDO.

  DEFINE BUFFER Brtb_dbase    FOR rtb.rtb_dbase.
  DEFINE BUFFER Brtb_dbfile   FOR rtb.rtb_dbfile.
  DEFINE BUFFER Brtb_hist     FOR rtb.rtb_hist.
  DEFINE BUFFER B2rtb_hist    FOR rtb.rtb_hist.
  DEFINE BUFFER Brtb_object   FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_ver      FOR rtb.rtb_ver.
  DEFINE BUFFER Brtb_dbalias  FOR rtb.rtb_dbalias.
  DEFINE BUFFER Brtb_sequence FOR rtb.rtb_sequence.

  rpt-block:
  DO:
     FIND LAST Brtb_hist
       WHERE Brtb_hist.wspace-id = Pwspace-id
         AND Brtb_hist.pmod      = Ppmod
         AND Brtb_hist.obj-type  = "PDBASE":U
         AND Brtb_hist.object    = PdbaseObj
         AND Brtb_hist.version   = Pversion NO-LOCK NO-ERROR.

     IF NOT AVAILABLE Brtb_hist THEN DO:
       Merror = SUBSTITUTE("PDBASE '&1' version &2 has never been assigned to workspace '&3'.",PdbaseObj,Pversion,Pwspace-id).
       LEAVE rpt-block.
     END.

    FIND Brtb_ver OF Brtb_hist NO-LOCK NO-ERROR.
    
    IF NOT AVAILABLE(Brtb_ver) THEN DO:
      Merror = SUBSTITUTE("Version record for PDBASE '&1' version &2 not available.",PdbaseObj,Pversion).
      LEAVE rpt-block.
    END.

    FIND Brtb_dbase OF Brtb_ver NO-LOCK NO-ERROR.
    
    IF NOT AVAILABLE(Brtb_dbase) THEN DO:
      Merror = SUBSTITUTE("Dbase record for PDBASE '&1' not available.",PdbaseObj).
      LEAVE rpt-block.
    END.

    FIND Brtb_object 
      WHERE Brtb_object.wspace-id = Brtb_hist.wspace-id
        AND Brtb_object.obj-type  = Brtb_hist.obj-type
        AND Brtb_object.object    = Brtb_hist.object
        NO-LOCK NO-ERROR.

    IF AVAILABLE Brtb_object
       AND Brtb_object.version = Brtb_hist.version 
       AND Brtb_object.pmod    = Brtb_hist.pmod THEN
      Mevent = 99999999.
    ELSE
      Mevent = Brtb_hist.last-event-count.

    DO ON ERROR UNDO, LEAVE:

      {rtb/i/rtb_rpthdr.i 
          &title="""Database Definition Report"""
          &criteria="""Workspace:"" Pwspace-id FORMAT ""x(16)"" SKIP
                     ""       DB:"" PdbaseObj FORMAT ""x(32)""  SKIP
                     ""  Version:"" Brtb_ver.VERSION            SKIP"
          &center-criteria=NO
          &STREAM=Sout}

      DO Mi = 1 TO 16:
        Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_ver.text-desc[Mi],60).
        DO WHILE Mline-out <> ?:
          PUT STREAM Sout UNFORMATTED Mline-out SKIP.
          Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
        END.
      END.  /* do mi = 1 to 16 */

      
      FOR EACH Brtb_dbalias OF Brtb_dbase NO-LOCK WITH FRAME alias-frame:
        DISPLAY STREAM Sout
          Brtb_dbalias.alias-name
          WITH FRAME alias-frame STREAM-IO.
      END.

      IF CAN-FIND(FIRST Brtb_sequence OF Brtb_dbase) THEN DO:

        PUT STREAM Sout
          SKIP(1)
          "Sequences" SKIP
          "---------".
          
        FOR EACH Brtb_sequence OF Brtb_dbase NO-LOCK WITH FRAME sequence-frame:
          DISPLAY STREAM Sout
            Brtb_sequence.rtb_seq-name LABEL "Sequence Name"
            Brtb_sequence.rtb_cycle-ok
            Brtb_sequence.rtb_seq-incr
            Brtb_sequence.rtb_seq-init
            Brtb_sequence.rtb_seq-min
            Brtb_sequence.rtb_seq-max
            WITH FRAME sequences-frame 1 COLUMN SIDE-LABELS STREAM-IO.
        END.
      END.
      
      FOR EACH Brtb_dbfile OF Brtb_dbase NO-LOCK WITH FRAME file-frame:

        FIND FIRST B2rtb_hist
          WHERE B2rtb_hist.wspace-id     = Pwspace-id
            AND B2rtb_hist.obj-type      = Brtb_dbfile.file-obj-type
            AND B2rtb_hist.object        = Brtb_dbfile.file-object
            AND B2rtb_hist.event-counter <= Mevent
            USE-INDEX rtb_hist02
            NO-LOCK.

        /*
         Prnt File releated Infomation
        */
        FORM HEADER
          "  File Name:" AT  1 Brtb_dbfile.local-file-name       AT 14 SKIP
          "File Object:" AT  1 B2rtb_hist.object FORMAT "x(32)"  AT 14 SKIP
          "    Version:" AT  1 B2rtb_hist.version                AT 14 SKIP(1)
        WITH FRAME file-header-frame
          WIDTH 120
          NO-BOX
          SIDE-LABELS
          NO-ATTR-SPACE
          PAGE-TOP.

        PAGE STREAM Sout.

        VIEW STREAM Sout FRAME file-header-frame.

        RUN rtb_print_file_detail (ROWID(B2rtb_hist), Mevent, PbriefList, PfullDesc).
                               
        IF RETURN-VALUE <> "" THEN DO:
          Merror = RETURN-VALUE.
          LEAVE rpt-block.
        END.

      END. /* of EACH Brtb_dbfile */

    END. /* DO: */

    {rtb/i/rtb_rptftr.i &STREAM=Sout}

  END. /* rpt-block */

  RETURN Merror.  

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_find_last) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_find_last Procedure 
PROCEDURE rtb_find_last PRIVATE :
/*------------------------------------------------------------------------------
  Purpose:     Find File or Field def. record for specified PFILE or PFIELD.
  Parameters:  INPUT  Pwspace-id  - Workspace ID
               INPUT  Pobj-type   - PFILE or PFIELD
               INPUT  Pobject     - Object name
               INPUT  Pevent      - History event of object
               OUPTUT Prowid      - ROWID of rtb_file or rtb_flddef
  Notes:       Formerly rtb_last.p
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Pwspace-id AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobj-type  AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pobject    AS CHARACTER NO-UNDO.
  DEFINE INPUT  PARAMETER Pevent     AS INTEGER   NO-UNDO.
  DEFINE OUTPUT PARAMETER Prowid     AS ROWID     NO-UNDO.

  DEFINE VARIABLE Merror AS CHARACTER   NO-UNDO INITIAL "".

  DEFINE BUFFER Brtb_file   FOR rtb.rtb_file.
  DEFINE BUFFER Brtb_flddef FOR rtb.rtb_flddef.
  DEFINE BUFFER Brtb_hist   FOR rtb.rtb_hist.
  DEFINE BUFFER Brtb_object FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_ver    FOR rtb.rtb_ver.

  $LAST-ROWID-BLOCK:
  DO:
    IF Pobj-type = "PFIELD" THEN DO:
      IF Pevent = 0 THEN DO:
        /*
         Look for current flddef record
        */
        FIND Brtb_object 
          WHERE Brtb_object.wspace-id = Pwspace-id
            AND Brtb_object.obj-type  = Pobj-type
            AND Brtb_object.object    = Pobject
            NO-LOCK NO-ERROR.

        IF AVAILABLE Brtb_object THEN
          FIND Brtb_ver OF Brtb_object NO-LOCK NO-ERROR.

        IF AVAILABLE Brtb_ver THEN
          FIND Brtb_flddef OF Brtb_ver NO-LOCK NO-ERROR.
      END.
      ELSE DO:
        /*
         Look for flddef record active at specified event
       */
        FIND FIRST Brtb_hist 
          WHERE Brtb_hist.wspace-id     = Pwspace-id
            AND Brtb_hist.obj-type      = Pobj-type
            AND Brtb_hist.object        = Pobject
            AND Brtb_hist.event-counter <= Pevent
            NO-LOCK NO-ERROR.

        IF AVAILABLE Brtb_hist THEN
          FIND Brtb_flddef OF Brtb_hist NO-LOCK NO-ERROR.
      END.

      IF AVAILABLE Brtb_flddef THEN
        Prowid = ROWID(Brtb_flddef).
      ELSE DO:
         ASSIGN Merror = SUBSTITUTE("Field Def not available for PFIELD '&1'.",Pobject)
                Prowid = ?.
         LEAVE $LAST-ROWID-BLOCK.
      END.
    END. /* of PFIELD */

    ELSE IF Pobj-type = "PFILE" THEN DO:
       IF Pevent = 0 THEN DO:
         /*
          Look for current flddef record
         */
         FIND Brtb_object 
            WHERE Brtb_object.wspace-id   = Pwspace-id
              AND Brtb_object.obj-type    = Pobj-type
              AND Brtb_object.object      = Pobject
              NO-LOCK NO-ERROR.

         IF AVAILABLE Brtb_object THEN
           FIND Brtb_ver OF Brtb_object NO-LOCK NO-ERROR.

         IF AVAILABLE Brtb_ver THEN
           FIND Brtb_file OF Brtb_ver NO-LOCK NO-ERROR.

       END.
       ELSE 
          DO: /* Look for file record active at specified event */
          FIND FIRST Brtb_hist 
               WHERE Brtb_hist.wspace-id = Pwspace-id
                 AND Brtb_hist.obj-type  = Pobj-type
                 AND Brtb_hist.object    = Pobject
                 AND Brtb_hist.event-counter <= Pevent
                 NO-LOCK NO-ERROR.

          IF AVAILABLE Brtb_hist THEN
             FIND Brtb_file OF Brtb_hist NO-LOCK NO-ERROR.
          END.

       Prowid = IF AVAILABLE Brtb_file THEN
                  ROWID(Brtb_file)
                ELSE
                  ?.

       END.
  END. /* $LAST-ROWID-BLOCK */

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_pdbase_object_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_pdbase_object_report Procedure 
PROCEDURE rtb_pdbase_object_report :
/*------------------------------------------------------------------------------
  Purpose:     Print PDBASE object report
  Parameters:  INPUT Pwspace-id - Workspace ID
               INPUT PdbaseObj  - PDBASE object name
               INPUT Ppmod      - PDBASE product module
               INPUT Pversion   - PDBASE object version
  Notes:       Formerly rtb_1100.p
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PdbaseObj  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.

  DEFINE VARIABLE Merror    AS CHARACTER NO-UNDO.
  DEFINE VARIABLE Mline-out AS CHARACTER NO-UNDO.
  DEFINE VARIABLE Mi        AS INTEGER   NO-UNDO.

  DEFINE BUFFER Brtb_dbase    FOR rtb.rtb_dbase.
  DEFINE BUFFER Brtb_dbfile   FOR rtb.rtb_dbfile.
  DEFINE BUFFER Brtb_hist     FOR rtb.rtb_hist.
  DEFINE BUFFER Brtb_ver      FOR rtb.rtb_ver.
  DEFINE BUFFER Brtb_dbalias  FOR rtb.rtb_dbalias.
  DEFINE BUFFER Brtb_sequence FOR rtb.rtb_sequence.

  $RPT-BLOCK:
  DO:

    FIND LAST Brtb_hist
      WHERE Brtb_hist.wspace-id = Pwspace-id
        AND Brtb_hist.pmod      = Ppmod
        AND Brtb_hist.obj-type  = "PDBASE":U
        AND Brtb_hist.object    = PdbaseObj
        AND Brtb_hist.version   = Pversion NO-LOCK NO-ERROR.

    IF NOT AVAILABLE Brtb_hist THEN DO:
      Merror = SUBSTITUTE("PDBASE '&1' version &2 has never been assigned to workspace '&3'.", PdbaseObj,Pversion,Pwspace-id).
      LEAVE $RPT-BLOCK.
    END.

    FIND Brtb_ver OF Brtb_hist NO-LOCK NO-ERROR.
    
    IF NOT AVAILABLE(Brtb_ver) THEN DO:
      Merror = SUBSTITUTE("Version record for PDBASE '&1' version &2 not available.", PdbaseObj,Pversion).
      LEAVE $RPT-BLOCK.
    END.

    {rtb/i/rtb_rpthdr.i
      &title="""PDBASE Object Report"""
      &criteria=""" Object:"" Brtb_ver.object                 SKIP
                 ""PModule:"" Brtb_ver.pmod  FORMAT ""X(80)"" SKIP
                 ""Version:"" Brtb_ver.version                SKIP"
      &center-criteria=NO
      &STREAM=Sout}

    FIND Brtb_dbase OF Brtb_ver NO-LOCK.

    DO Mi = 1 TO 16:
      Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_ver.text-desc[Mi],60).
      DO WHILE Mline-out <> ?:
        PUT STREAM Sout UNFORMATTED Mline-out SKIP.
        Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
      END.
    END.  /* do mi = 1 to 16 */

    FOR EACH Brtb_dbfile OF Brtb_dbase NO-LOCK 
      BY Brtb_dbfile.local-file-name
      WITH FRAME file-frame STREAM-IO WIDTH 120:
       
      DISPLAY STREAM Sout
        Brtb_dbfile.file-object FORMAT "X(60)"
        Brtb_dbfile.local-file-name.

    END.

    FOR EACH Brtb_dbalias OF Brtb_dbase NO-LOCK
      WITH FRAME alias-frame STREAM-IO:
      
      DISPLAY STREAM Sout
        Brtb_dbalias.alias-name.

    END.

    IF CAN-FIND(FIRST Brtb_sequence OF Brtb_dbase) THEN DO:
      PUT STREAM Sout UNFORMATTED
      SKIP(1)
      "Sequences" SKIP
      "---------".
    END.

    FOR EACH Brtb_sequence OF Brtb_dbase NO-LOCK
      WITH FRAME sequence-frame 1 COLUMN SIDE-LABELS STREAM-IO:
      
      DISPLAY STREAM Sout
        Brtb_sequence.rtb_seq-name LABEL "Sequence Name"
        Brtb_sequence.rtb_cycle-ok
        Brtb_sequence.rtb_seq-incr
        Brtb_sequence.rtb_seq-init
        Brtb_sequence.rtb_seq-min  
        Brtb_sequence.rtb_seq-max SKIP(1).

    END.

    {rtb/i/rtb_rptftr.i &STREAM=Sout}

  END. /* $RPT-BLOCK */

  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_pfield_object_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_pfield_object_report Procedure 
PROCEDURE rtb_pfield_object_report :
/*------------------------------------------------------------------------------
  Purpose:     Print PFIELD object report
  Parameters:  INPUT Pwspace-id  - Workspace ID
               INPUT PfieldObj   - PFIELD object name
               INPUT Ppmod       - PFIELD object product module
               INPUT Pversion    - PFIELD object version
  Notes:       Formerly rtb_1300.p
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PfieldObj  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.

  DEFINE VARIABLE Merror   AS CHARACTER   NO-UNDO INITIAL "".
  DEFINE VARIABLE MlineOut AS CHARACTER   NO-UNDO.

  DEFINE BUFFER Brtb_flddef FOR rtb.rtb_flddef.
  DEFINE BUFFER Brtb_hist   FOR rtb.rtb_hist.
  DEFINE BUFFER Brtb_ver    FOR rtb.rtb_ver.

  $RPT-BLOCK:
  DO:

    FIND LAST Brtb_hist
      WHERE Brtb_hist.wspace-id = Pwspace-id
        AND Brtb_hist.pmod      = Ppmod
        AND Brtb_hist.obj-type  = "PFIELD":U
        AND Brtb_hist.object    = PfieldObj
        AND Brtb_hist.version   = Pversion NO-LOCK NO-ERROR.

    IF NOT AVAILABLE Brtb_hist THEN DO:
      Merror = SUBSTITUTE("PFIELD '&1' version &2 has never been assigned to workspace '&3'.", PfieldObj,Pversion,Pwspace-id).
      LEAVE $RPT-BLOCK.
    END.

    {rtb/i/rtb_rpthdr.i 
        &title="""Field Object Report"""
        &criteria="""Workspace:""  Brtb_hist.wspace-id                SKIP
                   ""   Object:""  Brtb_hist.object FORMAT ""x(80)""  SKIP
                   ""  PModule:""  Brtb_hist.pmod   FORMAT ""x(80)""  SKIP
                   ""  Version:""  Brtb_hist.version SKIP(1)"
        &center-criteria=NO
        &STREAM=Sout}

    FIND Brtb_ver OF Brtb_hist NO-LOCK.

    FIND Brtb_flddef OF Brtb_ver NO-LOCK.

    DISPLAY STREAM Sout
      Brtb_flddef.rtb_label      COLON 10
      Brtb_flddef.rtb_label-sa   NO-LABEL 
      Brtb_flddef.rtb_data-type  LABEL "Type" COLON 64

      Brtb_flddef.rtb_col-label  COLON 10
      Brtb_flddef.rtb_col-label-sa NO-LABEL
      Brtb_flddef.rtb_extent     COLON 64

      Brtb_flddef.rtb_format     COLON 10
      Brtb_flddef.rtb_format-sa  NO-LABEL
      Brtb_flddef.rtb_decimals   COLON 64

      Brtb_flddef.rtb_initial    COLON 10
      Brtb_flddef.rtb_initial-sa NO-LABEL
      Brtb_flddef.rtb_fld-case   COLON 64 LABEL "Case Sense?"
      WITH FRAME field-frame WIDTH 80  SIDE-LABELS STREAM-IO NO-BOX.

    /*
     Print VIEW-AS text
    */
    PUT STREAM Sout
      SKIP(1)
      "View As:" SKIP. 
    
    MlineOut = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_flddef.rtb_view-as,60).
    
    DO WHILE MlineOut <> ?:
      PUT STREAM Sout UNFORMATTED "    " MlineOut SKIP.
      MlineOut = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
    END.

    /*
     Print validation expression
    */
    PUT STREAM Sout
      SKIP(1)
      "Val:" SKIP. 
    
    MlineOut = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_flddef.rtb_valexp,60).
    
    DO WHILE MlineOut <> ?:
      PUT STREAM Sout UNFORMATTED "    " MlineOut SKIP.
      MlineOut = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
    END.

    /*
     Print validation message
    */
    DISPLAY STREAM Sout
      Brtb_flddef.rtb_valmsg LABEL "Mesg" FORMAT "x(63)"
      Brtb_flddef.rtb_valmsg-sa NO-LABEL 
      Brtb_flddef.rtb_help LABEL "Help" 
      Brtb_flddef.rtb_help-sa NO-LABEL
      WITH FRAME field-frame-3 WIDTH 80 SIDE-LABELS STREAM-IO NO-BOX.

    /*
     Print description
    */
    PUT STREAM Sout
      SKIP(1)
      "Desc:" SKIP. 
    
    MlineOut = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_flddef.rtb_desc,60).
    
    DO WHILE MlineOut <> ?:
      PUT STREAM Sout UNFORMATTED "    " MlineOut SKIP.
      MlineOut = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
    END.

    {rtb/i/rtb_rptftr.i &STREAM=Sout}

  END. /* $RPT-BLOCK */

  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_pfile_object_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_pfile_object_report Procedure 
PROCEDURE rtb_pfile_object_report :
/*------------------------------------------------------------------------------
  Purpose:     Print PFILE object report
  Parameters:  Pwspace-id - Workspace ID
               PfileObj   - PFILE object name
               Ppmod      - Product module
               Pversion   - PFILE object version
  Notes:       Formerly rtb_1200.p
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PfileObj   AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.

  DEFINE VARIABLE Merror    AS CHARACTER   NO-UNDO INITIAL "".
  DEFINE VARIABLE Mflags    AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mline-out AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mline-cnt AS INTEGER     NO-UNDO.

  DEFINE BUFFER Brtb_field   FOR rtb.rtb_field.
  DEFINE BUFFER Brtb_file    FOR rtb.rtb_file.
  DEFINE BUFFER Brtb_filtrig FOR rtb.rtb_filtrig.
  DEFINE BUFFER Brtb_hist    FOR rtb.rtb_hist.
  DEFINE BUFFER Brtb_idxfld  FOR rtb.rtb_idxfld.
  DEFINE BUFFER Brtb_index   FOR rtb.rtb_index.
  DEFINE BUFFER Brtb_ver     FOR rtb.rtb_ver.

  $RPT-BLOCK:
  DO:

    {rtb/i/rtb_rpthdr.i
       &title="""PFILE Object Report"""
       &criteria=""" Object:"" PfileObj FORMAT ""X(80)""  SKIP
                  ""PModule:"" Ppmod    FORMAT ""X(80)""  SKIP
                  ""Version:"" Pversion FORMAT ""999999"" SKIP"
       &center-criteria=NO
       &STREAM=Sout}

    FIND LAST Brtb_hist
      WHERE Brtb_hist.wspace-id = Pwspace-id
        AND Brtb_hist.pmod      = Ppmod
        AND Brtb_hist.obj-type  = "PFILE":U
        AND Brtb_hist.object    = PfileObj
        AND Brtb_hist.version   = Pversion NO-LOCK NO-ERROR.
    
    IF NOT AVAILABLE Brtb_hist THEN DO:
      Merror = SUBSTITUTE("PFILE '&1' version &2 has never been assigned to workspace '&3'.", PfileObj,Pversion,Pwspace-id).
      LEAVE $RPT-BLOCK.
    END.

    FIND Brtb_ver OF Brtb_hist NO-LOCK NO-ERROR.
    
    IF NOT AVAILABLE Brtb_ver THEN DO:
      Merror = SUBSTITUTE("Version record for PFILE '&1' not available.",PfileObj).
      LEAVE $RPT-BLOCK.
    END.

    FIND Brtb_file OF Brtb_ver NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_file THEN DO:
      Merror = SUBSTITUTE("File record for PFILE '&' not available.",PfileObj).
      LEAVE $RPT-BLOCK.
    END.

    PAGE STREAM Sout.

    DISPLAY STREAM Sout
      Brtb_file.rtb_dump-name COLON 12
      Brtb_file.rtb_hidden LABEL "Hidden" COLON 63
      Brtb_file.rtb_frozen  COLON 12
      Brtb_file.rtb_fil-misc2[6]  COLON 12 FORMAT "x(50)" LABEL "Replication"
      Brtb_file.rtb_file-label COLON 12
      Brtb_file.rtb_file-label-sa NO-LABEL
      WITH FRAME file-frame
      SIDE-LABELS NO-ATTR-SPACE NO-BOX STREAM-IO.

    PUT STREAM Sout UNFORMATTED "Description:" SKIP. 
    Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_file.rtb_desc,60).
    DO WHILE Mline-out <> ?:
     PUT STREAM Sout UNFORMATTED "    " Mline-out SKIP.
     Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
    END.

    PUT STREAM Sout "Valexp:" SKIP. 
    Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_file.rtb_valexp,60).
    DO WHILE Mline-out <> ?:
     PUT STREAM Sout UNFORMATTED "    " Mline-out SKIP.
     Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
    END.

    PUT STREAM Sout "Valmsg:" SKIP.     
    Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_file.rtb_valmsg,60).
    DO WHILE Mline-out <> ?:
     PUT STREAM Sout UNFORMATTED "    " Mline-out SKIP.
     Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
    END.

    PUT STREAM Sout
      SKIP(1)
      "INDEX FLAGS: P)rimary, U)nique, W)ord, I)nactive, A)bbreviated, + Asc, - Desc" SKIP
      SKIP(1).

    /*
     Display File Index Section
    */
    FOR EACH Brtb_index OF Brtb_file NO-LOCK,
      EACH Brtb_idxfld OF Brtb_index NO-LOCK,
      FIRST Brtb_field OF Brtb_idxfld NO-LOCK
      BREAK BY Brtb_index.rtb_index-name
      WITH FRAME index-frame STREAM-IO NO-ATTR-SPACE WIDTH 120:

      FORM
        Mflags                      FORMAT "x(5)"  COLUMN-LABEL "Flags"
        Brtb_index.rtb_index-name   FORMAT "x(29)" COLUMN-LABEL "Index Name"
        Brtb_idxfld.rtb_ascending   FORMAT "+/- "  COLUMN-LABEL "Fi" SPACE(0)
        Brtb_field.local-field-name FORMAT "x(28)" COLUMN-LABEL "eld Name"
        WITH FRAME f-index 
        NO-BOX DOWN USE-TEXT STREAM-IO.

      IF FIRST-OF(Brtb_index.rtb_index-name) THEN DO:
        Mflags = "".
        IF Brtb_index.index-id = Brtb_file.rtb_prime-index-id THEN Mflags = "P".
        IF Brtb_index.rtb_unique THEN Mflags = Mflags + "U".
        IF Brtb_index.rtb_wordidx = 1 THEN Mflags = Mflags + "W".
        IF NOT Brtb_index.rtb_active THEN Mflags = Mflags + "I".
        IF AVAILABLE Brtb_idxfld AND Brtb_idxfld.rtb_abbreviate THEN Mflags = Mflags + "A".        
      END.

      IF FIRST-OF(Brtb_index.rtb_index-name) THEN
        DISPLAY STREAM Sout
          Mflags
          Brtb_index.rtb_index-name
          WITH FRAME f-index.

      DISPLAY STREAM Sout
        Brtb_idxfld.rtb_ascending
        Brtb_field.local-field-name
        WITH FRAME f-index.
      
      DOWN STREAM Sout WITH FRAME f-index.

      IF LAST-OF(Brtb_index.rtb_index-name) THEN DO:
        
        IF LENGTH(Brtb_index.rtb_desc) > 0 THEN DO:
          
          PUT STREAM Sout "Desc: ". 
          
          ASSIGN
            Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_index.rtb_desc,60)
            Mline-cnt = 0.

          DO WHILE Mline-out <> ?:
            Mline-cnt = Mline-cnt + 1.
            IF Mline-cnt > 1 THEN
              PUT STREAM Sout UNFORMATTED "     ".
            PUT STREAM Sout UNFORMATTED
              Mline-out SKIP.
            Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,"",60).
          END.

        END.

        PUT STREAM Sout SKIP(1).

      END.  /* if first-of(brtb_index ... */

    END. /* EACH Brtb_index */

    PUT SKIP(1).

    /*
     Display File Triggers Section
    */
    FOR EACH Brtb_filtrig OF Brtb_file NO-LOCK WITH FRAME triggers-frame:
      DISPLAY STREAM Sout
        Brtb_filtrig.rtb_event     LABEL "Trigger Event"
        Brtb_filtrig.rtb_override
        Brtb_filtrig.rtb_proc-name FORMAT "x(60)"
        WITH FRAME triggers-frame STREAM-IO NO-ATTR-SPACE WIDTH 120.
    END.

    PUT STREAM Sout SKIP(1).

    PUT STREAM Sout "FIELD FLAGS: I)n-index  M)andatory " SKIP.

    /*
    Display Field Section
    */
    FOR EACH Brtb_field OF Brtb_file NO-LOCK
      BY Brtb_field.rtb_order
      WITH NO-ATTR-SPACE STREAM-IO WIDTH 120:

      FIND FIRST Brtb_idxfld 
           WHERE Brtb_idxfld.pmod      = Brtb_file.pmod
             AND Brtb_idxfld.obj-type  = Brtb_file.obj-type
             AND Brtb_idxfld.object    = Brtb_file.object
             AND Brtb_idxfld.version   = Brtb_file.version
             AND Brtb_idxfld.field-id  = Brtb_field.field-id
             NO-LOCK NO-ERROR.

      Mflags = (IF Brtb_field.rtb_mandatory THEN "M" ELSE "")
             + (IF AVAILABLE Brtb_idxfld THEN "I" ELSE "").
          
      DISPLAY STREAM Sout
        Brtb_field.rtb_order        FORMAT ">>>>9"
        Brtb_field.local-field-name
        Brtb_field.flddef-object    FORMAT "x(60)"
        Mflags                      FORMAT "x(2)"  LABEL "Flags".

    END. /* of EACH Brtb_field */

    {rtb/i/rtb_rptftr.i &STREAM=Sout}

  END. /* $RPT-BLOCK */  

  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_print_file_detail) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_print_file_detail Procedure 
PROCEDURE rtb_print_file_detail PRIVATE :
/*------------------------------------------------------------------------------
  Purpose:     Print file/field details
  Parameters:  INPUT PhistRowid  - ROWID of rtb_object for PFILE
               INPUT PbriefList - TRUE to print list of fields in file
               INPUT PfullDesc  - TRUE to print complete field definitions
  Notes:       Formerly rtb0275.p
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER PhistRowid AS ROWID   NO-UNDO.
  DEFINE INPUT PARAMETER Pevent     AS INTEGER NO-UNDO.
  DEFINE INPUT PARAMETER PbriefList AS LOGICAL NO-UNDO.
  DEFINE INPUT PARAMETER PfullDesc  AS LOGICAL NO-UNDO.

  DEFINE VARIABLE Merror    AS CHARACTER               NO-UNDO.
  DEFINE VARIABLE Mflags    AS CHARACTER FORMAT "x(3)" NO-UNDO.
  DEFINE VARIABLE Mflddesc  AS CHARACTER               NO-UNDO.
  DEFINE VARIABLE Mline-out AS CHARACTER               NO-UNDO. 
  DEFINE VARIABLE MszLabel  AS CHARACTER FORMAT "x(9)" NO-UNDO.
  DEFINE VARIABLE MszValue  AS CHARACTER               NO-UNDO.
  DEFINE VARIABLE Mi        AS INTEGER                 NO-UNDO.
  DEFINE VARIABLE Mline     AS INTEGER                 NO-UNDO.
  DEFINE VARIABLE Mrowid    AS ROWID                   NO-UNDO.

  DEFINE BUFFER Brtb_file     FOR rtb.rtb_file.
  DEFINE BUFFER Brtb_field    FOR rtb.rtb_field.
  DEFINE BUFFER Brtb_filtrig  FOR rtb.rtb_filtrig.
  DEFINE BUFFER Brtb_flddef   FOR rtb.rtb_flddef.
  DEFINE BUFFER Brtb_fldtrig  FOR rtb.rtb_fldtrig.
  DEFINE BUFFER Brtb_hist     FOR rtb.rtb_hist.
  DEFINE BUFFER Brtb_idxfld   FOR rtb.rtb_idxfld.
  DEFINE BUFFER Brtb_index    FOR rtb.rtb_index.
  DEFINE BUFFER Brtb_ver      FOR rtb.rtb_ver.

  $FILE-DETAIL-BLOCK:
  DO:
    FIND Brtb_hist WHERE ROWID(Brtb_hist) = PhistRowid NO-LOCK.
    FIND Brtb_ver OF Brtb_hist NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_ver THEN DO:
      Merror = SUBSTITUTE("Version record for &1 '&2' version &3 not available.", Brtb_hist.obj-type,Brtb_hist.object,Brtb_hist.version).
      LEAVE $FILE-DETAIL-BLOCK.
    END.
    FIND Brtb_file OF Brtb_ver  NO-LOCK.

    PAGE STREAM Sout.

    DISPLAY STREAM Sout
      Brtb_hist.pmod                                 FORMAT "x(80)" COLON 12 
      Brtb_hist.module                               FORMAT "x(80)" COLON 12 
      Brtb_file.rtb_dump-name                                       COLON 12
      Brtb_file.rtb_hidden       LABEL "Hidden"                     COLON 12
      Brtb_file.rtb_frozen                                          COLON 12
      Brtb_file.rtb_fil-misc2[6] LABEL "Replication" FORMAT "x(50)" COLON 12  
      WITH FRAME file-frame STREAM-IO SIDE-LABELS NO-ATTR-SPACE NO-BOX WIDTH 120.

    /*
     Print Description
    */
    PUT STREAM Sout "Description:". 
    ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_file.rtb_desc,60).
           Mline     = 1.
    DO WHILE Mline-out <> ?:
      PUT STREAM Sout UNFORMATTED
        SUBSTITUTE("&1&2",FILL(" ",(IF Mline = 1 THEN 1 ELSE 13)),Mline-out) SKIP.
      ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
             Mline     = Mline + 1.
    END.  /* do while return-value = "" */  

    /*
     Print Valexp
    */
    PUT STREAM Sout "Valexp:". 
    ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_file.rtb_valexp,60).
           Mline     = 1.
    DO WHILE Mline-out <> ?:
      PUT STREAM Sout UNFORMATTED
        SUBSTITUTE("&1&2",FILL(" ",(IF Mline = 1 THEN 1 ELSE 9)),Mline-out) SKIP.
      ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
             Mline     = Mline + 1.
    END.
    IF Mline =1 THEN
      PUT STREAM Sout SKIP.

    /*
     Print Valmsg
    */
    PUT STREAM Sout "Valmsg:". 
    ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_file.rtb_valmsg,60).
           Mline     = 1.
    DO WHILE Mline-out <> ?:
      PUT STREAM Sout UNFORMATTED
       SUBSTITUTE("&1&2",FILL(" ",(IF Mline = 1 THEN 1 ELSE 8)),Mline-out) SKIP.
      ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
             Mline     = Mline + 1.
    END.

    FORM
      SKIP(1)
      "INDEX FLAGS: A)ctive  U)nique  P)rimary  +)ascending -)descending"
      SKIP(1)
      WITH FRAME idx-flags-frame NO-BOX PAGE-BOTTOM STREAM-IO.

    /*
     Print File Index Section
    */
    FOR EACH Brtb_index OF Brtb_file NO-LOCK,
        EACH Brtb_idxfld OF Brtb_index NO-LOCK,
        FIRST Brtb_field OF Brtb_idxfld NO-LOCK
        BREAK BY Brtb_index.rtb_index-name
        WITH FRAME index-frame STREAM-IO NO-BOX NO-ATTR-SPACE:

      VIEW STREAM Sout FRAME idx-flags-frame.

      IF FIRST-OF(Brtb_index.rtb_index-name) THEN DO:
        PUT STREAM Sout SKIP(1).
        Mflags = (IF Brtb_index.rtb_active THEN "A" ELSE "")
                 + (IF Brtb_index.rtb_unique THEN "U" ELSE "")
                 + (IF Brtb_file.rtb_prime-index-id  = Brtb_index.index-id THEN
                      "P"
                    ELSE
                      "").
      END.

      DISPLAY STREAM Sout
        Brtb_index.rtb_index-name                WHEN FIRST-OF(Brtb_index.rtb_index-name)
        Mflags                     LABEL "Flags" WHEN FIRST-OF(Brtb_index.rtb_index-name)
        Brtb_idxfld.rtb_ascending  FORMAT "+/-" NO-LABEL
        Brtb_field.local-field-name
        Brtb_idxfld.rtb_abbreviate FORMAT "Abbr/"  NO-LABEL
        WITH FRAME index-frame.

      /*
       Print index description
      */
      IF LAST-OF(Brtb_index.rtb_index-name) THEN DO:
        PUT STREAM Sout SKIP(1) "Desc:".
        ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_index.rtb_desc,60)
               Mline     = 1.
        DO WHILE Mline-out <> ?:
          PUT STREAM Sout UNFORMATTED
            FILL(" ",(IF Mline > 1 THEN 6 ELSE 1)) Mline-out SKIP. 
          ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60)
                 Mline = Mline + 1.
        END.
      END.  /* last-of(brtb_index.rtb_index-name) */

    END. /* EACH Brtb_index */

    PAGE STREAM Sout.

    FOR EACH Brtb_filtrig OF Brtb_file NO-LOCK WITH FRAME trigger-frame:
      DISPLAY STREAM Sout
        Brtb_filtrig.rtb_event LABEL "Trigger Event"
        Brtb_filtrig.rtb_override
        Brtb_filtrig.rtb_proc-name
        WITH FRAME trigger-frame NO-BOX STREAM-IO.
    END.

    PUT STREAM Sout SKIP(1).

    FORM
      "FIELD FLAGS: I)n-index  M)andatory  C)ase-sensitive" SKIP(1)
      WITH FRAME fld-flags-frame
      NO-BOX PAGE-BOTTOM STREAM-IO.

    IF PbriefList THEN
      VIEW STREAM Sout FRAME fld-flags-frame.

    /*
     Print Field Section
    */
    IF Pbrieflist THEN DO:

      FOR EACH Brtb_field OF Brtb_file NO-LOCK
        BY Brtb_field.rtb_order
        WITH NO-ATTR-SPACE:

        FIND FIRST Brtb_idxfld 
             WHERE Brtb_idxfld.pmod      = Brtb_file.pmod
               AND Brtb_idxfld.obj-type  = Brtb_file.obj-type
               AND Brtb_idxfld.object    = Brtb_file.object
               AND Brtb_idxfld.version   = Brtb_file.version
               AND Brtb_idxfld.field-id  = Brtb_field.field-id
               NO-LOCK NO-ERROR.

        Mflags = (IF Brtb_field.rtb_mandatory THEN "M" ELSE "")
               + (IF AVAILABLE Brtb_idxfld THEN "I" ELSE "").
           
        /*
         Find last valid record
        */
        RUN rtb_find_last (INPUT  Brtb_hist.wspace-id,
                       INPUT  Brtb_field.flddef-obj-type,
                       INPUT  Brtb_field.flddef-object,
                       INPUT  Pevent,
                       OUTPUT Mrowid).

        FIND Brtb_flddef WHERE ROWID(Brtb_flddef) = Mrowid NO-LOCK.

        Mflddesc = (IF Brtb_flddef.rtb_data-type = "DATETIME" THEN
                      "datetm":U
                    ELSE IF Brtb_flddef.rtb_data-type = "DATETIME-TZ" THEN
                      "datetm-tz":U
                    ELSE IF Brtb_flddef.rtb_data-type = "INT64" THEN
                      "I64":U
                    ELSE
                      SUBSTRING(Brtb_flddef.rtb_data-type,1,4,"CHARACTER":U))
                   + (IF Brtb_flddef.rtb_data-type = "Decimal" THEN
                        TRIM(STRING(Brtb_flddef.rtb_decimals))
                      ELSE
                        "")
                   + (IF Brtb_flddef.rtb_extent > 0 THEN
                        "[" + TRIM(STRING(Brtb_flddef.rtb_extent)) + "]" 
                      ELSE
                        "")
                   + ' "' + Brtb_flddef.rtb_format + '"'.

        Mflddesc = Mflddesc +  IF Brtb_flddef.rtb_initial = ? THEN
                                 " ?"
                               ELSE
                                 " " + Brtb_flddef.rtb_initial. 
        IF Brtb_flddef.rtb_fld-case THEN Mflags = Mflags + "C".

        DISPLAY STREAM Sout
          Brtb_field.local-field-name FORMAT "x(20)"
          Mflddesc                    FORMAT "x(46)" LABEL "Datatype,Format,Initial"
          Brtb_field.rtb_order        FORMAT "zzzz9"
          Mflags                      FORMAT "x(2)"  NO-LABEL
          WITH NO-BOX STREAM-IO.

      END.

      PAGE STREAM Sout.
    END.

    IF Pfulldesc THEN
      $DESC-BLOCK:
      FOR EACH Brtb_field OF Brtb_file NO-LOCK
        BREAK BY Brtb_field.local-field-name
        WITH FRAME field-frame STREAM-IO SIDE-LABELS NO-ATTR-SPACE:

        /*
         Find last valid record
        */
        RUN rtb_find_last (INPUT  Brtb_hist.wspace-id,
                       INPUT  Brtb_field.flddef-obj-type,
                       INPUT  Brtb_field.flddef-object,
                       INPUT  Pevent,
                       OUTPUT Mrowid).

        FIND Brtb_flddef WHERE ROWID(Brtb_flddef) = Mrowid NO-LOCK.

        IF FIRST(Brtb_field.local-field-name) THEN 
          PUT STREAM Sout SKIP(1) FILL("=",78) FORMAT "x(78)" SKIP.
        ELSE 
          PUT STREAM Sout FILL("-",78) FORMAT "x(78)" SKIP.

        FIND FIRST Brtb_idxfld
          WHERE Brtb_idxfld.pmod      = Brtb_file.pmod
            AND Brtb_idxfld.obj-type  = Brtb_file.obj-type
            AND Brtb_idxfld.object    = Brtb_file.object
            AND Brtb_idxfld.version   = Brtb_file.version
            AND Brtb_idxfld.field-id  = Brtb_field.field-id
            NO-LOCK NO-ERROR.

        ASSIGN 
          MszLabel = IF CAN-DO("BLOB,CLOB,XLOB":U,Brtb_flddef.rtb_Data-type) THEN
                       " Max Size:"
                     ELSE
                       "SQL Width:"
          MszValue = IF CAN-DO("BLOB,CLOB,XLOB":U,Brtb_flddef.rtb_Data-type) THEN
                       Brtb_flddef.rtb_Fld-misc2[1]
                     ELSE
                       TRIM(STRING(Brtb_flddef.rtb_Width,">>>>9")).

        DISPLAY STREAM Sout
          Brtb_flddef.object          COLON 8  LABEL "Object" FORMAT "x(32)"
          Brtb_flddef.version         COLON 69 LABEL "Ver"
          Brtb_flddef.pmod            COLON 8  LABEL "Pmod"   FORMAT "x(80)"
          Brtb_field.local-field-name COLON 8  LABEL "FIELD"
          Brtb_field.rtb_mandatory    COLON 47 LABEL "Mand?"
          Brtb_field.rtb_order        COLON 69 LABEL "Order"  FORMAT ">>>>9"
          Brtb_flddef.rtb_format      COLON 8  LABEL "Format"
          Brtb_flddef.rtb_format-sa   NO-LABEL
            "" WHEN Brtb_flddef.rtb_format-sa EQ ? @ Brtb_flddef.rtb_format-sa 
          MszLabel                    NO-LABEL AT 60
          MszValue                    COLON 69 LABEL ""
          (IF Brtb_flddef.rtb_data-type = "DATETIME":U THEN
             "datetm":U
           ELSE IF Brtb_flddef.rtb_data-type = "DATETIME-TZ":U THEN
             "datetm-tz":U
           ELSE IF Brtb_flddef.rtb_data-type = "INT64":U THEN
             "I64":U
           ELSE
             LC(SUBSTRING(Brtb_flddef.rtb_data-type,1,4,"CHARACTER":U)))
                    + (IF Brtb_flddef.rtb_data-type = "Decimals" THEN
                         TRIM(STRING(Brtb_flddef.rtb_decimals))
                       ELSE
                         "")
                    + (IF Brtb_flddef.rtb_extent > 0 THEN
                         "[" + TRIM(STRING(Brtb_flddef.rtb_extent)) + "]"
                       ELSE
                         "")
                                      COLON 8  LABEL "Type"
          Brtb_flddef.rtb_fld-case    COLON 69 LABEL "Case"
          Brtb_flddef.rtb_label       COLON 8  LABEL "Label"
          Brtb_flddef.rtb_label-sa             NO-LABEL 
            "" WHEN Brtb_flddef.rtb_label-sa EQ ? @ Brtb_flddef.rtb_label-sa 
          "In-Index:" TO 69 (AVAILABLE Brtb_idxfld) FORMAT "Yes/No"
          Brtb_flddef.rtb_col-label   COLON 8  LABEL "Column"
          Brtb_flddef.rtb_col-label-sa NO-LABEL
           "" WHEN Brtb_flddef.rtb_col-label-sa EQ ? @ Brtb_flddef.rtb_col-label-sa
          Brtb_flddef.rtb_initial     COLON 8 LABEL "Initial"
          Brtb_flddef.rtb_initial-sa          NO-LABEL
            "" WHEN Brtb_flddef.rtb_initial-sa EQ ? @ Brtb_flddef.rtb_initial-sa 
          WITH FRAME field-frame WIDTH 120 STREAM-IO.  

          PUT STREAM Sout "View-as:" TO 8.
          ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_flddef.rtb_view-as,60)
                 Mline     = 1.
          DO WHILE Mline-out <> ?:
            PUT STREAM Sout UNFORMATTED
              SUBSTITUTE("&1&2",FILL(" ",(IF Mline = 1 THEN 1 ELSE 9)),Mline-out) SKIP.
            ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60)
                   Mline     = Mline + 1.
          END.
          
          Mline = 1.
          
          PUT STREAM Sout "Valexp:" TO 8.
          ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_flddef.rtb_valexp,60)
                 Mline     = 1.
          DO WHILE Mline-out <> ?:
            PUT STREAM Sout UNFORMATTED
              SUBSTITUTE("&1&2",FILL(" ",(IF Mline = 1 THEN 1 ELSE 9)),Mline-out) SKIP.
            ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60)
                   Mline     = Mline + 1.
          END.  /* do while return-value = "" */
          IF Mline = 1 THEN
            PUT STREAM Sout SKIP.

          IF Brtb_flddef.rtb_valmsg <> "" THEN 
            PUT STREAM Sout
              "Valmsg:"              TO 8 
              Brtb_flddef.rtb_valmsg AT 10 " " 
              Brtb_flddef.rtb_valmsg-sa SKIP.

          IF Brtb_flddef.rtb_help <> "" THEN
            PUT STREAM Sout
              "Help:"              TO 8
              Brtb_flddef.rtb_help AT 10 " " 
              Brtb_flddef.rtb_help-sa SKIP.

          IF Brtb_flddef.rtb_desc <> "" THEN DO:
            PUT STREAM Sout "Desc:" TO 8. 
            ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_flddef.rtb_desc,60)
                   Mline     = 1.
            DO WHILE Mline-out <> ?:
              PUT STREAM Sout UNFORMATTED
                SUBSTITUTE("&1&2",FILL(" ",(IF Mline = 1 THEN 1 ELSE 9)),Mline-out) SKIP.
              ASSIGN Mline-out = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60)
                     Mline     = Mline + 1.
            END.  /* do while return-value = "" */      
          END.  /* if brtb_flddef.rtb_desc <> "" */

          FIND FIRST Brtb_fldtrig OF Brtb_field NO-LOCK NO-ERROR.
          IF AVAILABLE Brtb_fldtrig THEN
            PUT STREAM Sout
              "Trigger Proc: " Brtb_fldtrig.rtb_proc-name
                 " Override: " Brtb_fldtrig.rtb_override SKIP.

      END. /* $DESC-BLOCK */

  END. /* $FILE-DETAIL-BLOCK */

  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_table_definition_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_table_definition_report Procedure 
PROCEDURE rtb_table_definition_report :
/*------------------------------------------------------------------------------
  Purpose:     Print table definition report
  Parameters:  INPUT Pwspace-id  - Workspace ID
               INPUT PfileObj    - PFILE object name
               INPUT Ppmod       - PFILE object pmod
               INPUT Pversion    - PFILE object version
               INPUT Pbrieflist  - TRUE to print brief field list
               INPUT Pfulldesc   - TRUE to print field definitions
  Notes:       Formerly rtb_1201.w
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PfileObj   AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PbriefList AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PfullDesc  AS LOGICAL     NO-UNDO.

  DEFINE VARIABLE Merror AS CHARACTER   NO-UNDO INITIAL "".
  DEFINE VARIABLE Mevent AS INTEGER     NO-UNDO.

  DEFINE BUFFER Brtb_hist   FOR rtb.rtb_hist.
  DEFINE BUFFER Brtb_object FOR rtb.rtb_object.

  $RPT-BLOCK:
  DO:
    FIND LAST Brtb_hist
      WHERE Brtb_hist.wspace-id = Pwspace-id
        AND Brtb_hist.pmod      = Ppmod
        AND Brtb_hist.obj-type  = "PFILE":U
        AND Brtb_hist.object    = PfileObj
        AND Brtb_hist.version   = Pversion NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_hist THEN DO:
      Merror = SUBSTITUTE("PFILE '&1' version &2 has never been assigned to workspace '&3'.",
                          PfileObj,Pversion,Pwspace-id).
      LEAVE $RPT-BLOCK.
    END.

    FIND FIRST Brtb_object WHERE Brtb_object.wspace-id = Brtb_hist.wspace-id
                             AND Brtb_object.obj-type  = Brtb_hist.obj-type
                             AND Brtb_object.object    = Brtb_hist.object
                             NO-LOCK NO-ERROR.

    IF AVAILABLE Brtb_object
       AND Brtb_object.version = Brtb_hist.version 
       AND Brtb_object.pmod = Brtb_hist.pmod THEN 
      ASSIGN Mevent = 99999999.
    ELSE
      ASSIGN Mevent = Brtb_hist.last-event-count.

    DO ON ENDKEY UNDO, LEAVE
       ON ERROR  UNDO, LEAVE:

       {rtb/i/rtb_rpthdr.i
         &title="""Table Definition Report"""
         &criteria="""   Workspace:"" Brtb_hist.wspace-id SKIP
                    ""Table Object:"" Brtb_hist.object FORMAT ""x(32)""
                    "" Version:""     Brtb_hist.version SKIP(1)"
         &center-criteria=NO
         &STREAM=Sout}

       /* Print body of table report */
       RUN rtb_print_file_detail (INPUT ROWID(Brtb_hist),
                                  INPUT Mevent,
                                  INPUT Pbrieflist,
                                  INPUT Pfulldesc).
    END. /* DO: */

    {rtb/i/rtb_rptftr.i &STREAM=Sout}

  END.  /* $RPT-BLOCK */

  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_unapplied_changes_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_unapplied_changes_report Procedure 
PROCEDURE rtb_unapplied_changes_report :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id    AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PdbChanges    AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PtableChanges AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PfieldChanges AS LOGICAL     NO-UNDO.

  DEFINE VARIABLE Merror  AS CHARACTER   NO-UNDO.

  DEFINE BUFFER Brtb_object FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_ver    FOR rtb.rtb_ver.

  rpt-block:
  DO:
    {rtb/i/rtb_rpthdr.i &title="""Unapplied Changes Report"""
                        &criteria="""Workspace :"" Pwspace-id SKIP(1)"
                        &center-criteria=NO
                        &STREAM=Sout}

     IF PfieldChanges THEN
       FOR EACH Brtb_object 
          WHERE Brtb_object.wspace-id   = Pwspace-id
            AND Brtb_object.obj-type    = "PFIELD":U NO-LOCK,   
          FIRST Brtb_ver OF Brtb_object  NO-LOCK:

         IF Brtb_ver.obj-status = "C" AND Brtb_object.update-status = "C" THEN
           NEXT.

         RUN rtb_unapplied_field_changes (INPUT Pwspace-id, INPUT Brtb_object.object).
       END. /* pfield */

     IF PtableChanges THEN
       FOR EACH Brtb_object 
          WHERE Brtb_object.wspace-id   = Pwspace-id
            AND Brtb_object.obj-type    = "PFILE" NO-LOCK, 
          FIRST Brtb_ver OF Brtb_object NO-LOCK:

         IF Brtb_ver.obj-status = "C" AND Brtb_object.update-status = "C" THEN
           NEXT.

         RUN rtb_unapplied_file_changes (INPUT Pwspace-id, INPUT Brtb_object.object).
       END. /* pfile */

     IF PdbChanges THEN
       FOR EACH Brtb_object 
          WHERE Brtb_object.wspace-id   = Pwspace-id
            AND Brtb_object.obj-type    = "PDBASE" NO-LOCK, 
          FIRST Brtb_ver OF Brtb_object NO-LOCK:

         IF Brtb_ver.obj-status = "C" AND Brtb_object.update-status = "C" THEN
           NEXT.

         RUN rtb_unapplied_dbase_changes (INPUT Pwspace-id, INPUT Brtb_object.object).
       END. /* of EACH "PDBASE" */

    {rtb/i/rtb_rptftr.i &STREAM=Sout}

  END. /* rpt-block */

  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_unapplied_dbase_changes) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_unapplied_dbase_changes Procedure 
PROCEDURE rtb_unapplied_dbase_changes :
/*------------------------------------------------------------------------------
  Purpose:     Print unapplied database changes report.
  Parameters:  INPUT Pwspace-id  - Workspace ID
               INPUT PdbaseObj   - PDBASE object name.
  Notes:       Formerly rtb_ckda.p
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PdbaseObj  AS CHARACTER   NO-UNDO.

  DEFINE VARIABLE Merror  AS CHARACTER   NO-UNDO INITIAL "".
  DEFINE VARIABLE Mdirect AS LOGICAL     NO-UNDO INITIAL NO.
  DEFINE VARIABLE Mmatch  AS LOGICAL     NO-UNDO INITIAL YES.

  DEFINE BUFFER Brtb_dbase     FOR rtb.rtb_dbase.
  DEFINE BUFFER B2rtb_dbase    FOR rtb.rtb_dbase.
  DEFINE BUFFER Brtb_dbfile    FOR rtb.rtb_dbfile.
  DEFINE BUFFER B2rtb_dbfile   FOR rtb.rtb_dbfile.
  DEFINE BUFFER Brtb_object    FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_sequence  FOR rtb.rtb_sequence.
  DEFINE BUFFER B2rtb_sequence FOR rtb.rtb_sequence.
  DEFINE BUFFER Brtb_ver       FOR rtb.rtb_ver.

  rpt-block:
  DO:
    /*
     Get specified PDBASE object
    */
    FIND Brtb_object
      WHERE Brtb_object.wspace-id = Pwspace-id
        AND Brtb_object.obj-type  = "PDBASE":U
        AND Brtb_object.object    = PdbaseObj NO-LOCK NO-ERROR.

    IF NOT AVAILABLE Brtb_object THEN DO:
      Merror = SUBSTITUTE("PDBASE '&1' not available in workspace '&2'.",
                          PdbaseObj,Pwspace-id).
      LEAVE rpt-block.
    END.

    FIND Brtb_ver OF Brtb_object NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_ver THEN DO:
      Merror = SUBSTITUTE("Version record for PDBASE '&1' not available.",PdbaseObj).
      LEAVE rpt-block.
    END.

    /*
     Brtb_dbase contains editing buffer image
    */
    FIND Brtb_dbase OF Brtb_ver NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_dbase THEN DO:
      Merror = SUBSTITUTE("Dbase record for PDBASE '&1' not available.",PdbaseObj).
      LEAVE rpt-block.
    END.

    {rtb/i/rtb_rpthdr.i 
       &title="""Unapplied Changes in PDBASE object (old, new)"""
       &criteria="""Object:"" Brtb_dbase.object FORMAT ""x(32)"""
       &center-criteria=NO
       &frame-no=1
       &STREAM=Sout}

    FORM HEADER
      SUBSTITUTE("Unapplied Changes in PDBASE &1 (old,new):",Brtb_dbase.object)
      FORMAT "x(70)"
      SKIP(1)
      WITH FRAME f-shorthdr
      NO-BOX NO-LABELS PAGE-TOP STREAM-IO.

    IF INDEX(PROGRAM-NAME(2),"rtb_unapplied_changes_report":U) > 0 THEN DO:
      HIDE STREAM Sout FRAME header-frame1.
      VIEW STREAM Sout FRAME f-shorthdr.
    END.
    ELSE
     Mdirect = YES.

    IF Brtb_object.update-status = "N" THEN DO:
      HIDE STREAM Sout FRAME f-shorthdr.
      FOR EACH Brtb_dbfile OF Brtb_dbase NO-LOCK:
        DISPLAY STREAM Sout
          Brtb_dbfile.local-file-name COLUMN-LABEL "Local-File-Name"
          Brtb_dbfile.file-object      COLUMN-LABEL "FILE Object"
          WITH FRAME files-frame COLUMN 3 DOWN STREAM-IO.
      END.
      Mmatch = NO.
      LEAVE rpt-block.
    END.

    /*
     rtb_dbase contains committed image
    */
    FIND B2rtb_dbase 
       WHERE B2rtb_dbase.pmod    = Brtb_object.committed-pmod
         AND B2rtb_dbase.obj-type= Brtb_object.obj-type
         AND B2rtb_dbase.object  = Brtb_object.object
         AND B2rtb_dbase.version = Brtb_object.committed-version NO-LOCK.

    /*
     Look for deleted files and changes
    */
    FOR EACH B2rtb_dbfile OF B2rtb_dbase NO-LOCK:

      FIND FIRST Brtb_dbfile OF Brtb_dbase 
        WHERE Brtb_dbfile.dbfile-id = B2rtb_dbfile.dbfile-id NO-LOCK NO-ERROR.

      IF NOT AVAILABLE Brtb_dbfile 
         OR NOT(Brtb_dbfile.local-file-name = B2rtb_dbfile.local-file-name
         AND Brtb_dbfile.file-obj-type      = B2rtb_dbfile.file-obj-type
         AND Brtb_dbfile.file-object        = B2rtb_dbfile.file-object) THEN DO:

        IF NOT AVAILABLE Brtb_dbfile THEN DO:
          PUT STREAM Sout
            "   File: " B2rtb_dbfile.local-file-name + " DELETED" 
            FORMAT "x(70)" SKIP.
          Mmatch = no.
          NEXT.
        END.

        IF Brtb_dbfile.local-file-name <> B2rtb_dbfile.local-file-name THEN 
          PUT STREAM Sout
            "   File: " B2rtb_dbfile.local-file-name + " , "
            + Brtb_dbfile.local-file-name 
            + " RENAMED" FORMAT "x(70)" SKIP.

        IF Brtb_dbfile.file-object <> B2rtb_dbfile.file-object THEN
          PUT STREAM Sout
            " FilObj: " B2rtb_dbfile.file-object + " , " 
            + Brtb_dbfile.file-object FORMAT "x(70)" SKIP.

        Mmatch = no.

      END. /* not avial brtb_dbfile... */

    END. /* each b2rtb_file */

    /*
     Look for new files
    */
    FOR EACH Brtb_dbfile OF Brtb_dbase NO-LOCK:

      IF NOT CAN-FIND(FIRST B2rtb_dbfile OF B2rtb_dbase
                      WHERE B2rtb_dbfile.dbfile-id = Brtb_dbfile.dbfile-id) THEN DO:
        PUT STREAM Sout
          "  File: " Brtb_dbfile.local-file-name + " ADDED" 
          FORMAT "x(70)" SKIP.
      END.

      Mmatch = no. 
      NEXT.
    END. /* EACH Brtb_dbfile */

    /*
     Look for new sequences
    */
    FOR EACH Brtb_sequence OF Brtb_dbase NO-LOCK:
      FIND B2rtb_sequence OF B2rtb_dbase
        WHERE B2rtb_sequence.rtb_seq-name = Brtb_sequence.rtb_seq-name
        NO-LOCK NO-ERROR.
      IF NOT AVAILABLE B2rtb_sequence THEN DO:
        PUT STREAM Sout
          "New sequence: " Brtb_sequence.rtb_seq-name SKIP
          "    Cycle-ok: " STRING(Brtb_sequence.rtb_cycle-ok) SKIP
          "   Increment: " STRING(Brtb_sequence.rtb_seq-incr) SKIP
          "     Initial: " STRING(Brtb_sequence.rtb_seq-init) SKIP
          "     Maximum: " (IF Brtb_sequence.rtb_seq-max = ? THEN
                              "?"
                            ELSE
                              STRING(Brtb_sequence.rtb_seq-max)) SKIP
          "     Minimum: " (IF Brtb_sequence.rtb_seq-min = ? THEN
                              "?" 
                            ELSE
                              STRING(Brtb_sequence.rtb_seq-min)) SKIP.
          Mmatch = no.
      END.
      ELSE IF Brtb_sequence.rtb_cycle-ok <> B2rtb_sequence.rtb_cycle-ok
           OR Brtb_sequence.rtb_seq-incr <> B2rtb_sequence.rtb_seq-incr
           OR Brtb_sequence.rtb_seq-init <> B2rtb_sequence.rtb_seq-init
           OR Brtb_sequence.rtb_seq-min  <> B2rtb_sequence.rtb_seq-min
           OR Brtb_sequence.rtb_seq-max  <> B2rtb_sequence.rtb_seq-max THEN DO:
         
        PUT STREAM Sout
           "    Sequence: " Brtb_sequence.rtb_seq-name SKIP
           "    Cycle-ok: " STRING(B2rtb_sequence.rtb_cycle-ok) "," 
                            STRING(Brtb_sequence.rtb_cycle-ok) SKIP
           "   Increment: " STRING(B2rtb_sequence.rtb_seq-incr) "," 
                            STRING(Brtb_sequence.rtb_seq-incr) SKIP
           "     Initial: " STRING(Brtb_sequence.rtb_seq-init) SKIP

           "     Maximum: " (IF B2rtb_sequence.rtb_seq-max = ?
                             THEN "?"
                             ELSE STRING(B2rtb_sequence.rtb_seq-max))
                            + "," +
                            (IF Brtb_sequence.rtb_seq-max = ?
                             THEN "?"
                             ELSE STRING(Brtb_sequence.rtb_seq-max)) 
                             FORMAT "x(78)" SKIP

           "     Minimum: " (IF B2rtb_sequence.rtb_seq-min = ?
                             THEN "?"
                             ELSE STRING(B2rtb_sequence.rtb_seq-min))
                            + "," +
                            (IF Brtb_sequence.rtb_seq-min = ?
                             THEN "?" 
                             ELSE STRING(Brtb_sequence.rtb_seq-min)) 
                             FORMAT "x(78)" SKIP.
        Mmatch = no.
      END.
    END.  /* for each brtb_sequence */

    /*
     Look for deleted sequences
    */
    FOR EACH B2rtb_sequence OF B2rtb_dbase NO-LOCK:

      FIND Brtb_sequence OF Brtb_dbase
         WHERE Brtb_sequence.rtb_seq-name = B2rtb_sequence.rtb_seq-name
         NO-LOCK NO-ERROR.

      IF NOT AVAILABLE Brtb_sequence THEN DO:
        PUT "DELETED Sequence: " B2rtb_sequence.rtb_seq-name SKIP.
        Mmatch = no.
      END.

    END.

  END. /* rpt-block */

  IF Mmatch AND (FRAME f-shorthdr:VISIBLE OR Mdirect) THEN
    PUT STREAM Sout UNFORMATTED
      "No Changes" SKIP(1).

  HIDE STREAM Sout FRAME f-shorthdr.

  IF Mdirect THEN DO:
    {rtb/i/rtb_rptftr.i &STREAM=Sout}
  END.

  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_unapplied_field_changes) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_unapplied_field_changes Procedure 
PROCEDURE rtb_unapplied_field_changes :
/*------------------------------------------------------------------------------
  Purpose:     Print unapplied field changes.
  Parameters:  INPUT Pwspace-id  - Workspace ID
               INPUT PfieldObj   - PFIELD object name
  Notes:       Formerly rtb_ckfl.p
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PfieldObj  AS CHARACTER   NO-UNDO.

  DEFINE VARIABLE Merror  AS CHARACTER   NO-UNDO INITIAL "".
  DEFINE VARIABLE Mi      AS INTEGER     NO-UNDO.
  DEFINE VARIABLE Mdirect AS LOGICAL     NO-UNDO INITIAL NO.
  DEFINE VARIABLE Mmatch  AS LOGICAL     NO-UNDO INITIAL YES.

  DEFINE BUFFER Brtb_flddef  FOR rtb.rtb_flddef.
  DEFINE BUFFER B2rtb_flddef FOR rtb.rtb_flddef.
  DEFINE BUFFER Brtb_object  FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_ver     FOR rtb.rtb_ver.

  $RPT-BLOCK:
  DO:
    FIND Brtb_object
      WHERE Brtb_object.wspace-id = Pwspace-id
        AND Brtb_object.obj-type  = "PFIELD":U
        AND Brtb_object.object    = PfieldObj NO-LOCK NO-ERROR.

    IF NOT AVAILABLE Brtb_object THEN DO:
      Merror = SUBSTITUTE("Object record for PFILE '&1' not available.",PfieldObj).
      LEAVE $RPT-BLOCK.
    END.

    FIND Brtb_ver OF Brtb_object NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_ver THEN DO:
      Merror = SUBSTITUTE("Version record for PFIELD '&1' not available.",PfieldObj).
      LEAVE $RPT-BLOCK.
    END.

    FIND B2rtb_flddef OF Brtb_ver NO-LOCK.

    {rtb/i/rtb_rpthdr.i 
       &title="""Unapplied Changes in PFIELD object (old, new)"""
       &criteria="""Object:"" B2rtb_flddef.object SKIP"
       &center-criteria=NO
       &frame-no=1
       &STREAM=Sout}

    FORM HEADER
      SUBSTITUTE("Unapplied Changes in PFIELD &1 (old, new):",B2rtb_flddef.object)
      FORMAT "x(70)"
      SKIP(1)
      WITH FRAME f-shorthdr
      NO-BOX NO-LABELS PAGE-TOP STREAM-IO.

    IF INDEX(PROGRAM-NAME(2),"rtb_unapplied_changes_report":U) > 0 THEN DO:
      HIDE STREAM Sout FRAME header-frame1.
      VIEW STREAM Sout FRAME f-shorthdr.
    END.
    ELSE
      Mdirect = YES.

    IF Brtb_ver.obj-status = "C" AND Brtb_object.update-status = "C" THEN
      LEAVE $RPT-BLOCK.

    IF Brtb_object.update-status = "N" THEN DO:

      HIDE STREAM Sout FRAME f-shorthdr.

      DISPLAY STREAM Sout
        SKIP(1)
        "New PFIELD object:" SKIP
        Brtb_object.OBJECT            COLON 10  FORMAT "X(80)"                     
        Brtb_object.module            COLON 10  FORMAT "X(80)"
        B2rtb_flddef.pmod             COLON 10  FORMAT "X(80)"
        B2rtb_flddef.rtb_label        COLON 10
        B2rtb_flddef.rtb_label-sa                              NO-LABEL 
        B2rtb_flddef.rtb_data-type    COLON 64                 LABEL "Type"
        B2rtb_flddef.rtb_col-label    COLON 10
        B2rtb_flddef.rtb_col-label-sa                          NO-LABEL
        B2rtb_flddef.rtb_extent       COLON 64
        B2rtb_flddef.rtb_format       COLON 10
        B2rtb_flddef.rtb_format-sa                             NO-LABEL
        B2rtb_flddef.rtb_decimals     COLON 64
        B2rtb_flddef.rtb_initial      COLON 10
        B2rtb_flddef.rtb_initial-sa   NO-LABEL
        B2rtb_flddef.rtb_fld-case     LABEL "Case Sense?" COLON 64 
        WITH FRAME field-frame WIDTH 120 SIDE-LABELS NO-BOX STREAM-IO.      

      IF B2rtb_flddef.rtb_view-as = ? THEN
        PUT STREAM Sout "  View As: ?" SKIP.
      ELSE DO:
        PUT STREAM Sout
          "  View As: " 
          + SUBSTRING(B2rtb_flddef.rtb_view-as,1,63) FORMAT "x(76)" SKIP.
        DO Mi = 1 TO LENGTH(B2rtb_flddef.rtb_view-as) BY 63:
          PUT STREAM Sout
            FILL(" ",9)
            + SUBSTRING(B2rtb_flddef.rtb_view-as,Mi,63) FORMAT "x(76)" SKIP.
        END.
      END.

      IF B2rtb_flddef.rtb_valexp = ? THEN
        PUT STREAM Sout "      Val: ?" SKIP.
      ELSE DO:
        PUT STREAM Sout
          "      Val: "
          + SUBSTRING(B2rtb_flddef.rtb_valexp,1,63) FORMAT "x(76)" SKIP.

        DO Mi = 1 TO LENGTH(B2rtb_flddef.rtb_valexp) BY 63:
          PUT STREAM Sout
            FILL(" ",9)
            + SUBSTRING(B2rtb_flddef.rtb_valexp,Mi,63) FORMAT "x(76)" SKIP.
        END.
      END.

      DISPLAY
        B2rtb_flddef.rtb_valmsg                LABEL "Mesg" COLON 10 FORMAT "x(50)"
        B2rtb_flddef.rtb_valmsg-sa             NO-LABEL 
        B2rtb_flddef.rtb_help                  LABEL "Help" COLON 10 FORMAT "x(50)"
        B2rtb_flddef.rtb_help-sa               NO-LABEL
        SUBSTRING(B2rtb_flddef.rtb_desc,1,65)  LABEL "Desc" COLON 10 FORMAT "x(65)"
        SUBSTRING(B2rtb_flddef.rtb_desc,66,65) NO-LABEL     COLON 10 FORMAT "x(65)"
        WITH FRAME field-frame-2 WIDTH 80 SIDE-LABELS NO-BOX STREAM-IO.      

      Mmatch = NO.

      LEAVE $RPT-BLOCK.

    END. /* update-status = N */

    IF Brtb_ver.obj-status <> "W" THEN
      LEAVE $RPT-BLOCK.

    /*
     The rtb_flddef buffer holds the committed image 
     The B2rtb_flddef buffer holds the edited image
    */
    FIND Brtb_flddef 
      WHERE Brtb_flddef.pmod     = Brtb_object.committed-pmod
        AND Brtb_flddef.obj-type = Brtb_object.obj-type
        AND Brtb_flddef.object   = Brtb_object.object
        AND Brtb_flddef.version  = Brtb_object.committed-version NO-LOCK.

    IF NOT(B2rtb_flddef.rtb_data-type = Brtb_flddef.rtb_data-type
       AND DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_initial,Brtb_flddef.rtb_initial)
       AND DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_label,Brtb_flddef.rtb_label)
       AND B2rtb_flddef.rtb_label-sa = Brtb_flddef.rtb_label-sa
       AND DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_format,Brtb_flddef.rtb_format)
       AND B2rtb_flddef.rtb_format-sa = Brtb_flddef.rtb_format-sa
       AND B2rtb_flddef.rtb_decimals = Brtb_flddef.rtb_decimals
       AND B2rtb_flddef.rtb_extent = Brtb_flddef.rtb_extent
       AND DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_view-as,Brtb_flddef.rtb_view-as)
       AND DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_valexp,Brtb_flddef.rtb_valexp)
       AND DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_valmsg,Brtb_flddef.rtb_valmsg)
       AND B2rtb_flddef.rtb_valmsg-sa = Brtb_flddef.rtb_valmsg-sa
       AND DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_help,Brtb_flddef.rtb_help)
       AND B2rtb_flddef.rtb_help-sa   = Brtb_flddef.rtb_help-sa
       AND DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_col-label,Brtb_flddef.rtb_col-label)
       AND B2rtb_flddef.rtb_col-label-sa = Brtb_flddef.rtb_col-label-sa
       AND B2rtb_flddef.rtb_fld-case = Brtb_flddef.rtb_fld-case
       AND DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_desc,Brtb_flddef.rtb_desc)
       ) THEN DO:

      PUT STREAM Sout SKIP(1).
      PUT STREAM Sout
        "Unapplied changes in PFIELD object, (old,new):" SKIP
        " Object: " Brtb_flddef.object FORMAT "x(32)" SKIP.

      IF B2rtb_flddef.rtb_data-type <> Brtb_flddef.rtb_data-type THEN
        PUT STREAM Sout
          "   Type: " Brtb_flddef.rtb_data-type + " , " 
          + B2rtb_flddef.rtb_data-type FORMAT "x(70)" SKIP.

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_label,Brtb_flddef.rtb_label) THEN
        PUT STREAM Sout
          "  Label: " (IF Brtb_flddef.rtb_label = ? THEN
                         "?" 
                       ELSE Brtb_flddef.rtb_label)
          + " , "  +  (IF B2rtb_flddef.rtb_label = ? THEN
                         "?" 
                       ELSE
                         B2rtb_flddef.rtb_label) FORMAT "x(70)" SKIP.

      IF B2rtb_flddef.rtb_label-sa <> Brtb_flddef.rtb_label-sa THEN
        PUT STREAM Sout
          "Label SA:" (IF Brtb_flddef.rtb_label-sa = ? THEN
                         "?" 
                       ELSE
                         Brtb_flddef.rtb_label-sa)
           + " , " +  (IF B2rtb_flddef.rtb_label-sa = ? THEN
                         "?" 
                       ELSE
                         B2rtb_flddef.rtb_label-sa) FORMAT "x(70)" SKIP.

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_col-label,Brtb_flddef.rtb_col-label) THEN
        PUT STREAM Sout
          " Column: " (IF Brtb_flddef.rtb_col-label = ? THEN
                         "?"
                       ELSE Brtb_flddef.rtb_col-label)
          + " , " + (IF B2rtb_flddef.rtb_col-label = ? THEN
                       "?"
                     ELSE
                       B2rtb_flddef.rtb_col-label) FORMAT "x(70)" SKIP.

      IF B2rtb_flddef.rtb_col-label-sa <> Brtb_flddef.rtb_col-label-sa THEN
        PUT STREAM Sout
          " Col SA: " (IF Brtb_flddef.rtb_col-label-sa = ? THEN
                         "?"
                       ELSE
                         Brtb_flddef.rtb_col-label-sa)
          + " , " + (IF B2rtb_flddef.rtb_col-label-sa = ? THEN
                       "?"
                     ELSE
                       B2rtb_flddef.rtb_col-label-sa) FORMAT "x(70)" SKIP.

      IF B2rtb_flddef.rtb_extent <> Brtb_flddef.rtb_extent THEN
        PUT STREAM Sout
          " Extent: " Brtb_flddef.rtb_extent " , " B2rtb_flddef.rtb_extent SKIP.

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_format,Brtb_flddef.rtb_format) THEN
        PUT STREAM Sout
          " Format: " Brtb_flddef.rtb_format + " , " 
          + B2rtb_flddef.rtb_format FORMAT "x(79)" SKIP.

      IF B2rtb_flddef.rtb_format-sa <> Brtb_flddef.rtb_format-sa THEN
        PUT STREAM Sout
          "Format SA: " (IF Brtb_flddef.rtb_format-sa = ? THEN
                           "?"
                         ELSE
                           Brtb_flddef.rtb_format-sa)
          + " , " + (IF B2rtb_flddef.rtb_format-sa = ? THEN
                       "?"
                     ELSE
                       B2rtb_flddef.rtb_format-sa) FORMAT "x(70)" SKIP.

      IF B2rtb_flddef.rtb_fld-case <> Brtb_flddef.rtb_fld-case THEN
        PUT STREAM Sout
          "   Case: " Brtb_flddef.rtb_fld-case  " , " B2rtb_flddef.rtb_fld-case SKIP.

      IF B2rtb_flddef.rtb_decimals <> Brtb_flddef.rtb_decimals THEN
        PUT STREAM Sout
          "Decimal:" Brtb_flddef.rtb_decimals " , " B2rtb_flddef.rtb_decimals SKIP.

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_initial,Brtb_flddef.rtb_initial) THEN
        PUT STREAM Sout
          "Initial: " (IF Brtb_flddef.rtb_initial = ? THEN
                         "?" 
                       ELSE
                         Brtb_flddef.rtb_initial) 
          + " , " +  (IF B2rtb_flddef.rtb_initial = ? THEN
                        "?"
                      ELSE
                        B2rtb_flddef.rtb_initial) FORMAT "x(70)" SKIP.

      IF B2rtb_flddef.rtb_initial-sa <> Brtb_flddef.rtb_initial-sa THEN
        PUT STREAM Sout
          "Init SA: " (IF Brtb_flddef.rtb_initial-sa = ? THEN
                         "?" 
                       ELSE
                          Brtb_flddef.rtb_initial-sa) 
          + " , " +  (IF B2rtb_flddef.rtb_initial-sa = ? THEN
                        "?"
                      ELSE
                        B2rtb_flddef.rtb_initial-sa) FORMAT "x(70)" SKIP.

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_view-as,Brtb_flddef.rtb_view-as) THEN DO:
        IF Brtb_flddef.rtb_view-as = ? THEN
          PUT STREAM Sout "View As: ?" SKIP.
        ELSE DO Mi = 1 TO LENGTH(Brtb_flddef.rtb_view-as) BY 63: 
          PUT STREAM Sout
            (IF Mi = 1 THEN "View As: " ELSE "         ")
            + SUBSTRING(Brtb_flddef.rtb_view-as,Mi,63) FORMAT "x(79)" SKIP.
        END.

        IF B2rtb_flddef.rtb_view-as = ? THEN
          PUT STREAM Sout "  ,   :  ?" SKIP. 
        ELSE DO Mi = 1 TO LENGTH(B2rtb_flddef.rtb_view-as) BY 63:
          PUT STREAM Sout
            (IF Mi = 1 THEN "    ,  : " ELSE "         ")
            + SUBSTRING(B2rtb_flddef.rtb_view-as,Mi,63) FORMAT "x(79)" SKIP.
        END.
      END. /* view-as */

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_valexp,Brtb_flddef.rtb_valexp) THEN DO:
        IF Brtb_flddef.rtb_valexp = ? THEN
          PUT STREAM Sout " Valexp: ?" SKIP.
        ELSE DO Mi = 1 TO LENGTH(Brtb_flddef.rtb_valexp) BY 63: 
          PUT STREAM Sout
            (IF Mi = 1 THEN " Valexp: " ELSE "         ")
            + SUBSTRING(Brtb_flddef.rtb_valexp,Mi,63) FORMAT "x(79)" SKIP.
        END.

        IF B2rtb_flddef.rtb_valexp = ? THEN
          PUT STREAM Sout "  ,   :  ?" SKIP. 
        ELSE DO Mi = 1 TO LENGTH(B2rtb_flddef.rtb_valexp) BY 63:
          PUT STREAM Sout
            (IF Mi = 1 THEN "    ,  : " ELSE "         ")
            + SUBSTRING(B2rtb_flddef.rtb_valexp,Mi,63) FORMAT "x(79)" SKIP.
        END.
      END. /* valexp */

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_valmsg, Brtb_flddef.rtb_valmsg) THEN
        PUT STREAM Sout
          " Valmsg: " Brtb_flddef.rtb_valmsg FORMAT "x(70)" SKIP
          "   ,   : " B2rtb_flddef.rtb_valmsg FORMAT "x(70)" SKIP.

      IF B2rtb_flddef.rtb_valmsg-sa <> Brtb_flddef.rtb_valmsg-sa THEN
        PUT STREAM Sout
          "Valmsg SA: " (IF Brtb_flddef.rtb_valmsg-sa = ? THEN
                           "?" 
                         ELSE
                           Brtb_flddef.rtb_valmsg-sa) 
          + " , " +  (IF B2rtb_flddef.rtb_valmsg-sa = ? THEN
                        "?"
                      ELSE
                        B2rtb_flddef.rtb_valmsg-sa) FORMAT "x(70)" SKIP.

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_help,Brtb_flddef.rtb_help) THEN
        PUT STREAM Sout
          "   Help: " Brtb_flddef.rtb_help SKIP
          "   ,   : " B2rtb_flddef.rtb_help SKIP.

      IF B2rtb_flddef.rtb_help-sa <> Brtb_flddef.rtb_help-sa THEN
        PUT STREAM Sout
          "Help SA: " (IF Brtb_flddef.rtb_help-sa = ? THEN
                         "?" 
                       ELSE
                         Brtb_flddef.rtb_help-sa) 
          + " , " +  (IF B2rtb_flddef.rtb_help-sa = ? THEN
                        "?"
                      ELSE
                        B2rtb_flddef.rtb_help-sa) FORMAT "x(70)" SKIP.

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,B2rtb_flddef.rtb_desc,Brtb_flddef.rtb_desc) THEN
        PUT STREAM Sout
          "   Desc: " + SUBSTRING(Brtb_flddef.rtb_desc,1,70) FORMAT "x(79)" SKIP
          "       : " + SUBSTRING(Brtb_flddef.rtb_desc,71,70) FORMAT "x(79)" SKIP
          "   ,   : " + SUBSTRING(B2rtb_flddef.rtb_desc,1,70) FORMAT "x(79)" SKIP
          "       : " + SUBSTRING(B2rtb_flddef.rtb_desc,71,70) FORMAT "x(79)" SKIP.

       Mmatch = no.

    END. /* difference found */

  END. /* $RPT-BLOCK */

  IF Mmatch AND (FRAME f-shorthdr:VISIBLE OR Mdirect) THEN
    PUT STREAM Sout UNFORMATTED
      "No Changes" SKIP(1).

  HIDE STREAM Sout FRAME f-shorthdr.

  IF Mdirect THEN DO:
    {rtb/i/rtb_rptftr.i &STREAM=Sout}
  END.

  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_unapplied_file_changes) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_unapplied_file_changes Procedure 
PROCEDURE rtb_unapplied_file_changes :
/*------------------------------------------------------------------------------
  Purpose:     Print unapplied file changes
  Parameters:  INPUT Pwspace-id   - Workspace ID
               INPUT PfielObj     - PFILE object name.
  Notes:       Formerly rtb_ckfi.p
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PfileObj   AS CHARACTER   NO-UNDO.

  DEFINE VARIABLE Merror  AS CHARACTER   NO-UNDO INITIAL "".
  DEFINE VARIABLE Mflags  AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mtemp   AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mi      AS INTEGER     NO-UNDO.
  DEFINE VARIABLE Mdirect AS LOGICAL     NO-UNDO INITIAL NO.
  DEFINE VARIABLE Mmatch  AS LOGICAL     NO-UNDO INITIAL YES.

  DEFINE BUFFER Brtb_field    FOR rtb.rtb_field.
  DEFINE BUFFER B2rtb_field   FOR rtb.rtb_field.
  DEFINE BUFFER Brtb_file     FOR rtb.rtb_file.
  DEFINE BUFFER B2rtb_file    FOR rtb.rtb_file.
  DEFINE BUFFER Brtb_filtrig  FOR rtb.rtb_filtrig.
  DEFINE BUFFER B2rtb_filtrig FOR rtb.rtb_filtrig.
  DEFINE BUFFER Brtb_fldtrig  FOR rtb.rtb_fldtrig.
  DEFINE BUFFER B2rtb_fldtrig FOR rtb.rtb_fldtrig.
  DEFINE BUFFER Brtb_idxfld   FOR rtb.rtb_idxfld.
  DEFINE BUFFER B2rtb_idxfld  FOR rtb.rtb_idxfld.
  DEFINE BUFFER Brtb_index    FOR rtb.rtb_index.
  DEFINE BUFFER B2rtb_index   FOR rtb.rtb_index.
  DEFINE BUFFER Brtb_object   FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_ver      FOR rtb.rtb_ver.

  FORM
    Mflags                      FORMAT "x(5)"  COLUMN-LABEL "Flags"
    Brtb_index.rtb_index-name   FORMAT "x(29)" COLUMN-LABEL "Index Name"
    Brtb_idxfld.rtb_ascending   FORMAT "+/- "  COLUMN-LABEL "Fi" SPACE(0)
    Brtb_field.local-field-name FORMAT "x(28)" COLUMN-LABEL "eld Name"
    WITH FRAME f-index 
    NO-BOX DOWN USE-TEXT STREAM-IO.

  MfirstPfilePrint = LINE-COUNTER(Sout) > 5.

  rpt-block:
  DO:
    FIND Brtb_object
      WHERE Brtb_object.wspace-id = Pwspace-id
        AND Brtb_object.obj-type  = "PFILE":U
        AND Brtb_object.object    = PfileObj NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_object THEN DO:
      Merror = SUBSTITUTE("Object record for PFILE '&1' not available.",PfileObj).
      LEAVE rpt-block.
    END.

    FIND Brtb_ver OF Brtb_object NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_ver THEN DO:
      Merror = SUBSTITUTE("Version record for PFILE '&1' not available.",PfileObj).
      LEAVE rpt-block.
    END.

    FIND FIRST Brtb_file OF Brtb_ver NO-LOCK.

    {rtb/i/rtb_rpthdr.i 
       &title="""Unapplied Changes in PFILE object (old, new)"""
       &criteria="""Object:"" Brtb_file.object SKIP"
       &center-criteria=NO
       &frame-no=1
       &STREAM=Sout}

    FORM HEADER
      SUBSTITUTE("Unapplied Changes in PFILE &1 (old, new):",Brtb_file.object)
      FORMAT "x(70)"
      SKIP(1)
      WITH FRAME f-shorthdr
      NO-BOX NO-LABELS PAGE-TOP STREAM-IO.

    IF INDEX(PROGRAM-NAME(2),"rtb_unapplied_changes_report":U) > 0 THEN DO:
      HIDE STREAM Sout FRAME header-frame1.
      VIEW STREAM Sout FRAME f-shorthdr.
    END.
    ELSE
      Mdirect = YES.

    IF Brtb_ver.obj-status = "C" AND Brtb_object.update-status = "C" THEN
      LEAVE rpt-block.

    IF Brtb_object.update-status = "N" THEN DO:
      HIDE STREAM Sout FRAME f-shorthdr.
      PUT STREAM Sout
        SKIP(1)
        "New PFILE object:" SKIP
        "     Object: " Brtb_file.object FORMAT "x(32)" SKIP
        "     Hidden: " Brtb_file.rtb_hidden SKIP
        "     Frozen: " Brtb_file.rtb_frozen SKIP
        "       Dump: " Brtb_file.rtb_dump-name SKIP
        "Replication: " Brtb_file.rtb_fil-misc2[6] SKIP
        "      Label: " Brtb_file.rtb_file-label  " " Brtb_file.rtb_file-label-sa SKIP.

      DO Mi = 0 TO 3:
         Mtemp = SUBSTRING(Brtb_file.rtb_desc,Mi * 63 + 1,63).
         IF Mi = 0 AND Mtemp = ? THEN Mtemp = "?".
         IF (Mtemp = ? OR (TRIM(Mtemp) = "" AND Mi > 0)) THEN NEXT.
         PUT STREAM Sout (IF Mi = 0 THEN "   Desc: " ELSE FILL(" ",9)) 
                         + Mtemp FORMAT "x(80)" SKIP.
      END.
      DO Mi = 0 TO 3:
         Mtemp = SUBSTRING(Brtb_file.rtb_valexp,Mi * 63 + 1,63).
         IF Mi = 0 AND Mtemp = ? THEN Mtemp = "?".
         IF (Mtemp = ? OR (TRIM(Mtemp) = "" AND Mi > 0)) THEN NEXT.
         PUT STREAM Sout (IF Mi = 0 THEN " Valexp: " ELSE FILL(" ",9))
                          + Mtemp FORMAT "x(80)" SKIP.
      END.
      PUT STREAM Sout " Valmsg: " + Brtb_file.rtb_valmsg FORMAT "x(75)" 
          " " Brtb_file.rtb_valmsg-sa SKIP.

      PUT STREAM Sout SKIP(1).

      IF CAN-FIND(FIRST Brtb_filtrig OF Brtb_file) THEN DO:
        FOR EACH Brtb_filtrig OF Brtb_file NO-LOCK:
           DISPLAY STREAM Sout
             SPACE(3)
             Brtb_filtrig.rtb_event                    COLUMN-LABEL "File Trigger Event"
             Brtb_filtrig.rtb_override FORMAT "Yes/No" COLUMN-LABEL "Override?"
             Brtb_filtrig.rtb_proc-name                COLUMN-LABEL "Proc-name"
             WITH NO-BOX NO-ATTR-SPACE STREAM-IO.
        END.
        PUT STREAM Sout SKIP(1).
      END.

      FOR EACH Brtb_field OF Brtb_file NO-LOCK: /* current image */
        DISPLAY STREAM Sout
          SPACE(1)
          Brtb_field.local-field-name                  COLUMN-LABEL "Local Fieldname"
          Brtb_field.rtb_order        FORMAT ">>>>9"   COLUMN-LABEL "Ord."
          Brtb_field.rtb_mandatory    FORMAT "yes/no"  COLUMN-LABEL "Mand"
          Brtb_field.flddef-object                     COLUMN-LABEL "Field Object"
          WITH NO-BOX NO-ATTR-SPACE STREAM-IO.
        FIND FIRST Brtb_fldtrig OF Brtb_field NO-LOCK NO-ERROR.
        IF AVAILABLE Brtb_fldtrig THEN
          DISPLAY STREAM Sout 
            SPACE(35)
            Brtb_fldtrig.rtb_proc-name LABEL "Trigger"
            WITH FRAME frame-fldtrig NO-BOX NO-ATTR-SPACE SIDE-LABELS STREAM-IO.
      END. /* each brtb_field */

      IF (LINE-COUNTER(Sout) + 6) > PAGE-SIZE(Sout) THEN
        PAGE STREAM Sout.
      ELSE
        PUT STREAM Sout SKIP(1).

      Mflags = "".

      PUT STREAM Sout
        SKIP(1)
        "INDEX FLAGS: P)rimary, U)nique, W)ord, I)nactive, A)bbreviated, + Asc, - Desc" SKIP
        SKIP(1).

      FOR EACH Brtb_index OF Brtb_file NO-LOCK: /* current image */
        FIND LAST Brtb_idxfld OF Brtb_index NO-LOCK NO-ERROR.
        IF Brtb_index.index-id = Brtb_file.rtb_prime-index-id THEN
          Mflags = "P".
        IF Brtb_index.rtb_unique THEN
          Mflags = Mflags + "U".
        IF Brtb_index.rtb_wordidx = 1 THEN
          Mflags = Mflags + "W".
        IF NOT Brtb_index.rtb_active THEN
          Mflags = Mflags + "I".
        IF AVAILABLE Brtb_idxfld AND Brtb_idxfld.rtb_abbreviate THEN
          Mflags = Mflags + "A".

        DISPLAY STREAM Sout
          Mflags
          Brtb_index.rtb_index-name
          WITH FRAME f-index.

        FOR EACH Brtb_idxfld OF Brtb_index NO-LOCK:
          FIND FIRST Brtb_field OF Brtb_file 
            WHERE Brtb_field.field-id = Brtb_idxfld.field-id NO-LOCK.
          
          DISPLAY STREAM Sout
              Brtb_idxfld.rtb_ascending
              Brtb_field.local-field-name
              WITH FRAME f-index.
          DOWN STREAM Sout WITH FRAME f-index.

        END. /* EACH Brtb_idxfld */

        DO Mi = 0 TO 3:
          Mtemp = SUBSTRING(Brtb_index.rtb_desc,Mi * 63 + 1,63).
          IF Mi = 0 AND Mtemp = ? THEN Mtemp = "?".
          IF (Mtemp = ? OR (TRIM(Mtemp) = "" AND Mi > 0)) THEN NEXT.
          PUT STREAM Sout (IF Mi = 0 THEN "      Desc:" ELSE FILL(" ",12))
                           + Mtemp FORMAT "x(80)" SKIP.
        END.

        PUT STREAM Sout SKIP(1).

      END. /* of EACH Brtb_index */

      LEAVE rpt-block.

    END. /* update-status = n */

    /*
     The following section is not executed if the object had
     a update-status of "N"! or the object is not work in process
    */
    IF Brtb_ver.obj-status <> "W" THEN LEAVE rpt-block.

    /*
     NOTE: The Brtb_file buffer contains the edit image of the file 
           The B2tb_file buffer contains the commited image of the file
    */
    FIND B2rtb_file /* committed image buffer */  
         WHERE B2rtb_file.pmod     = Brtb_object.committed-pmod
           AND B2rtb_file.obj-type = Brtb_object.obj-type
           AND B2rtb_file.object   = Brtb_object.object
           AND B2rtb_file.version  = Brtb_object.committed-version
           NO-LOCK.

    IF NOT(DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_desc,B2rtb_file.rtb_desc)
           AND DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_valexp,B2rtb_file.rtb_valexp)
           AND DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_valmsg,B2rtb_file.rtb_valmsg)
           AND Brtb_file.rtb_hidden = B2rtb_file.rtb_hidden
           AND Brtb_file.rtb_frozen = B2rtb_file.rtb_frozen
           AND DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_dump-name,B2rtb_file.rtb_dump-name)
           AND DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_fil-misc2[6],B2rtb_file.rtb_fil-misc2[6])
           AND Brtb_file.rtb_prime-index-id = B2rtb_file.rtb_prime-index-id 
           AND Brtb_file.rtb_valmsg-sa = B2rtb_file.rtb_valmsg-sa
           AND DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_file-label,B2rtb_file.rtb_file-label)
           AND Brtb_file.rtb_file-label-sa = B2rtb_file.rtb_file-label-sa) THEN DO:

      IF MfirstPfilePrint THEN DO:
        PUT STREAM Sout SKIP(1)
                   SUBSTITUTE("Unapplied Changes in PFILE &1 (old, new):",B2rtb_file.object)
                   FORMAT "x(70)"
                   SKIP(1).
        MfirstPfilePrint = NO.
      END.

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_file-label,
                               B2rtb_file.rtb_file-label) THEN
        PUT STREAM Sout "  Label: " B2rtb_file.rtb_file-label + " , "
                        + Brtb_file.rtb_file-label FORMAT "x(70)" SKIP.

      IF Brtb_file.rtb_file-label-sa <> B2rtb_file.rtb_file-label-sa THEN
         PUT STREAM Sout "Label-sa:" B2rtb_file.rtb_file-label-sa " , "
                          Brtb_file.rtb_file-label-sa SKIP.

      IF Brtb_file.rtb_hidden <> B2rtb_file.rtb_hidden THEN 
        PUT STREAM Sout " Hidden: " B2rtb_file.rtb_hidden " , "
                        Brtb_file.rtb_hidden SKIP.

      IF Brtb_file.rtb_frozen <> B2rtb_file.rtb_frozen THEN
        PUT STREAM Sout " Frozen: " B2rtb_file.rtb_frozen " , " Brtb_file.rtb_frozen SKIP.

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_dump-name,B2rtb_file.rtb_dump-name) THEN
        PUT STREAM Sout "   Dump: "  B2rtb_file.rtb_dump-name
                         + " , " + Brtb_file.rtb_dump-name FORMAT "x(80)" SKIP.

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_fil-misc2[6], B2rtb_file.rtb_fil-misc2[6]) THEN
        PUT STREAM Sout
          "Replic.: "
          ((IF Brtb_file.rtb_fil-misc2[6] = ? THEN "?" ELSE B2rtb_file.rtb_fil-misc2[6])
            + " , " 
            + (IF Brtb_file.rtb_fil-misc2[6] = ? THEN "?" ELSE Brtb_file.rtb_fil-misc2[6]))  FORMAT "x(60)" SKIP.

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_desc,B2rtb_file.rtb_desc) THEN DO:
        DO Mi = 0 TO 3:
          Mtemp = SUBSTRING(B2rtb_file.rtb_desc,Mi * 63 + 1,63).
          IF Mi = 0 AND Mtemp = ? THEN Mtemp = "?".
          IF (Mtemp = ? OR (TRIM(Mtemp) = "" AND Mi > 0)) THEN NEXT.
          PUT STREAM Sout (IF Mi = 0 THEN "   Desc: " ELSE FILL(" ",9)) + Mtemp FORMAT "x(80)" SKIP.
        END.
        DO Mi = 0 TO 3:
          Mtemp = SUBSTRING(Brtb_file.rtb_desc,Mi * 63 + 1,63).
          IF Mi = 0 AND Mtemp = ? THEN Mtemp = "?".
          IF (Mtemp = ? OR (TRIM(Mtemp) = "" AND Mi > 0)) THEN NEXT.
          PUT (IF Mi = 0 THEN "   ,   : " ELSE FILL(" ",9)) + Mtemp FORMAT "x(80)" SKIP.
        END.
      END. /* rtb_desc */

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_valexp,B2rtb_file.rtb_valexp) THEN DO:
        DO Mi = 0 TO 3:
          Mtemp = SUBSTRING(B2rtb_file.rtb_valexp,Mi * 63 + 1,63).
          IF Mi = 0 AND Mtemp = ? THEN Mtemp = "?".
          IF (Mtemp = ? OR (TRIM(Mtemp) = "" AND Mi > 0)) THEN NEXT.
          PUT STREAM Sout (IF Mi = 0 THEN " Valexp: " ELSE FILL(" ",9)) + Mtemp FORMAT "x(80)" SKIP.
        END.
        DO Mi = 0 TO 3:
          Mtemp = SUBSTRING(Brtb_file.rtb_valexp,Mi * 63 + 1,63).
          IF Mi = 0 AND Mtemp = ? THEN Mtemp = "?".
          IF (Mtemp = ? OR (TRIM(Mtemp) = "" AND Mi > 0)) THEN NEXT.
          PUT STREAM Sout (IF Mi = 0 THEN "     , : " ELSE FILL(" ",9)) + Mtemp FORMAT "x(80)" SKIP.
        END.
      END. /* rtb_valexp */

      IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':u,Brtb_file.rtb_valmsg, B2rtb_file.rtb_valmsg) THEN
        PUT STREAM Sout
          " Valmsg: " + B2rtb_file.rtb_valmsg FORMAT "x(80)" SKIP
          "   ,   : " + Brtb_file.rtb_valmsg  FORMAT "x(80)" SKIP.

      IF Brtb_file.rtb_valmsg-sa <> B2rtb_file.rtb_valmsg-sa THEN
        PUT STREAM Sout
          " Valmsg-sa: " B2rtb_file.rtb_valmsg-sa " , " Brtb_file.rtb_valmsg-sa SKIP.

      IF Brtb_file.rtb_prime-index-id <> B2rtb_file.rtb_prime-index-id THEN DO:
        FIND FIRST Brtb_index OF Brtb_file 
          WHERE Brtb_index.index-id = Brtb_file.rtb_prime-index-id NO-LOCK.
        FIND FIRST B2rtb_index OF B2rtb_file
          WHERE B2rtb_index.index-id = B2rtb_file.rtb_prime-index-id NO-LOCK.
        PUT STREAM Sout
           " PriIdx: " + B2rtb_index.rtb_index-name + " , " 
                       + Brtb_index.rtb_index-name FORMAT "x(80)" SKIP. 
      END.
      Mmatch = NO.
    END. /* compare attributes */

    /*
     Look for field changes
    */
    RUN rtb_check_file_field_changes (OUTPUT Mmatch,
                                      BUFFER Brtb_file,
                                      BUFFER B2rtb_file).
    /*
     Look for index changes
    */
    RUN rtb_check_file_index_changes (OUTPUT Mmatch,
                                      BUFFER Brtb_file,
                                      BUFFER B2rtb_file).

    FOR EACH B2rtb_filtrig OF B2rtb_file NO-LOCK:
      FIND FIRST Brtb_filtrig OF Brtb_file
           WHERE Brtb_filtrig.rtb_event  = B2rtb_filtrig.rtb_event NO-LOCK NO-ERROR.
      IF NOT AVAILABLE Brtb_filtrig OR
         NOT(Brtb_filtrig.rtb_override = B2rtb_filtrig.rtb_override AND
         DYNAMIC-FUNCTION('fnRtbCompareStrings':U,Brtb_filtrig.rtb_proc-name,B2rtb_filtrig.rtb_proc-name) AND
         Brtb_filtrig.rtb_trig-crc = B2rtb_filtrig.rtb_trig-crc) THEN DO:

        IF MfirstPfilePrint THEN DO:
          PUT STREAM Sout SKIP(1)
                     SUBSTITUTE("Unapplied Changes in PFILE &1 (old, new):",B2rtb_file.object)
                     FORMAT "x(70)"
                     SKIP(1).
          MfirstPfilePrint = NO. 
        END.

        IF NOT AVAILABLE Brtb_filtrig THEN
          PUT STREAM Sout "  File Trigger: " B2rtb_filtrig.rtb_event + " DELETED " 
                          FORMAT "x(50)" SKIP.
        ELSE DO:
          PUT STREAM Sout "  File Trigger: " B2rtb_filtrig.rtb_event SKIP.

          IF Brtb_filtrig.rtb_override <> B2rtb_filtrig.rtb_override THEN 
            PUT "   Override: " B2rtb_filtrig.rtb_override " , "
                                Brtb_filtrig.rtb_override SKIP.

          IF NOT DYNAMIC-FUNCTION('fnRtbCompareStrings':U,Brtb_filtrig.rtb_proc-name,B2rtb_filtrig.rtb_proc-name) THEN 
            PUT STREAM Sout "  Proc Name: " B2rtb_filtrig.rtb_proc-name FORMAT "x(32)" " , "
                                            Brtb_filtrig.rtb_proc-name FORMAT "x(32)" SKIP.

          IF Brtb_filtrig.rtb_trig-crc <> B2rtb_filtrig.rtb_trig-crc THEN 
            PUT "        CRC: " B2rtb_filtrig.rtb_trig-crc FORMAT ">>>>>9" " , "
                                Brtb_filtrig.rtb_trig-crc FORMAT ">>>>>9" SKIP.
        END. /* else do */
        Mmatch = NO.
        LEAVE.
      END.
    END. /* of EACH B2rtb_filtrig ... */

    FOR EACH Brtb_filtrig OF Brtb_file NO-LOCK:
      IF NOT CAN-FIND(FIRST B2rtb_filtrig OF B2rtb_file WHERE
                            B2rtb_filtrig.rtb_event = Brtb_filtrig.rtb_event) THEN DO:
        IF MfirstPfilePrint THEN DO:
          PUT STREAM Sout SKIP(1)
                          SUBSTITUTE("Unapplied Changes in PFILE &1 (old, new):",B2rtb_file.object)
                          FORMAT "x(70)"
                          SKIP(1).
          MfirstPfilePrint = NO. 
        END.

        PUT STREAM Sout "  File Trigger: " Brtb_filtrig.rtb_event + " ADDED " 
                        FORMAT "x(50)" SKIP.
        Mmatch = NO.
        LEAVE.
      END.
    END. /* of EACH Brtb_filtrig ... */

  END. /* rpt-block */

  IF Mmatch AND (FRAME f-shorthdr:VISIBLE OR Mdirect) THEN
    PUT STREAM Sout UNFORMATTED
      "No Changes" SKIP(1).

  HIDE STREAM Sout FRAME f-shorthdr.

  IF Mdirect THEN DO:
    {rtb/i/rtb_rptftr.i &STREAM=Sout}
  END.

  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

