&ANALYZE-SUSPEND _VERSION-NUMBER UIB_v9r12
&ANALYZE-RESUME
&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _DEFINITIONS Procedure 
/*------------------------------------------------------------------------
    File        : rtb_rpt_wspace.p
    Description : Roundtable Workspace reports procedure library
    Notes       : Contents:
                    * Workspace Report
                    * Workspace Changes Report
                    * Workspace Deployment Site Report
                    * Workspace Differences Report
                    * Workspace Event History Report
                    * Workspace Import Report
                    * Workspace Release Report
                    * Versions in Workspace Report
  ----------------------------------------------------------------------*/

{rtb/i/rtb_rptinit.i}


DEFINE TEMP-TABLE Ttask NO-UNDO
  FIELD task-num LIKE rtb.rtb_task.task-num
  INDEX task-num task-num.

DEFINE TEMP-TABLE Thist NO-UNDO
  FIELD action        AS CHARACTER
  FIELD obj-type      AS CHARACTER LABEL "Type":U    FORMAT "x(6)"
  FIELD object        AS CHARACTER LABEL "Object":U  FORMAT "x(35)"
  FIELD object-guid   AS CHARACTER LABEL "GUID":U    FORMAT "x(25)"
  FIELD version       AS INTEGER   LABEL "Ver":U     FORMAT "999999"
  FIELD pmod          AS CHARACTER LABEL "Pmodule":U FORMAT "x(20)"
  FIELD sub-type      AS CHARACTER LABEL "Subtype":U FORMAT "x(12)"
  FIELD task-num      AS INTEGER   LABEL "Task#"     FORMAT ">>>>>>>>9"
  FIELD release-num   AS INTEGER   LABEL "Release"   FORMAT ">>>>>>9"
  FIELD event-counter AS INTEGER   LABEL "Event#"    FORMAT ">>,>>>,>>9"
  FIELD upd-notes     AS CHARACTER
  INDEX action   action
  INDEX obj-type obj-type
  INDEX object object
  INDEX version version DESCENDING.

DEFINE TEMP-TABLE Tver NO-UNDO
  FIELD ver-rowid    AS ROWID
  FIELD ver-order    AS INTEGER
  FIELD hist-rowid   AS ROWID
  FIELD pmod         AS CHARACTER
  FIELD prev-version AS INTEGER
  INDEX Tver-ind1 IS UNIQUE PRIMARY ver-order
  INDEX Tver-ind2 ver-rowid.

DEFINE TEMP-TABLE Tver-notes NO-UNDO
  FIELD upd-notes LIKE rtb.rtb_ver.upd-notes.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


&ANALYZE-SUSPEND _UIB-PREPROCESSOR-BLOCK 

/* ********************  Preprocessor Definitions  ******************** */

&Scoped-define PROCEDURE-TYPE Procedure
&Scoped-define DB-AWARE no



/* _UIB-PREPROCESSOR-BLOCK-END */
&ANALYZE-RESUME



/* *********************** Procedure Settings ************************ */

&ANALYZE-SUSPEND _PROCEDURE-SETTINGS
/* Settings for THIS-PROCEDURE
   Type: Procedure
   Allow: 
   Frames: 0
   Add Fields to: Neither
   Other Settings: CODE-ONLY COMPILE
 */
&ANALYZE-RESUME _END-PROCEDURE-SETTINGS

/* *************************  Create Window  ************************** */

&ANALYZE-SUSPEND _CREATE-WINDOW
/* DESIGN Window definition (used by the UIB) 
  CREATE WINDOW Procedure ASSIGN
         HEIGHT             = 20.91
         WIDTH              = 60.
/* END WINDOW DEFINITION */
                                                                        */
&ANALYZE-RESUME

 


&ANALYZE-SUSPEND _UIB-CODE-BLOCK _CUSTOM _MAIN-BLOCK Procedure 


/* ***************************  Main Block  *************************** */

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME


/* **********************  Internal Procedures  *********************** */

&IF DEFINED(EXCLUDE-print_task_info) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE print_task_info Procedure 
PROCEDURE print_task_info PRIVATE :
/*------------------------------------------------------------------------------
  Purpose:     
  Parameters:  <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Pdetail   AS LOGICAL    NO-UNDO.

  DEFINE VARIABLE MlineOut     AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mi           AS INTEGER     NO-UNDO.

  DEFINE BUFFER Brtb_task FOR rtb.rtb_task.

  FORM
    Brtb_task.task-num      FORMAT ">>>>>>>>9"
    Brtb_task.summary       FORMAT "x(70)" COLUMN-LABEL "Summary"
    Brtb_task.user-task-ref FORMAT "x(30)" COLUMN-LABEL "User Ref" 
    WITH FRAME task-detail-frame DOWN NO-BOX STREAM-IO WIDTH 120.
  
  FORM
    Brtb_task.task-num      FORMAT ">>>>>>>>9"
    Brtb_task.summary       FORMAT "x(70)" COLUMN-LABEL "Summary"
    Brtb_task.user-task-ref FORMAT "x(30)" COLUMN-LABEL "User Ref" 
    WITH FRAME task-summary-frame DOWN NO-BOX STREAM-IO WIDTH 120.
 
  FOR EACH Ttask NO-LOCK:

    FIND FIRST Brtb_task
      WHERE Brtb_task.task-num = Ttask.task-num NO-LOCK NO-ERROR.

    IF AVAILABLE Brtb_task THEN DO:
      
      /*
       Print detail
      */
      IF Pdetail THEN DO:
        
        DISPLAY STREAM Sout
          Brtb_task.task-num
          Brtb_task.summary
          Brtb_task.user-task-ref
          WITH FRAME task-detail-frame.

        DO Mi = 1 TO 15:
          IF Brtb_task.description[Mi] <> "" THEN DO:
            MlineOut = DYNAMIC-FUNCTION('fnRtbBreakLines':U, Brtb_task.description[Mi],100).
            DO WHILE MlineOut <> ?:
              PUT STREAM Sout UNFORMATTED
                SPACE(10) MLineOut SKIP.
              MlineOut = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,100).
            END.
          END.
        END.  /* do mi = 1 to 15 */        

        PUT STREAM Sout UNFORMATTED SKIP(1).
        DOWN WITH FRAME task-detail-frame.

      END.

      /*
       Summary only
      */
      ELSE DO:
        DISPLAY STREAM Sout
          Brtb_task.task-num
          Brtb_task.SUMMARY
          Brtb_task.user-task-ref
          WITH FRAME task-summary-frame.
        DOWN WITH FRAME task-summary-frame.
      END.

    END. /* available brtb_task */

  END. /* for each Ttask */

  EMPTY TEMP-TABLE Ttask.

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbVersionsInWorkspace) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbVersionsInWorkspace Procedure 
PROCEDURE rptRtbVersionsInWorkspace :
/*------------------------------------------------------------------------------
  Purpose:     Version in workspace report wrapper
  Parameters:  INPUT
                 Pwspace-id     = Workspace ID
                 Pobj-type      = Object type
                 Pobject        = Object name
                 Pversion       = Object version (used by import reporting)
                 PverNotes      = TRUE to show version notes
                 Ppmodlist      = CSV list of product modules
                 TABLE-HANDLE   = Filespecs temp-table
               OUTPUT
                 <none>
  Notes:
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pobj-type  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pobject    AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PverNotes  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PpmodList  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_versions_in_workspace_report (INPUT Pwspace-id,
                                        INPUT Pobj-type,
                                        INPUT Pobject,
                                        INPUT Pversion,
                                        INPUT PverNotes,
                                        INPUT PpmodList).

  OUTPUT STREAM Sout CLOSE.  

  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbWorkspace) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbWorkspace Procedure 
PROCEDURE rptRtbWorkspace :
/*------------------------------------------------------------------------------
  Purpose:     Workspace report wrapper
  Parameters:  INPUT
                 Pwspace-id        = Workspace ID
                 PshowObjects      = TRUE to show objects in workspace
                 PshowDetails      = TRUE to show object details
                 PhowVerNotes      = TRUE to show object version notes
                 TABLE-HANDLE PhTT = Filespecs temp-table
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id    AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PshowObjects  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PshowDetails  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PshowVerNotes AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_workspace_report (INPUT Pwspace-id,
                            INPUT PshowObjects,
                            INPUT PshowDetails,
                            INPUT PshowVerNotes).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbWorkspaceChanges) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbWorkspaceChanges Procedure 
PROCEDURE rptRtbWorkspaceChanges :
/*------------------------------------------------------------------------------
  Purpose:     Workspace changes report wrapper
  Parameters:  INPUT
                 Pwspace-id        = Workspace ID
                 PbegEvent         = Beginning event number    
                 PendEvent         = Ending event number   
                 PobjDetails       = TRUE to print object detals    
                 PobjVersionNotes  = TRUE to print version notes
                 PlatestVerOnly    = TRUE to show only latest version
                 TABLE-HANDLE Phtt = Filespecs temp-table
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id       AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PbegEvent        AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PendEvent        AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PobjDetails      AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjVersionNotes AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PlatestVerOnly   AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_changes_report (INPUT Pwspace-id,
                          INPUT PbegEvent,
                          INPUT PendEvent,
                          INPUT PobjDetails,
                          INPUT PobjVersionNotes,
                          INPUT PlatestVerOnly).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbWorkspaceDeploySite) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbWorkspaceDeploySite Procedure 
PROCEDURE rptRtbWorkspaceDeploySite :
/*------------------------------------------------------------------------------
  Purpose:     Workspace deployment site(s) report wrapper
  Parameters:  INPUT
                 Pwspace-id        = Workspace ID
                 Psite-code        = Deployment site code or * for all
                 PactiveOnly       = TRUE to show only active sites
                 PshowNotes        = TRUE to show site notes
                 PlastOnly         = TRUE to show last deployment only
                 TABLE-HANDLE PhTT = Filespecs tamp-table
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id   AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Psite-code   AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PactiveOnly  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PshowNotes   AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PlastDepOnly AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_site_report (INPUT Pwspace-id,
                       INPUT Psite-code,
                       INPUT PactiveOnly,
                       INPUT PshowNotes,
                       INPUT PlastDepOnly).
                       
  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbWorkspaceDifferences) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbWorkspaceDifferences Procedure 
PROCEDURE rptRtbWorkspaceDifferences :
/*------------------------------------------------------------------------------
  Purpose:     Workspace differences report wrapper
  Parameters:  INPUT
                 Pwspace-id-1      = First workspace ID
                 Pwspace-id-2      = Second workspace ID
                 TABLE-HANDLE Phtt = Filespecs temp-table
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id-1 AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pwspace-id-2 AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_differences_report (INPUT Pwspace-id-1,
                              INPUT PWspace-id-2).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbWorkspaceEvents) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbWorkspaceEvents Procedure 
PROCEDURE rptRtbWorkspaceEvents :
/*------------------------------------------------------------------------------
  Purpose:     Workspace event history report wrapper
  Parameters:  INPUT
                 Pwspace-id = Workspace ID
                 Puser-id   = User ID or "All"
                 PbegDate   = Start date or ? (for all dates)
                 PendDate   = End date or ? (for all date)
                 PbegEvent  = Starting event or 0 (for all events)
                 PendEvent  = Ending event or 0 (for all events) 
                 Ppmod      = Product module or MATCHES pattern
                 Pobj-type  = Object type or MATCHES pattern
                 Pobject    = Object name or MATCHES pattern
                 TABLE-HANDLE PhTT = Filespecs temp-table
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Puser-id   AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PbegDate   AS DATE        NO-UNDO.
  DEFINE INPUT PARAMETER PendDate   AS DATE        NO-UNDO.
  DEFINE INPUT PARAMETER PbegEvent  AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PendEvent  AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pobj-type  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pobject    AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_event_history_report (INPUT Pwspace-id,
                                INPUT Puser-id,
                                INPUT PbegDate,
                                INPUT PendDate,
                                INPUT PbegEvent,
                                INPUT PendEvent,
                                INPUT Ppmod,
                                INPUT Pobj-type,
                                INPUT Pobject).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbWorkspaceImport) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbWorkspaceImport Procedure 
PROCEDURE rptRtbWorkspaceImport :
/*------------------------------------------------------------------------------
  Purpose:     Workspace Import Report wrapper
  Parameters:  INPUT Pwspace-id   - Workspace ID
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_import_report (INPUT Pwspace-id).

  OUTPUT STREAM Sout CLOSE.  

  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rptRtbWorkspaceRelease) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rptRtbWorkspaceRelease Procedure 
PROCEDURE rptRtbWorkspaceRelease :
/*------------------------------------------------------------------------------
  Purpose:     Workspace release report wrapper
  Parameters:  INPUT
                 Pwspace-id        = Workspace ID
                 PbegRelease       = Beginning release #
                 PendRelease       = Ending release #
                 PpageHeaders      = TRUE to print page headers
                 PreleaseInfo      = TRUE to print release info
                 PobjectHeaders    = TRUE to print object headers
                 PobjDetails       = TRUE to print object details
                 PtaskSummary      = TRUE to print task summaries
                 PtaskDetail       = TRUE to print task details
                 PshowVariants     = TRUE to print object variants
                 PsuppressAssigns  = FALSE to print object assignments
                 PlatestVerOnly    = TRUE to print latest object version only
                 PversionInfo      = TRUE to print version info
                 PobjReleaseInfo   = TRUE to print object release info
                 PobjTaskInfo      = TRUE to print object task info
                 PobjVersionNotes  = TRUE to print object version notes
                 TABLE-HANDLE PhTT = Filespecs temp-table
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id       AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PbegRelease      AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PendRelease      AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PpageHeaders     AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PreleaseInfo     AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjectHeaders   AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjDetails      AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PtaskSummary     AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PtaskDetail      AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PshowVariants    AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PsuppressAssigns AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PlatestVerOnly   AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PversionInfo     AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjReleaseInfo  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjTaskInfo     AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjVersionNotes AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER TABLE-HANDLE PhTT.

  RUN rtb_report_initialize (INPUT TABLE-HANDLE PhTT).

  RUN rtb_release_report (INPUT Pwspace-id,
                          INPUT PbegRelease,
                          INPUT PendRelease,
                          INPUT PpageHeaders,
                          INPUT PreleaseInfo,
                          INPUT PobjectHeaders,
                          INPUT PobjDetails,
                          INPUT PtaskSummary,
                          INPUT PtaskDetail,
                          INPUT PshowVariants,
                          INPUT PsuppressAssigns,
                          INPUT PlatestVerOnly,
                          INPUT PversionInfo,
                          INPUT PobjReleaseInfo,
                          INPUT PobjTaskInfo,
                          INPUT PobjVersionNotes).

  IF RETURN-VALUE <> "" THEN 
    PUT STREAM Sout UNFORMATTED "ERROR :" RETURN-VALUE.

  OUTPUT STREAM Sout CLOSE.

  /*
   Return the return value of the above procedure
  */
  RETURN RETURN-VALUE.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_changes_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_changes_report Procedure 
PROCEDURE rtb_changes_report :
/*------------------------------------------------------------------------------
  Purpose:     Print Worspace Changes report
  Parameters:  INPUT
                 Pwspace-id        - Workspace ID
                 PbegEvent         - Beginning event number    
                 PendEvent         - Ending event number   
                 PobjDetails       - TRUE to print object detals    
                 PobjVersionNotes  - TRUE to print version notes
                 PlatestVerOnly    - TRUE to show only latest version
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id       AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PbegEvent        AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PendEvent        AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PobjDetails      AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjVersionNotes AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PlatestVerOnly   AS LOGICAL     NO-UNDO.

  DEFINE VARIABLE Mdescription AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Merror       AS CHARACTER   NO-UNDO INITIAL "".
  DEFINE VARIABLE Mline        AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mobj-type    AS CHARACTER   NO-UNDO INITIAL "".
  DEFINE VARIABLE Mobject      AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mtextlines   AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mi           AS INTEGER     NO-UNDO.

  DEFINE BUFFER Brtb_hist    FOR rtb.rtb_hist.
  DEFINE BUFFER Brtb_release FOR rtb.rtb_release.
  DEFINE BUFFER Brtb_ver     FOR rtb.rtb_ver.
  DEFINE BUFFER Brtb_wscount FOR rtb.rtb_wscount.
  DEFINE BUFFER Brtb_wspace  FOR rtb.rtb_wspace.

  $RPT-BLOCK:
  DO:

    /*
     Validate input parameters
    */
    FIND Brtb_wspace WHERE Brtb_wspace.wspace-id = Pwspace-id NO-LOCK NO-ERROR.
    IF NOT AVAILABLE Brtb_wspace THEN DO:
      Merror = SUBSTITUTE("Workspace '&1' record is not available.",Pwspace-id).
      LEAVE $RPT-BLOCK.
    END.

    IF PendEvent = 0 THEN DO:
      FIND Brtb_wscount WHERE Brtb_wscount.wspace-id = Pwspace-id NO-LOCK NO-ERROR.
      IF AVAILABLE Brtb_wscount THEN
          PendEvent = Brtb_wscount.event-counter.
    END.

    FIND FIRST Brtb_hist
      WHERE Brtb_hist.wspace-id      = Pwspace-id
        AND Brtb_hist.event-counter <= PendEvent
        AND Brtb_hist.event-counter >= PbegEvent
      USE-INDEX rtb_hist02
      NO-LOCK NO-ERROR.

    IF AVAILABLE Brtb_hist THEN
      ASSIGN Mobject = Brtb_hist.object.
    ELSE
      LEAVE $RPT-BLOCK.

    DO WHILE AVAILABLE Brtb_hist
      ON ENDKEY UNDO, LEAVE
      ON ERROR UNDO, LEAVE 
      WITH FRAME object-desc-frame
        STREAM-IO:

      {rtb/i/rtb_rpthdr.i 
          &title="""Workspace Changes Report"""
          &criteria="""  Workspace: "" Brtb_wspace.wspace-id   SKIP
                     ""      Desc.: "" Brtb_wspace.description SKIP
                     ""From Event#: "" TRIM(STRING(PbegEvent))
                     ""  To Event#: "" TRIM(STRING(PendEvent)) SKIP(1)"
          &center-criteria=NO
          &STREAM=Sout}
                          
      IF Mobj-type <> Brtb_hist.obj-type THEN DO:
        PAGE STREAM Sout.
        PUT STREAM Sout UNFORMATTED SKIP(1) TRIM(CAPS(Brtb_hist.obj-type)) " Objects" SKIP. 
        Mobj-type = Brtb_hist.obj-type.
      END.

      Mobject = Brtb_hist.object.

      FIND Brtb_ver
        WHERE Brtb_ver.pmod     = Brtb_hist.pmod
          AND Brtb_ver.obj-type = Brtb_hist.obj-type
          AND Brtb_ver.object   = Brtb_hist.object
          AND Brtb_ver.version  = Brtb_hist.version
        NO-LOCK NO-ERROR.
      IF NOT AVAILABLE Brtb_ver THEN DO:
        Merror = SUBSTITUTE("Version record for &1 '&2' version &3 not available.",
                            Brtb_hist.obj-type,Brtb_hist.object,Brtb_hist.version).
        LEAVE $RPT-BLOCK.
      END.

      RUN rtb_get_object_version_details (Brtb_ver.pmod,
                                          Brtb_ver.obj-type,
                                          Brtb_ver.object,
                                          Brtb_ver.version,
                                          OUTPUT Mdescription,
                                          OUTPUT Mtextlines).
      FIND FIRST Brtb_release
        WHERE Brtb_release.wspace-id      = Pwspace-id
          AND Brtb_release.event-counter <= Brtb_hist.event-counter
        NO-LOCK NO-ERROR.

      DISPLAY STREAM Sout
        Brtb_hist.action
        Brtb_hist.version
        Brtb_hist.pmod           FORMAT "x(70)"
        Brtb_hist.obj-type 
        Brtb_hist.event-counter
        Brtb_release.release-num LABEL "Release#" FORMAT ">>>>9" 
          WHEN AVAILABLE Brtb_release
        SKIP(0)
        (Brtb_hist.object + " / " + Mdescription) FORMAT "x(100)" 
        LABEL "Object / Description"
        WITH FRAME object-desc-frame WIDTH 120
        NO-ATTR-SPACE
        STREAM-IO.

      IF PobjDetails THEN DO Mi = 1 TO 9:
        Mline = SUBSTRING(Mtextlines,(Mi - 1) * 76 + 1, 76).
        IF TRIM(Mline) <> "" THEN
          PUT STREAM Sout Mline FORMAT "x(76)" SKIP.
      END.

      IF PobjVersionNotes THEN DO:
        IF Brtb_ver.upd-notes <> ? AND Brtb_ver.upd-notes <> "" THEN DO:
          PUT STREAM Sout "------------- Version Note -------------" SKIP.
          
          Mline = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_ver.upd-notes,60).
          DO WHILE Mline <> ?:
            PUT STREAM Sout UNFORMATTED Mline SKIP.
            Mline = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
          END.  /* do while return-value = "" */  
        END.  /* if  brtb_ver.upd-notes <> ? and .. */
      END.  /* if PobjVersionNotes */

      IF PobjDetails OR PobjVersionNotes THEN PUT SKIP(1).

      /* Find next object to process */
      IF PlatestVerOnly THEN
        FIND NEXT Brtb_hist
          WHERE Brtb_hist.wspace-id     =  Pwspace-id
            AND Brtb_hist.object        <> Mobject
            AND Brtb_hist.event-counter <= PendEvent
            AND Brtb_hist.event-counter >= PbegEvent
            USE-INDEX rtb_hist02
            NO-LOCK NO-ERROR.
      ELSE
        FIND NEXT Brtb_hist
          WHERE Brtb_hist.wspace-id     = Pwspace-id
            AND Brtb_hist.event-counter <= PendEvent
            AND Brtb_hist.event-counter >= PbegEvent
            USE-INDEX rtb_hist02
            NO-LOCK NO-ERROR.

    END. /* while available brtb_hist */

    {rtb/i/rtb_rptftr.i &STREAM=Sout}

  END. /* $RPT-BLOCK */

  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_differences_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_differences_report Procedure 
PROCEDURE rtb_differences_report :
/*------------------------------------------------------------------------------
  Purpose:     Print workspace differences report
  Parameters:  INPUT
                 Pwspace-id-1      = First workspace ID
                 Pwspace-id-2      = Second workspace ID
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id-1 LIKE rtb.rtb_wspace.wspace-id  NO-UNDO.
  DEFINE INPUT PARAMETER Pwspace-id-2 LIKE rtb.rtb_wspace.wspace-id  NO-UNDO.

  DEFINE VARIABLE Merror  AS CHARACTER   NO-UNDO INITIAL "".
  DEFINE VARIABLE Mheader AS CHARACTER   NO-UNDO FORMAT "x(76)".

  DEFINE BUFFER Brtb_object  FOR rtb.rtb_object.
  DEFINE BUFFER B2rtb_object FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_ver     FOR rtb.rtb_ver.
  DEFINE BUFFER B2rtb_ver    FOR rtb.rtb_ver.
  DEFINE BUFFER Brtb_wspace  FOR rtb.rtb_wspace.


  $RPT-BLOCK:
  DO:
    /*
     Validate params
    */
    IF NOT CAN-FIND(FIRST Brtb_wspace WHERE Brtb_wspace.wspace-id = Pwspace-id-1) THEN DO:
      Merror = SUBSTITUTE("Cannot find Workspace '&1'.",Pwspace-id-1).
      LEAVE $RPT-BLOCK.
    END.
    IF NOT CAN-FIND(FIRST Brtb_wspace WHERE Brtb_wspace.wspace-id = Pwspace-id-2) THEN DO:
      Merror = SUBSTITUTE("Cannot find Workspace '&1'.",Pwspace-id-2).
      LEAVE $RPT-BLOCK.
    END.
    IF Pwspace-id-1 = Pwspace-id-2 THEN DO:
      Merror = "Workspace IDs must be different.".
      LEAVE $RPT-BLOCK.
    END.

    /*
     Objects in Workspace 1 but not in Workspace 2...
    */
    ASSIGN Pwspace-id-1 = TRIM(Pwspace-id-1)
           Pwspace-id-2 = TRIM(Pwspace-id-2)
           Mheader    = "Objects in " + QUOTER(TRIM(Pwspace-id-1))
                           + " but not in " + QUOTER(TRIM(Pwspace-id-2)).

    {rtb/i/rtb_rpthdr.i 
        &title="""Workspace Differences Report"""
        &criteria="Mheader"
        &center-criteria=NO
        &STREAM=Sout}


    FOR EACH Brtb_object
      WHERE Brtb_object.wspace-id = Pwspace-id-1
      NO-LOCK,
      EACH Brtb_ver OF Brtb_object
      NO-LOCK:

      FIND FIRST B2rtb_object
        WHERE B2rtb_object.wspace-id   = Pwspace-id-2
        AND   B2rtb_object.obj-type    = Brtb_object.obj-type
        AND   B2rtb_object.object      = Brtb_object.object
        AND   B2rtb_object.pmod        = Brtb_object.pmod
        NO-LOCK NO-ERROR.

      IF NOT AVAILABLE B2rtb_object THEN DO:
        DISPLAY STREAM Sout
          Brtb_object.object   FORMAT "x(40)"
          Brtb_object.obj-type FORMAT "x(6)"
          Brtb_object.pmod     FORMAT "x(50)"
          Brtb_object.version  FORMAT "999999"
          Brtb_ver.obj-status
          WITH FRAME workspace-1-frame DOWN
          NO-ATTR-SPACE NO-BOX STREAM-IO WIDTH 120.
      END.
    END.

    /*
     Objects in Workspace 2 but not in Workspace 1...
    */
    Mheader = "Objects in " + QUOTER(TRIM(Pwspace-id-2))
                 + " but not in " + QUOTER(TRIM(Pwspace-id-1)).

    DISPLAY STREAM Sout "" WITH FRAME force-frame STREAM-IO. 
    PAGE STREAM Sout.

          
    FOR EACH Brtb_object
      WHERE Brtb_object.wspace-id = Pwspace-id-2
      NO-LOCK,
      EACH Brtb_ver OF Brtb_object
      NO-LOCK:

      FIND FIRST B2rtb_object
        WHERE B2rtb_object.wspace-id   = Pwspace-id-1
        AND   B2rtb_object.obj-type    = Brtb_object.obj-type
        AND   B2rtb_object.object      = Brtb_object.object
        AND   B2rtb_object.pmod        = Brtb_object.pmod
        USE-INDEX rtb_object4 NO-LOCK NO-ERROR.

      IF NOT AVAILABLE B2rtb_object THEN DO:
        DISPLAY STREAM Sout
          Brtb_object.object   FORMAT "x(30)"
          Brtb_object.obj-type FORMAT "x(6)"
          Brtb_object.pmod     FORMAT "x(50)"
          Brtb_object.version  FORMAT "999999"
          Brtb_ver.obj-status
          WITH FRAME workspace-2-frame DOWN
          NO-ATTR-SPACE NO-BOX STREAM-IO WIDTH 120.
      END.  /* if not avaiable B2rtb_object */
    END.  /* for each Brtb_object */

    /*
     Objects with different versions...
    */
    Mheader = "Objects with different versions".

    DISPLAY STREAM Sout "" WITH FRAME force-frame STREAM-IO.
    PAGE STREAM Sout.
              
    FOR EACH Brtb_object
      WHERE Brtb_object.wspace-id = Pwspace-id-1
      NO-LOCK,
      EACH Brtb_ver OF Brtb_object
      NO-LOCK:

      FIND FIRST B2rtb_object
        WHERE B2rtb_object.wspace-id   = Pwspace-id-2
        AND   B2rtb_object.obj-type    = Brtb_object.obj-type
        AND   B2rtb_object.object      = Brtb_object.object
        AND   B2rtb_object.pmod        = Brtb_object.pmod
        USE-INDEX rtb_object4 NO-LOCK NO-ERROR.

      FIND B2rtb_ver OF B2rtb_object NO-LOCK NO-ERROR.

      IF AVAILABLE B2rtb_object THEN DO:
        IF Brtb_object.version <> B2rtb_object.version THEN DO:
          DISPLAY STREAM Sout
            Brtb_object.object   FORMAT "x(30)"
            Brtb_object.obj-type FORMAT "x(6)"
            Pwspace-id-1         
            Brtb_object.version  FORMAT "999999"
            Brtb_ver.obj-status
            Pwspace-id-2         
            B2rtb_object.version FORMAT "999999"
            B2rtb_ver.obj-status
            WITH FRAME version-frame DOWN
            NO-ATTR-SPACE NO-BOX STREAM-IO WIDTH 120.
        END.  /* if Brtb_object.vers <> */     
      END.  /* if available Brtb_object */
    END.  /* for each Brtb_object */

    {rtb/i/rtb_rptftr.i &STREAM=Sout}

  END. /* $RPT-BLOCK */

  RETURN Merror.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_event_history_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_event_history_report Procedure 
PROCEDURE rtb_event_history_report :
/*------------------------------------------------------------------------------
  Purpose:     Print workspace event history report
  Parameters:  INPUT
                 Pwspace-id = Workspace ID
                 Puser-id   = User ID or "All"
                 PbegDate   = Start date or ? (for all dates)
                 PendDate   = End date or ? (for all date)
                 PbegEvent  = Starting event or 0 (for all events)
                 PendEvent  = Ending event or 0 (for all events) 
                 Ppmod      = Product module or MATCHES pattern
                 Pobj-type  = Object type or MATCHES pattern
                 Pobject    = Object name or MATCHES pattern
               OUTPUT
                 <none>

  Notes:       Formerly rtb_0404.w
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Puser-id   AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PbegDate   AS DATE        NO-UNDO.
  DEFINE INPUT PARAMETER PendDate   AS DATE        NO-UNDO.
  DEFINE INPUT PARAMETER PbegEvent  AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PendEvent  AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER Ppmod      AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pobj-type  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pobject    AS CHARACTER   NO-UNDO.

  DEFINE VARIABLE Mdescription AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mtextlines   AS CHARACTER   NO-UNDO.

  DEFINE BUFFER Brtb_hist    FOR rtb.rtb_hist.
  DEFINE BUFFER Brtb_wscount FOR rtb.rtb_wscount.
  DEFINE BUFFER Brtb_wspace  FOR rtb.rtb_wspace.

  $RPT-BLOCK:
  DO ON ERROR UNDO, LEAVE:
    IF Pwspace-id = "" THEN LEAVE $RPT-BLOCK.

    FIND Brtb_wspace WHERE Brtb_wspace.wspace-id = Pwspace-id NO-LOCK NO-ERROR.

    IF NOT AVAILABLE Brtb_wspace THEN LEAVE $RPT-BLOCK.

    FIND Brtb_wscount OF Brtb_wspace NO-LOCK NO-ERROR.

    IF NOT AVAILABLE Brtb_wscount THEN LEAVE $RPT-BLOCK.

    /*
     Set criteria defaults
    */
    FIND FIRST Brtb_hist WHERE Brtb_hist.wspace-id = Pwspace-id
      USE-INDEX rtb_hist03 NO-LOCK NO-ERROR. /*earliest event */

    ASSIGN Puser-id  = IF Puser-id EQ "" THEN
                         "All":U
                       ELSE
                         Puser-id
           PbegDate  = IF PbegDate EQ ? THEN
                         IF AVAILABLE Brtb_hist THEN
                           Brtb_hist.change-date
                         ELSE
                           TODAY
                       ELSE
                         PbegDate
           PendDate  = IF PendDate EQ ? THEN
                         TODAY
                       ELSE
                          PendDate
           PbegEvent = IF PbegEvent EQ 0 THEN
                         1
                       ELSE
                         PbegEvent
           PendEvent = IF PendEvent EQ 0 THEN
                         99999999
                       ELSE
                         PendEvent
           
           Ppmod     = IF Ppmod EQ "" THEN
                         "*":U
                       ELSE
                         Ppmod
           Pobj-type = IF Ppmod EQ "" THEN
                         "*":U
                       ELSE
                         Pobj-type
           PObject   = IF Pobject  EQ "" THEN
                         "*":U
                       ELSE
                         Pobject.
                      
    FOR EACH Brtb_hist 
      WHERE (IF Puser-id = "All" THEN
               TRUE
             ELSE
               Brtb_hist.user-id = Puser-id)
        AND Brtb_hist.change-date   >= PbegDate
        AND Brtb_hist.change-date   <= PendDate
        AND Brtb_hist.wspace-id     =  Pwspace-id
        AND Brtb_hist.event-counter >= PbegEvent
        AND Brtb_hist.event-counter <= PendEvent
        AND (IF Ppmod = "*" THEN
               TRUE
             ELSE
               Brtb_hist.pmod MATCHES Ppmod)
        AND (IF Pobj-type = "*" THEN
               TRUE
             ELSE
               Brtb_hist.obj-type MATCHES Pobj-type)
        AND (IF Pobject = "*" THEN
               TRUE
             ELSE
               Brtb_hist.object MATCHES Pobject)
        NO-LOCK
        BY Brtb_hist.event-counter DESCENDING
        ON ENDKEY UNDO, LEAVE
        ON ERROR UNDO, LEAVE 
        WITH FRAME hist-frame
          STREAM-IO:
          
      /*
       Define report header
      */
      {rtb/i/rtb_rpthdr.i 
          &title="""Workspace Event History Report"""
          &criteria=""" Workspace:""  Brtb_wspace.wspace-id   SKIP
                     ""      Desc:""  Brtb_wspace.description SKIP
                     "" From Date:""  PbegDate
                     ""    To Date:"" PendDate                SKIP
                     ""From Event:""  PbegEvent 
                     ""  To Event:""  PendEvent               SKIP
                     ""      User:""  Puser-id                SKIP(1)"
          &center-criteria=NO
          &STREAM=Sout}

      DISPLAY STREAM Sout
        IF Brtb_hist.event-counter = 99999999 THEN
          "     WIP" 
        ELSE
          STRING(Brtb_hist.event-counter,">>>>>>>9") LABEL "   Event#"  
        Brtb_hist.change-date
        Brtb_hist.action
        Brtb_hist.user-id
        Brtb_hist.object  FORMAT "x(30)"
        Brtb_hist.version
        Brtb_hist.pmod    FORMAT "x(30)" /* not enough space for 50 chars */
        Brtb_hist.obj-type
        WITH FRAME hist-frame
        WIDTH 120
        NO-BOX
        NO-ATTR-SPACE
        STREAM-IO.

    END. /* for each brtb_hist */

    {rtb/i/rtb_rptftr.i &STREAM=Sout}

  END. /* $RPT-BLOCK */

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_import_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_import_report Procedure 
PROCEDURE rtb_import_report :
/*------------------------------------------------------------------------------
  Purpose:     Print Import Report
  Parameters:  INPUT Pwspace-id  - Workspace ID
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.

  DEFINE BUFFER Brtb_import  FOR rtb.rtb_import.
  DEFINE BUFFER Brtb_object  FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_pmod    FOR rtb.rtb_pmod.
  DEFINE BUFFER Brtb_ver     FOR rtb.rtb_ver.
  DEFINE BUFFER Brtb_wssrc   FOR rtb.rtb_wssrc.

  DEFINE VARIABLE Mprimary AS LOGICAL NO-UNDO.

  FOR EACH Brtb_import NO-LOCK
    WHERE Brtb_import.wspace-id = Pwspace-id
    WITH FRAME details-frame:

    FORM
      "  Object: " Brtb_import.object     AT 12 FORMAT "x(100)" SKIP
      "    Type: " Brtb_import.obj-type   AT 12                 SKIP
      "  Status: " Brtb_import.imp-status AT 12                 SKIP
      "  Action: " Brtb_import.imp-action AT 12                 SKIP
      "    Done: " Brtb_import.done       AT 12 FORMAT "YES/NO" SKIP(1)
    
      "  Pmodule: " Brtb_object.pmod      AT 12  FORMAT "X(50)" Brtb_import.pmod      AT 65  FORMAT "X(50)" SKIP
      "  Version: " Brtb_object.version   AT 12                 Brtb_import.version   AT 65                 SKIP
      "   Module: " Brtb_object.module    AT 12  FORMAT "X(50)" Brtb_import.module    AT 65  FORMAT "X(50)" SKIP
      "    Group: " Brtb_object.obj-group AT 12                 Brtb_import.obj-group AT 65                 SKIP
    
      "Workspace:" AT 54 Brtb_import.src-wspace-id AT 65 SKIP 
      " Task-num:" AT 54 Brtb_import.task-num      AT 65 SKIP
      "  Primary: " Mprimary                       AT 12 FORMAT "YES/NO" SKIP
      "   Status: " Brtb_ver.obj-status            AT 12     
      "   Orphan:" AT 54 Brtb_import.orphan        AT 65 FORMAT "YES/NO" SKIP
    
      WITH FRAME details-frame
      NO-LABELS
      STREAM-IO
      WIDTH 120.
  
    {rtb/i/rtb_rpthdr.i 
      &title="""Workspace Import Report"""
      &criteria="""Workspace:"" Pwspace-id"
      &center-criteria=NO
      &STREAM=Sout}
      
     FORM HEADER 
       "Current" AT 12 "Import" AT 65 SKIP
       "-------" AT 12 "------" AT 65 SKIP
       WITH FRAME title-frame NO-BOX PAGE-TOP STREAM-IO.

     VIEW STREAM Sout FRAME title-frame.
     
     FIND FIRST Brtb_object NO-LOCK
       WHERE Brtb_object.wspace-id = Pwspace-id
         AND Brtb_object.obj-type  = Brtb_import.obj-type
         AND Brtb_object.object    = Brtb_import.object
         AND Brtb_object.pmod      = Brtb_import.pmod NO-ERROR.

     IF AVAILABLE Brtb_object THEN DO:
       
       FIND FIRST Brtb_pmod NO-LOCK
         WHERE Brtb_pmod.pmod = Brtb_object.pmod NO-ERROR.
        
       Mprimary = CAN-FIND(FIRST Brtb_wssrc 
                             WHERE Brtb_wssrc.wspace-id     = Pwspace-id
                               AND Brtb_wssrc.src-wspace-id = Pwspace-id
                               AND Brtb_wssrc.product-id    = Brtb_pmod.product-id).
       
       FIND Brtb_ver NO-LOCK
         WHERE Brtb_ver.pmod     = Brtb_object.pmod
           AND Brtb_ver.obj-type = Brtb_object.obj-type
           AND Brtb_ver.object   = Brtb_object.object
           AND Brtb_ver.version  = Brtb_object.version NO-ERROR.
     
       DISPLAY STREAM Sout
         Brtb_object.pmod       
         Brtb_object.version 
         Brtb_object.module 
         Brtb_object.obj-group
         Brtb_ver.obj-status
         Mprimary.

     END.

     DISPLAY STREAM Sout
       Brtb_import.obj-type
       Brtb_import.object
       Brtb_import.imp-status
       Brtb_import.src-wspace-id
       Brtb_import.task-num
       Brtb_import.module
       Brtb_import.obj-group
       Brtb_import.pmod
       Brtb_import.version
       Brtb_import.imp-action
       Brtb_import.done
       Brtb_import.orphan.

     PUT STREAM Sout UNFORMATTED FILL("-",120) SKIP.

  END. /* of EACH Brtb_import */

  {rtb/i/rtb_rptftr.i &STREAM=Sout}

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_release_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_release_report Procedure 
PROCEDURE rtb_release_report :
/*------------------------------------------------------------------------------
  Purpose:     Print a workspace release report
  Parameters:  INPUT
                 Pwspace-id        = Workspace ID
                 PbegRelease       = Beginning release #
                 PendRelease       = Ending release #
                 PpageHeaders      = TRUE to print page headers
                 PreleaseInfo      = TRUE to print release info
                 PobjectHeaders    = TRUE to print object headers
                 PobjDetails       = TRUE to print object details
                 PtaskSummary      = TRUE to print task summaries
                 PtaskDetail       = TRUE to print task details
                 PshowVariants     = TRUE to print object variants
                 PsuppressAssigns  = FALSE to print object assignments
                 PlatestVerOnly    = TRUE to print latest object version only
                 PversionInfo      = TRUE to print version info
                 PobjReleaseInfo   = TRUE to print object release info
                 PobjTaskInfo      = TRUE to print object task info
                 PobjVersionNotes  = TRUE to print object version notes
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id       AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PbegRelease      AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PendRelease      AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PpageHeaders     AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PreleaseInfo     AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjectHeaders   AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjDetails      AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PtaskSummary     AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PtaskDetail      AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PshowVariants    AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PsuppressAssigns AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PlatestVerOnly   AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PversionInfo     AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjReleaseInfo  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjTaskInfo     AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PobjVersionNotes AS LOGICAL     NO-UNDO.

  DEFINE VARIABLE Mdescription AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Merror       AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mline        AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MlineOut     AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mheading     AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mtextlines   AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE MbegEvent    AS INTEGER     NO-UNDO.
  DEFINE VARIABLE MendEvent    AS INTEGER     NO-UNDO.
  DEFINE VARIABLE Mi           AS INTEGER     NO-UNDO.
  DEFINE VARIABLE Mrelease-num AS INTEGER     NO-UNDO.
  DEFINE VARIABLE MfirstVar    AS LOGICAL     NO-UNDO.


  DEFINE BUFFER Brtb_hist     FOR rtb.rtb_hist.
  DEFINE BUFFER B2rtb_hist    FOR rtb.rtb_hist.
  DEFINE BUFFER Brtb_object   FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_release  FOR rtb.rtb_release.
  DEFINE BUFFER Brtb_ver      FOR rtb.rtb_ver.
  DEFINE BUFFER Brtb_wscount  FOR rtb.rtb_wscount.
  DEFINE BUFFER Brtb_wspace   FOR rtb.rtb_wspace.
  DEFINE BUFFER B2rtb_wspace FOR rtb.rtb_wspace.

  FORM
    SKIP(1)
    Mheading FORMAT "x(40)"
    SKIP(1)
    WITH FRAME f-heading NO-BOX NO-LABELS STREAM-IO.

  $RPT-BLOCK:
  DO:
    /*
     Validate input parameters
    */
    FIND Brtb_wspace WHERE Brtb_wspace.wspace-id = Pwspace-id NO-LOCK NO-ERROR.

    IF NOT AVAILABLE Brtb_wspace THEN DO:
      Merror = SUBSTITUTE("Cannot generate report as workspace '&1' record is not available.",
                          Pwspace-id).
      LEAVE $RPT-BLOCK.
    END.

    FIND Brtb_wscount OF Brtb_wspace NO-LOCK NO-ERROR.

    IF NOT AVAILABLE Brtb_wscount THEN DO:
      Merror = "Cannot generate report as workspace count record is not available.".
      LEAVE $RPT-BLOCK.
    END.  /* if Pwspace-id = "" */

    IF PendRelease <> 0 THEN DO:
      FIND Brtb_release WHERE Brtb_release.wspace-id   = Pwspace-id
                          AND Brtb_release.release-num = PendRelease
                          NO-LOCK NO-ERROR.
      IF AVAILABLE Brtb_release THEN
        ASSIGN MendEvent = Brtb_release.event-counter.
      ELSE DO:
        Merror = SUBSTITUTE("Release record for release &1 not available.",PendRelease).
        LEAVE $RPT-BLOCK.
      END.
    END.  /* if PendRelease <> 0 */
    ELSE
      ASSIGN MendEvent = Brtb_wscount.event-counter.

    IF PbegRelease <> 0 THEN DO:
      FIND Brtb_release WHERE Brtb_release.wspace-id   = Pwspace-id
                          AND Brtb_release.release-num = PbegRelease
                          NO-LOCK NO-ERROR.
      IF AVAILABLE Brtb_release THEN
        ASSIGN MbegEvent = Brtb_release.event-counter.
      ELSE DO:
        Merror = SUBSTITUTE("Release record for release &1 not available.",PendRelease).
        LEAVE $RPT-BLOCK.
      END.
    END.  /* if PbegRelease <> 0 */
    ELSE                             
      ASSIGN MbegEvent = 0.

    /*
     View page header
    */
    IF PpageHeaders THEN DO:
      {rtb/i/rtb_rpthdr.i 
        &title="""Release Report"""
        &criteria="""Workspace:"" Brtb_wspace.wspace-id   SKIP
                   ""     Desc:"" Brtb_wspace.description SKIP
                   ""From Rel#:"" TRIM(STRING(PbegRelease)) FORMAT ""x(4)""
                   "" Event#:""   TRIM(STRING(MbegEvent)) 
                   "" To Rel#:""  TRIM(STRING(PendRelease)) FORMAT ""x(4)""
                   "" Event#:""   TRIM(String(MendEvent))
                   SKIP(1)"
        &center-criteria=NO
        &STREAM=Sout}
    END.

    /*
     View release information at top of report.
    */
    IF PreleaseInfo THEN DO:
      FOR EACH Brtb_release WHERE Brtb_release.wspace-id   = Pwspace-id
                              AND Brtb_release.release-num >= PbegRelease
                              AND Brtb_release.release-num <= PendRelease
                              NO-LOCK:
        DISPLAY STREAM Sout
          Brtb_release.release-num    COLUMN-LABEL "Rel#"
          Brtb_release.event-counter  FORMAT ">>>>>>9"
          Brtb_release.date-stamp     COLUMN-LABEL "Date"
          Brtb_release.user-stamp
          Brtb_release.user-note      FORMAT "x(39)"
          WITH FRAME releases-frame NO-BOX DOWN STREAM-IO.
      END.  /* for each brtb_release */
      PUT STREAM Sout SKIP(1).
    END.  /* if PreleaseInfo */

    /*
     Create temp-tables for applicable objects and tasks
    */
    temp-table-block:
    FOR EACH Brtb_hist WHERE Brtb_hist.wspace-id      = Pwspace-id
                         AND Brtb_hist.event-counter  > MbegEvent
                         AND Brtb_hist.event-counter <= MendEvent
                         NO-LOCK:
      /*
       If specified, ignore assignments
      */
      IF PsuppressAssigns AND Brtb_hist.action = "Assign":U THEN
        NEXT.

      /*
       If specified, only consider latest version
      */
      IF PlatestVerOnly THEN DO:
        IF CAN-FIND(FIRST B2rtb_hist
                    WHERE B2rtb_hist.wspace-id = Pwspace-id
                      AND B2rtb_hist.obj-type  = Brtb_hist.obj-type
                      AND B2rtb_hist.object    = Brtb_hist.object
                      AND B2rtb_hist.event-counter > MbegEvent
                      AND B2rtb_hist.event-counter > Brtb_hist.event-counter) THEN
          NEXT temp-table-block.
      END.

      /*
       Don't consider assignments if object is unchanged from beginning
       release to the ending release - e.g. deleting a WIP object and
       reverting to the previous version should be seen as no change to
       the object between releases
      */
      IF Brtb_hist.action = "Assign":U THEN DO:
        FIND FIRST B2rtb_hist
          WHERE B2rtb_hist.wspace-id      = Pwspace-id
            AND B2rtb_hist.obj-type       = Brtb_hist.obj-type
            AND B2rtb_hist.object         = Brtb_hist.object
            AND B2rtb_hist.event-counter <= MbegEvent NO-LOCK NO-ERROR.
        IF AVAILABLE B2rtb_hist
           AND B2rtb_hist.pmod    = Brtb_hist.pmod
           AND B2rtb_hist.version = Brtb_hist.version
           AND CAN-DO("Assign,Add":U,B2rtb_hist.action) THEN
          NEXT temp-table-block.
      END.

      /*
       Get version record
      */
      FIND Brtb_ver WHERE Brtb_ver.pmod     = Brtb_hist.pmod
                      AND Brtb_ver.obj-type = Brtb_hist.obj-type
                      AND Brtb_ver.object   = Brtb_hist.object
                      AND Brtb_ver.version  = Brtb_hist.version
                      NO-LOCK NO-ERROR.
      IF NOT AVAILABLE Brtb_ver THEN
        NEXT temp-table-block.
      IF NOT CAN-FIND(FIRST Ttask
                      WHERE Ttask.task-num = Brtb_ver.task-num) THEN DO:
        CREATE Ttask.
        ASSIGN Ttask.task-num = Brtb_ver.task-num.
      END.  /* else do */

      /*
       If selected, get associated release number
      */
      IF PobjReleaseInfo THEN DO:
        FIND FIRST Brtb_release 
          WHERE Brtb_release.wspace-id      = Pwspace-id
            AND Brtb_release.event-counter >= Brtb_hist.event-counter
            USE-INDEX rtb_release02
            NO-LOCK NO-ERROR.
        IF AVAILABLE Brtb_release THEN
          Mrelease-num = Brtb_release.release-num.
      END.

      CREATE Thist.
      ASSIGN Thist.action        = Brtb_hist.action
             Thist.obj-type      = Brtb_hist.obj-type
             Thist.object        = Brtb_hist.object
             Thist.object-guid   = Brtb_hist.object-guid
             Thist.version       = Brtb_hist.version
             Thist.pmod          = Brtb_hist.pmod
             Thist.event-counter = Brtb_hist.event-counter
             Thist.sub-type      = Brtb_ver.sub-type
             Thist.task-num      = Brtb_ver.task-num
             Thist.upd-notes     = Brtb_ver.upd-notes
             Thist.release-num   = Mrelease-num.

    END. /* temp-table-block */

    /*
     View task information at top of report
    */
    IF PtaskSummary OR PtaskDetail THEN
      RUN print_task_info (INPUT PtaskDetail).

    objects-block:
    FOR EACH Thist
      BREAK BY Thist.action BY Thist.obj-type BY Thist.object:

      IF FIRST-OF(Thist.action) THEN DO:
        CASE Thist.action:
          WHEN "Add":U THEN
            Mheading = "OBJECTS ADDED":U.
          WHEN "Assign":U THEN
            Mheading = "OBJECTS ASSIGNED":U.
          WHEN "Delete":U THEN
            Mheading = "OBJECTS DELETED":U.
        END CASE.
        PUT STREAM Sout UNFORMATTED
          SKIP(1) Mheading.
      END.

      /*
       Header for each type of object
      */
      IF PobjectHeaders AND FIRST-OF(Thist.obj-type) THEN DO:
        Mheading = TRIM(CAPS(Thist.obj-type)) + " Objects".
        DISPLAY STREAM Sout Mheading WITH FRAME f-heading.
      END.  /* first-of(thist.obj-type) */

      /*
       Print object name and type only once (even if multiple
       versions
      */
      IF FIRST-OF(Thist.object) THEN
        DISPLAY STREAM Sout
          Thist.obj-type
          Thist.object
          Thist.pmod FORMAT 'x(50)'
          Thist.sub-type
          WITH FRAME obj-frame DOWN NO-BOX STREAM-IO WIDTH 120.

      IF PversionInfo
         OR PobjTaskInfo
         OR PobjReleaseInfo
         OR PobjDetails
         OR PobjVersionNotes THEN DO:
        IF PversionInfo THEN
          PUT STREAM Sout
            "Version: " AT 8
            SUBSTITUTE("&1.&2.&3",
                       SUBSTRING(STRING(Thist.version,"999999"),1,2),
                       SUBSTRING(STRING(Thist.version,"999999"),3,2),
                       SUBSTRING(STRING(Thist.version,"999999"),5,2)).

        IF PobjTaskInfo THEN
          PUT STREAM Sout "Task:" AT 27 Thist.task-num.

        IF PobjReleaseInfo THEN
          PUT STREAM Sout "Release:" AT 44 Thist.release-num.

        IF PobjDetails THEN
          PUT STREAM Sout "Event#: " AT 60 Thist.event-counter.

        IF PobjVersionNotes THEN DO:
          ASSIGN 
            MlineOut = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Thist.upd-notes,100)
            Mi       = 0.
          DO WHILE MlineOut <> ?:
            Mi = Mi + 1.
            IF Mi = 1 THEN
              PUT STREAM Sout UNFORMATTED
                "Note:"  AT 8
                MlineOut AT 14 SKIP.
            ELSE
              PUT STREAM Sout UNFORMATTED 
                MlineOut AT 14 SKIP.
            MlineOut = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,100).
          END.  /* do while mlineout <> "" */
          IF Mi > 0 THEN
            PUT STREAM Sout SKIP(1).
        END. /* if pobjversionnotes */
      END.

      /*
       Show object details
      */
      IF PobjDetails THEN DO:
        /*
         Get version information
        */
        RUN rtb_get_object_version_details (INPUT  Thist.pmod,
                                            INPUT  Thist.obj-type,
                                            INPUT  Thist.object,
                                            INPUT  Thist.version,
                                            OUTPUT Mdescription,
                                            OUTPUT Mtextlines).
        DO Mi = 2 TO 9:
          ASSIGN Mline = SUBSTRING(Mtextlines,(Mi - 1) * 76 + 1, 76).
          IF Mline = ? THEN
            LEAVE.
          IF TRIM(Mline) <> "" THEN DO:
            PUT STREAM Sout Mline AT 8 FORMAT "x(76)" SKIP.
          END.
        END.
      END.  /* if pobjdetails */

      /*
       Check object variants
      */
      IF PshowVariants AND LAST-OF(Thist.object) THEN DO:  
        MfirstVar = YES.
        FOR EACH B2rtb_wspace NO-LOCK:
          IF B2rtb_wspace.wspace-id <> Pwspace-id THEN NEXT.
          FOR EACH Brtb_object
            WHERE Brtb_object.object-guid = Thist.object-guid
              AND Brtb_object.pmod        <> Thist.pmod NO-LOCK:
                
            IF Brtb_object.pmod  <> Thist.pmod THEN DO:
              IF MfirstVar THEN DO:
                PUT STREAM Sout
                  SKIP(1)
                  "VARIANTS" AT 8
                  SKIP(1).
                MfirstVar = NO.
              END.
              DISPLAY STREAM Sout
                Brtb_object.wspace-id COLUMN-LABEL "Workspace"
                Brtb_object.pmod      COLUMN-LABEL "Pmodule" FORMAT 'x(50)'
                SUBSTITUTE("&1.&2.&3",
                           SUBSTRING(STRING(Brtb_object.version,"999999"),1,2),
                           SUBSTRING(STRING(Brtb_object.version,"999999"),3,2),
                           SUBSTRING(STRING(Brtb_object.version,"999999"),5,2))
                           COLUMN-LABEL "Version"
                WITH FRAME variant-frame DOWN NO-BOX COLUMN 8 STREAM-IO.
            END. /* if brtb_object.pmod... */
          END. /* for each brtb_object */
        END. /* for each brtb_wspace */
        IF NOT MfirstVar THEN
          PUT STREAM Sout SKIP(1).
      END. /* if PshowVariants... */
    END. /* objects-block */

    EMPTY TEMP-TABLE Thist.

    {rtb/i/rtb_rptftr.i &STREAM=Sout}

  END. /* $RPT-BLOCK */

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_site_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_site_report Procedure 
PROCEDURE rtb_site_report :
/*------------------------------------------------------------------------------
  Purpose:     Print deployment site report
  Parameters:  INPUT
                 Pwspace-id        = Workspace ID
                 Psite-code        = Deployment site code or * for all
                 PactiveOnly       = TRUE to show only active sites
                 PshowNotes        = TRUE to show site notes
                 PlastOnly         = TRUE to show last deployment only
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id   AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Psite-code   AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PactiveOnly  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PshowNotes   AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PlastDepOnly AS LOGICAL     NO-UNDO.

  DEFINE VARIABLE Mlicense-type AS CHARACTER   NO-UNDO FORMAT "x(12)"
                                                       LABEL "Type".
  DEFINE VARIABLE Mstatus       AS CHARACTER   NO-UNDO LABEL "Status".
  DEFINE VARIABLE Mcurrent-rel  AS INTEGER     NO-UNDO.
  DEFINE VARIABLE Mi            AS INTEGER     NO-UNDO.

  DEFINE BUFFER Brtb_deploy  FOR rtb.rtb_deploy.
  DEFINE BUFFER Brtb_release FOR rtb.rtb_release.
  DEFINE BUFFER Brtb_site    FOR rtb.rtb_site.

  /*
   Get current release for workspace
  */
  FIND FIRST Brtb_release 
    WHERE Brtb_release.wspace-id = Pwspace-id NO-LOCK NO-ERROR.

  Mcurrent-rel = IF AVAILABLE Brtb_release THEN
                   Brtb_release.release-num
                 ELSE
                   0.

  /*
   For each site meeting criteria...
  */
  FOR EACH Brtb_site
    WHERE Brtb_site.wspace-id = Pwspace-id 
      AND (IF Psite-code = "*" THEN
             YES
           ELSE
             Brtb_site.site-code = Psite-code)
      AND (IF PactiveOnly THEN
             Brtb_site.site-status = "A":U
           ELSE
             YES) NO-LOCK:

    {rtb/i/rtb_rpthdr.i 
            &title="""Workspace Deployment Sites Report"""
            &criteria="""   Workspace:""       Pwspace-id   SKIP
                       ""        Site:""       Psite-code   SKIP
                       "" Active Only?""       PactiveOnly
                       ""Last Deployment Only?"" PlastDepOnly 
                       ""Show Notes?""           PshowNotes SKIP(1)"
            &center-criteria=NO
            &STREAM=Sout}

    ASSIGN Mstatus       = (IF Brtb_site.site-status = "A":U THEN
                              "Active"
                            ELSE
                              "Inactive")
           Mlicense-type = (IF Brtb_site.licence-type = "D" THEN
                              "Devlopment"
                            ELSE IF Brtb_site.licence-type = "P" THEN
                              "Partner"
                            ELSE IF Brtb_site.licence-type = "Q" THEN
                              "Query"
                            ELSE IF Brtb_site.licence-type = "R" THEN
                              "Runtime"
                            ELSE IF Brtb_site.licence-type = "S" THEN
                              "Source"
                            ELSE
                              "Unknown").
    /*
     Show site info
    */
    DISPLAY STREAM Sout
      SKIP(2)
      Brtb_site.site-code    COLON 12
      Mstatus                COLON 12
      Mlicense-type          COLON 12
      WITH FRAME site-frame NO-BOX SIDE-LABELS STREAM-IO.

    /*
     Show notes if requested
    */
    IF PshowNotes THEN DO Mi = 1 TO 10:
      IF Brtb_site.site-note[Mi] <> "" THEN
        DISPLAY STREAM Sout
          "Notes:" TO 12 WHEN Mi = 1
          Brtb_site.site-note[Mi] AT 14
          WITH NO-BOX NO-LABELS WIDTH 85 STREAM-IO.
    END.

    PUT STREAM Sout SKIP(1).

    /*
     Show deployments for this site
    */
    FOR EACH Brtb_deploy OF Brtb_site NO-LOCK
      BREAK BY Brtb_deploy.deploy-sequence:

      IF FIRST-OF(Brtb_deploy.deploy-sequence) THEN
        DISPLAY STREAM Sout
          SKIP(1)
          "Deployments:" AT 12
          SKIP(1)
          WITH NO-BOX NO-LABELS STREAM-IO.
          
      FIND Brtb_release
        WHERE Brtb_release.wspace-id   = Pwspace-id
          AND Brtb_release.release-num = Brtb_deploy.release-num
          NO-LOCK NO-ERROR.

      DISPLAY STREAM Sout
        Brtb_deploy.deploy-sequence
        Brtb_deploy.release-num
        Brtb_release.user-note
        (IF Brtb_deploy.release-num = Mcurrent-rel THEN
           "CURRENT"
         ELSE
           "NOT CURRENT") FORMAT "x(11)"
        WITH FRAME deploy-frame NO-BOX DOWN COLUMN 12 STREAM-IO.

      IF PlastDepOnly THEN LEAVE.
    END.
  END.

  {rtb/i/rtb_rptftr.i &STREAM=Sout}

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_versions_in_workspace_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_versions_in_workspace_report Procedure 
PROCEDURE rtb_versions_in_workspace_report :
/*------------------------------------------------------------------------------
  Purpose:     Print versions in workspace report
  Parameters:  INPUT
                 Pwspace-id     = Workspace ID
                 Pobj-type      = Object type
                 Pobject        = Object name
                 Pversion       = Object version (used by import reporting)
                 PverNotes      = TRUE to show version notes
                 Ppmodlist      = CSV list of product modules
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pobj-type  AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pobject    AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER Pversion   AS INTEGER     NO-UNDO.
  DEFINE INPUT PARAMETER PverNotes  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PpmodList  AS CHARACTER   NO-UNDO.

  DEFINE VARIABLE Mline       AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mver-order  AS INTEGER     NO-UNDO.
  DEFINE VARIABLE Mtver-rowid AS ROWID       NO-UNDO.

  DEFINE BUFFER Brtb_hist FOR rtb.rtb_hist.
  DEFINE BUFFER Brtb_ver  FOR rtb.rtb_ver.

  FORM
    Brtb_ver.version        COLUMN-LABEL "Version"
    Brtb_ver.prev-ver       COLUMN-LABEL "Prev Version"
    Brtb_ver.pmod           COLUMN-LABEL "Pmodule"       FORMAT "X(80)" SKIP
    WITH FRAME version-frame DOWN NO-BOX WIDTH 120 STREAM-IO.

  {rtb/i/rtb_rpthdr.i
    &title="""Versions in Workspace Report"""
    &criteria="""  Workspace: "" Pwspace-id                   SKIP
               ""     Object: "" Pobject    FORMAT ""X(100)"" SKIP
               ""       Type: "" Pobj-type                    SKIP(1)"
    &center-criteria=NO
    &STREAM=Sout}

  /*
   Scan the history records and create entries for the corresponding
   rtb_ver records in the temp table.

   Examine the history records in the order of the oldest records to the
   most recent.  If Pversion is passed, then this loop tries to find the
   hist record beginning with that version regardless of workspace.
  */

  FOR EACH Brtb_hist NO-LOCK
    WHERE Brtb_hist.wspace-id = Pwspace-id
      AND Brtb_hist.obj-type  = Pobj-type
      AND Brtb_hist.object    = Pobject
      AND (IF Pversion <> ? THEN
             Brtb_hist.version GE Pversion
           ELSE
             TRUE)
    BY Brtb_hist.event-counter:

    FIND Brtb_ver NO-LOCK
      WHERE Brtb_ver.obj-type = Pobj-type
        AND Brtb_ver.object   = Pobject
        AND Brtb_ver.pmod     = Brtb_hist.pmod
        AND Brtb_ver.version  = Brtb_hist.version.

    IF NOT CAN-FIND(Tver WHERE Tver.ver-rowid = ROWID(Brtb_ver)) THEN DO:
      CREATE Tver.
      ASSIGN
        Tver.ver-rowid    = ROWID(Brtb_ver)
        Tver.ver-order    = Mver-order
        Tver.hist-rowid   = ROWID(Brtb_hist)
        Tver.pmod         = Brtb_ver.pmod
        Tver.prev-version = Brtb_ver.prev-version
        Mver-order        = Mver-order - 1.
    END.  /* if not can-find(tver ... then */

  END.  /* for each Brtb_hist */

  /*
   Some rtb_ver records will not have a corresponding rtb_hist record.
   This means that the above rtb_hist logic will have failed to insert
   them into the temp table.  This can happen if, for example, the rtb_ver
   record was created by another workspace or at a partner site. The
   following logic will attempt to locate these records and ensure that
   entries are created for them in the temp table.                                                                 */

  FOR EACH Brtb_ver NO-LOCK
    WHERE Brtb_ver.obj-type = Pobj-type
      AND Brtb_ver.object   = Pobject
      AND CAN-DO(PpmodList, Brtb_ver.pmod)
      AND (IF Pversion <> ? THEN
             Brtb_hist.version GE Pversion
           ELSE
             TRUE):

    IF NOT CAN-FIND(Tver WHERE Tver.ver-rowid = ROWID(Brtb_ver)) THEN DO:
      
      /*
       If there is record in the temp table that is lacking its previous
       version, which matches the version of this record, then insert
       this record following it.
      */
      FIND FIRST Tver
        WHERE Tver.pmod         = Brtb_ver.pmod
          AND Tver.prev-version = Brtb_ver.VERSION NO-ERROR.

      IF AVAILABLE Tver THEN DO:
        
        /*
         Reorder the existing Tver records to allow the insertion of
         a new Tver record for this Brtb_ver.record.
        */
        ASSIGN Mver-order  = Tver.ver-order
               Mtver-rowid = ROWID(Tver).

        FIND LAST Tver.                                          
        
        DO WHILE (ROWID(Tver) <> Mtver-rowid):
          ASSIGN Tver.ver-order = Tver.ver-order + 1.          
          FIND PREV Tver.                                          
        END.  /* do while */

        CREATE Tver.
        ASSIGN Tver.ver-rowid    = ROWID(Brtb_ver)
               Tver.ver-order    = Mver-order + 1
               Tver.hist-rowid   = ROWID(Brtb_hist)
               Tver.pmod         = Brtb_ver.pmod
               Tver.prev-version = Brtb_ver.prev-version.

      END.  /* if available tver then */

    END.  /* if not can-find(tver ... then */

  END.  /* for each Brtb_ver */

  /*
   Display the contents of the temp table.
  */
  FOR EACH Tver:
    
    FIND Brtb_ver WHERE ROWID(Brtb_ver) = Tver.ver-rowid NO-LOCK.

    DISPLAY STREAM Sout
      Brtb_ver.version
      Brtb_ver.prev-ver
      Brtb_ver.pmod
      WITH FRAME version-frame.

    /*
     Only display Version Notes if selected by user.
    */
    IF PverNotes THEN DO:
      
      PUT STREAM Sout
        SKIP(1)
        "Note:" AT 9.

      FOR EACH Tver-notes:
        DELETE Tver-notes.
      END.

      Mline = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_ver.upd-notes,60).
      
      DO WHILE Mline <> ?:
        CREATE Tver-notes.
        ASSIGN Tver-notes.upd-notes = Mline.
        Mline = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
      END.  /* do while Mline = "" */  

      FIND LAST Tver-notes NO-ERROR.
      
      DO WHILE AVAILABLE Tver-notes 
        AND (Tver-notes.upd-notes = "" OR Tver-notes.upd-notes = ?):
        DELETE Tver-notes.
        FIND PREV Tver-notes NO-ERROR.
      END.  /* do while available Tver-notes... */

      FOR EACH Tver-notes:
        PUT STREAM Sout UNFORMATTED Tver-notes.upd-notes AT 18 SKIP.
      END.  /* for each Tver-notes */

      PUT STREAM Sout SKIP(1).

    END.  /* if PverNotes then */

    DOWN STREAM Sout WITH FRAME version-frame.

  END.  /* for each tver */  

  {rtb/i/rtb_rptftr.i &STREAM=Sout}

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

&IF DEFINED(EXCLUDE-rtb_workspace_report) = 0 &THEN

&ANALYZE-SUSPEND _UIB-CODE-BLOCK _PROCEDURE rtb_workspace_report Procedure 
PROCEDURE rtb_workspace_report :
/*------------------------------------------------------------------------------
  Purpose:     Print workspace report
  Parameters:  INPUT
                 Pwspace-id   = Workspace ID
                 PshowObjects = TRUE to show objects in workspace
                 PshowDetails = TRUE to show object details
                 PhowVerNotes = TRUE to show object version notes 
               OUTPUT
                 <none>
  Notes:       
------------------------------------------------------------------------------*/
  DEFINE INPUT PARAMETER Pwspace-id    AS CHARACTER   NO-UNDO.
  DEFINE INPUT PARAMETER PshowObjects  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PshowDetails  AS LOGICAL     NO-UNDO.
  DEFINE INPUT PARAMETER PshowVerNotes AS LOGICAL     NO-UNDO.

  DEFINE VARIABLE Mdescription AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mline        AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mtextlines   AS CHARACTER   NO-UNDO.
  DEFINE VARIABLE Mi           AS INTEGER     NO-UNDO.

  DEFINE BUFFER Brtb_moddef  FOR rtb.rtb_moddef.
  DEFINE BUFFER Brtb_module  FOR rtb.rtb_module.
  DEFINE BUFFER Brtb_object  FOR rtb.rtb_object.
  DEFINE BUFFER Brtb_wscount FOR rtb.rtb_wscount.
  DEFINE BUFFER Brtb_wspace  FOR rtb.rtb_wspace.
  DEFINE BUFFER Brtb_ver     FOR rtb.rtb_ver.

  $RPT-BLOCK:
  DO ON ERROR UNDO, LEAVE:

    IF Pwspace-id = "" THEN LEAVE $RPT-BLOCK.

    FIND Brtb_wspace WHERE Brtb_wspace.wspace-id = Pwspace-id NO-LOCK NO-ERROR.

    IF NOT AVAILABLE Brtb_wspace THEN LEAVE $RPT-BLOCK.

    FIND Brtb_wscount OF Brtb_wspace NO-LOCK NO-ERROR.

    IF NOT AVAILABLE Brtb_wscount THEN LEAVE $RPT-BLOCK.

    FOR EACH Brtb_module NO-LOCK
      WHERE Brtb_module.wspace-id = Pwspace-id,
        Brtb_moddef OF Brtb_module NO-LOCK
      BREAK BY Brtb_module.module
      ON ENDKEY UNDO, LEAVE
      ON ERROR UNDO, LEAVE 
      WITH FRAME module-frame STREAM-IO:

      {rtb/i/rtb_rpthdr.i 
         &title="""Workspace Report"""
         &criteria="""  Workspace:"" Brtb_wspace.wspace-id   SKIP
                    ""Description:"" Brtb_wspace.description SKIP"
         &center-criteria=NO
         &STREAM=Sout}

      IF FIRST(Brtb_module.module) THEN DO:

        PUT STREAM Sout "Workspace Path:" AT 2.
        
        DO Mi = 1 TO NUM-ENTRIES(Brtb_wspace.wspace-path):
          PUT STREAM Sout ENTRY(Mi, Brtb_wspace.wspace-path) AT 18 FORMAT "x(100)" SKIP.
        END.

        DISPLAY STREAM Sout
          Brtb_wscount.event-counter     COLON 16 LABEL "Event Count" FORMAT ">>>>>>>>9"
          Brtb_wspace.last-update-schema COLON 16 LABEL "Last Update" FORMAT ">>>>>>>>9" SKIP
          WITH FRAME frame-counts NO-BOX SIDE-LABELS STREAM-IO.

        IF Brtb_wspace.locked-by <> "" THEN
          PUT STREAM Sout UNFORMATTED "     Locked By:" AT 2 Brtb_wspace.locked-by AT 18.

      END.

      PUT STREAM Sout SKIP(1).

      DISPLAY STREAM Sout
        Brtb_module.module    FORMAT "x(100)"
        Brtb_moddef.description
        Brtb_moddef.naming-code
        Brtb_moddef.directory FORMAT "x(100)"
        WITH FRAME module-frame WIDTH 120 NO-BOX 1 COLUMNS STREAM-IO.

      IF PshowObjects THEN 
        FOR EACH Brtb_object NO-LOCK OF Brtb_module 
          WITH FRAME object-desc-frame:

          FIND Brtb_ver OF Brtb_object NO-LOCK.

          RUN rtb_get_object_version_details (Brtb_ver.pmod, Brtb_ver.obj-type, Brtb_ver.object, Brtb_ver.version, OUTPUT Mdescription, OUTPUT Mtextlines).
          
          DISPLAY STREAM Sout
            Brtb_object.obj-group FORMAT "x(20)"
            Brtb_object.obj-type  FORMAT "x(6)"
            (Brtb_object.object + " / " + Mdescription) FORMAT "x(90)" LABEL "Object / Description" SKIP
            WITH FRAME object-desc-frame WIDTH 120 STREAM-IO.

          IF PshowDetails AND Mtextlines <> "" THEN DO:
            DO Mi = 1 TO 9:
              Mline = SUBSTRING(Mtextlines,(Mi - 1) * 76 + 1, 76).
              IF TRIM(Mline) <> "" THEN
                PUT STREAM Sout "  " Mline FORMAT "x(76)" SKIP.
            END.  
            PUT STREAM Sout SKIP(1).
          END.

          IF PshowVerNotes THEN DO:
            
            IF Brtb_ver.upd-notes <> ? AND Brtb_ver.upd-notes <> "" THEN DO:
              PUT STREAM Sout "  Note:" SKIP.
              Mline = DYNAMIC-FUNCTION('fnRtbBreakLines':U,Brtb_ver.upd-notes,60).
              DO WHILE Mline <> ?:
                PUT STREAM Sout UNFORMATTED "    " Mline SKIP.
                Mline = DYNAMIC-FUNCTION('fnRtbBreakLines':U,?,60).
              END.
              PUT STREAM Sout SKIP(1).
            END.  /* if  brtb_ver.upd-notes <> ? and .. */

          END.  /* if pshowvernotes */

      END. /* for each Brtb_object */

    END. /* for each Brtb_module,.. */

    {rtb/i/rtb_rptftr.i &STREAM=Sout}

  END. /* $RPT-BLOCK */

  RETURN.

END PROCEDURE.

/* _UIB-CODE-BLOCK-END */
&ANALYZE-RESUME

&ENDIF

